@@ -2,8 +2,8 @@
 # This file is part of Xpra.
 # Copyright (C) 2011-2013 Antoine Martin <antoine@devloop.org.uk>
 
-from xpra.x11.gtk_x11 import gdk_display_source			 #@UnusedImport
-from xpra.x11.bindings.keyboard_bindings import X11KeyboardBindings		#@UnresolvedImport
+from xpra.x11.bindings import posix_display_source      #@UnusedImport
+from xpra.x11.bindings.keyboard_bindings import X11KeyboardBindings	    #@UnresolvedImport
 keyboard_bindings = X11KeyboardBindings()
 
 
