@@ -3,6 +3,37 @@
 # Copyright (C) 2012, 2013 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
+#
+# To create multiple output files which can be used to generate charts (using test_measure_perf_charts.py)
+# build a config class (copy from perf_config_default.py -- make changes as necessary).
+#
+# Then determine the values of the following variables:
+#   prefix: a string to identify the data set
+#   id: a string to identify the variable that the data set is testing (for example '14' because we're testing xpra v14 in this data set)
+#   repetitions: decide how many times you want to run the tests
+#
+# The data file names you will produce will then be in the format:
+#   prefix_id_rep#.csv
+#
+# With this information in hand you can now create a script that will run the tests, containing commands like:
+#   ./test_measure_perf.py as an example:
+
+# For example:
+#
+# ./test_measure_perf.py all_tests_40 ./data/all_tests_40_14_1.csv 1 14 > ./data//all_tests_40_14_1.log
+# ./test_measure_perf.py all_tests_40 ./data/all_tests_40_14_2.csv 2 14 > ./data//all_tests_40_14_2.log
+#
+# In this example script, I'm running test_measure_perf 2 times, using a config class named "all_tests_40.py",
+# and outputting the results to data files using the prefix "all_tests_40", for version 14.
+#
+# The additional arguments "1 14", "2 14" are custom paramaters which will be written to the "Custom Params" column
+# in the corresponding data files.
+#
+# Where you see "1", "2" in the file names or params, that's referring to the corresponding repetition of the tests.
+#
+# Once this script has run, you can open up test_measure_perf_charts.py and take a look at the
+# instructions there for generating the charts.
+#
 
 import re
 import sys
@@ -16,7 +47,7 @@ log = Logger()
 def getoutput(cmd, env=None):
     try:
         process = subprocess.Popen(cmd, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, env=env, close_fds=True)
-    except Exception, e:
+    except Exception as e:
         print("error running %s: %s" % (cmd, e))
         raise e
     (out,err) = process.communicate()
@@ -131,13 +162,13 @@ def try_to_stop(process, grace=0):
     if is_process_alive(process, grace):
         try:
             process.terminate()
-        except Exception, e:
+        except Exception as e:
             print("could not stop process %s: %s" % (process, e))
 def try_to_kill(process, grace=0):
     if is_process_alive(process, grace):
         try:
             process.kill()
-        except Exception, e:
+        except Exception as e:
             print("could not stop process %s: %s" % (process, e))
 
 def find_matching_lines(out, pattern):
@@ -199,7 +230,7 @@ def get_stderr(command):
         process = subprocess.Popen(command, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
         _,err = process.communicate()
         return err
-    except Exception, e:
+    except Exception as e:
         print("error running %s: %s" % (DETECT_XVNC_VERSION_CMD, e))
 
 err = get_stderr(DETECT_XVNC_VERSION_CMD)
@@ -252,21 +283,19 @@ def zero_iptables():
         #print("output(%s)=%s" % (cmd, out))
 
 def update_proc_stat():
-    proc_stat = open("/proc/stat", "rU")
-    time_total = 0
-    for line in proc_stat:
-        values = line.split()
-        if values[0]=="cpu":
-            time_total = sum([int(x) for x in values[1:]])
-            #print("time_total=%s" % time_total)
-            break
-    proc_stat.close()
+    with open("/proc/stat", "rU") as proc_stat:
+        time_total = 0
+        for line in proc_stat:
+            values = line.split()
+            if values[0]=="cpu":
+                time_total = sum([int(x) for x in values[1:]])
+                #print("time_total=%s" % time_total)
+                break
     return time_total
 
 def update_pidstat(pid):
-    stat_file = open("/proc/%s/stat" % pid, "rU")
-    data = stat_file.read()
-    stat_file.close()
+    with open("/proc/%s/stat" % pid, "rU") as stat_file:
+        data = stat_file.read()
     pid_stat = data.split()
     #print("update_pidstat(%s): %s" % (pid, pid_stat))
     return pid_stat
@@ -479,7 +508,7 @@ def with_server(start_server_command, stop_server_commands, in_tests, get_stats_
                             }
                     data.update(measure_client(server_pid, name, client_cmd, get_stats_cb))
                     results.append([data.get(x, "") for x in HEADERS])
-                except Exception, e:
+                except Exception as e:
                     import traceback
                     traceback.print_exc()
                     errors += 1
@@ -500,11 +529,11 @@ def with_server(start_server_command, stop_server_commands, in_tests, get_stats_
                         try:
                             stop_process = subprocess.Popen(s, stdin=None, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)
                             stop_process.wait()
-                        except Exception, e:
+                        except Exception as e:
                             print("error: %s" % e)
                     try_to_kill(server_process, 5)
                 time.sleep(1)
-        except KeyboardInterrupt, e:
+        except KeyboardInterrupt as e:
             print("caught %s: stopping this series of tests" % e)
             break
     return results
@@ -697,13 +726,13 @@ def test_xpra():
                             for speed in speed_options:
                                 for speaker in XPRA_SPEAKER_OPTIONS:
                                     for mic in XPRA_MICROPHONE_OPTIONS:
-                                        comp_options = []
-                                        if XPRA_VERSION_NO>=[0, 14]:
+                                        comp_options = [None]
+                                        if XPRA_VERSION_NO>=[0, 13]:
                                             comp_options = config.XPRA_COMPRESSORS_OPTIONS
                                         for comp in comp_options:
                                             comp_level_options = config.XPRA_COMPRESSION_LEVEL_OPTIONS
                                             for compression in comp_level_options:
-                                                packet_encoders_options = []
+                                                packet_encoders_options = [None]
                                                 if XPRA_VERSION_NO>=[0, 14]:
                                                     packet_encoders_options = config.XPRA_PACKET_ENCODERS_OPTIONS
                                                 for packet_encoders in packet_encoders_options:
@@ -837,7 +866,7 @@ def get_vnc_stats(initial_stats=None, all_stats=[]):
             print("tcbench starting: %s, logging to %s" % (command, config.TCBENCH_LOG))
             proc = subprocess.Popen(command, stdin=None, stdout=tcbench_log, stderr=tcbench_log)
             return {"tcbench" : proc}
-        except Exception, e:
+        except Exception as e:
             import traceback
             traceback.print_exc()
             print("error running %s: %s" % (command, e))
