@@ -19,6 +19,8 @@ FAKE_BACKING_DELAY = int(os.environ.get("XPRA_FAKE_BACKING_DELAY", "5"))
 
 class FakeBacking(object):
 
+    HAS_ALPHA = True
+
     def __init__(self, wid, *args):
         self.wid = wid
         self.fake_delay = FAKE_BACKING_DELAY
@@ -33,3 +35,8 @@ class FakeBacking(object):
 
     def cairo_draw(self, context, x, y):
         pass
+
+    def get_encoding_properties(self):
+        return {
+                "encodings.rgb_formats"    : ["RGBA", "RGB"],
+               }
