@@ -14,45 +14,52 @@ required_extensions = ["GL_ARB_texture_rectangle", "GL_ARB_vertex_program"]
 
 
 WHITELIST = {}
+GREYLIST = {}
 BLACKLIST = {"vendor" : ["nouveau", "Humper", "VMware, Inc."],
              "renderer" : ["Software Rasterizer"]}
+
 if False:
     #for testing:
     BLACKLIST["vendor"].append("NVIDIA Corporation")
     WHITELIST["renderer"] = ["GeForce GTX 760/PCIe/SSE2"]
-    
+
 if sys.platform.startswith("darwin") or sys.platform.startswith("win"):
-    #crashes were reported with the Intel driver on OSX 
-    BLACKLIST["vendor"].append("Intel Inc.")
+    #crashes were reported with the Intel driver on OSX and win32:
+    GREYLIST.setdefault("vendor", []).append("Intel Inc.")
+if sys.platform.startswith("darwin"):
+    #frequent crashes on osx with GT 650M: (see ticket #808)
+    GREYLIST.setdefault("vendor", []).append("NVIDIA Corporation")
 
 
-DEFAULT_ALPHA = not sys.platform.startswith("win") and not sys.platform.startswith("darwin")
+#alpha requires gtk3 or *nix only for gtk2:
+DEFAULT_ALPHA = sys.version>'3' or (not sys.platform.startswith("win") and not sys.platform.startswith("darwin"))
 GL_ALPHA_SUPPORTED = os.environ.get("XPRA_ALPHA", DEFAULT_ALPHA) in (True, "1")
-DEFAULT_DOUBLE_BUFFERED = 0
-if sys.platform.startswith("win"):
-    #needed on win32?
-    DEFAULT_DOUBLE_BUFFERED = 1
-DOUBLE_BUFFERED = os.environ.get("XPRA_OPENGL_DOUBLE_BUFFERED", str(DEFAULT_DOUBLE_BUFFERED))=="1"
+#not working with gtk3 yet?
+CAN_DOUBLE_BUFFER = True
+#needed on win32?:
+DEFAULT_DOUBLE_BUFFERED = sys.platform.startswith("win") and CAN_DOUBLE_BUFFER
+DOUBLE_BUFFERED = os.environ.get("XPRA_OPENGL_DOUBLE_BUFFERED", str(int(DEFAULT_DOUBLE_BUFFERED)))=="1"
+
 
 def get_visual_name(visual):
-    import gtk.gdk
+    from xpra.gtk_common.gtk_util import STATIC_GRAY, GRAYSCALE, STATIC_COLOR, PSEUDO_COLOR, TRUE_COLOR, DIRECT_COLOR
     if not visual:
         return ""
     return {
-           gtk.gdk.VISUAL_STATIC_GRAY   : "STATIC_GRAY",
-           gtk.gdk.VISUAL_GRAYSCALE     : "GRAYSCALE",
-           gtk.gdk.VISUAL_STATIC_COLOR  : "STATIC_COLOR",
-           gtk.gdk.VISUAL_PSEUDO_COLOR  : "PSEUDO_COLOR",
-           gtk.gdk.VISUAL_TRUE_COLOR    : "TRUE_COLOR",
-           gtk.gdk.VISUAL_DIRECT_COLOR  : "DIRECT_COLOR"}.get(visual.type, "unknown")
+           STATIC_GRAY      : "STATIC_GRAY",
+           GRAYSCALE        : "GRAYSCALE",
+           STATIC_COLOR     : "STATIC_COLOR",
+           PSEUDO_COLOR     : "PSEUDO_COLOR",
+           TRUE_COLOR       : "TRUE_COLOR",
+           DIRECT_COLOR     : "DIRECT_COLOR"}.get(visual.type, "unknown")
 
 def get_visual_byte_order(visual):
-    import gtk.gdk
+    from xpra.gtk_common.gtk_util import LSB_FIRST, MSB_FIRST
     if not visual:
         return ""
     return {
-            gtk.gdk.LSB_FIRST   : "LSB",
-            gtk.gdk.MSB_FIRST   : "MSB"}.get(visual.byte_order, "unknown")
+            LSB_FIRST   : "LSB",
+            MSB_FIRST   : "MSB"}.get(visual.byte_order, "unknown")
 
 def visual_to_str(visual):
     if not visual:
@@ -64,30 +71,29 @@ def visual_to_str(visual):
     return str(d)
 
 def get_DISPLAY_MODE(want_alpha=GL_ALPHA_SUPPORTED):
-    import gtk.gdkgl
-    #gtk.gdkgl.MODE_DEPTH
-    mode = 0
+    from xpra.client.gl.gtk_compat import MODE_RGBA, MODE_ALPHA, MODE_RGB, MODE_DOUBLE, MODE_SINGLE
+    #MODE_DEPTH
     if want_alpha:
-        mode = mode | gtk.gdkgl.MODE_RGBA | gtk.gdkgl.MODE_ALPHA
+        mode = MODE_RGBA | MODE_ALPHA
     else:
-        mode = mode | gtk.gdkgl.MODE_RGB
+        mode = MODE_RGB
     if DOUBLE_BUFFERED:
-        mode = mode | gtk.gdkgl.MODE_DOUBLE
+        mode = mode | MODE_DOUBLE
     else:
-        mode = mode | gtk.gdkgl.MODE_SINGLE
+        mode = mode | MODE_SINGLE
     return mode
 
 def get_MODE_names(mode):
-    import gtk.gdkgl
-    friendly_mode_names = {gtk.gdkgl.MODE_RGB       : "RGB",
-                           gtk.gdkgl.MODE_RGBA      : "RGBA",
-                           gtk.gdkgl.MODE_ALPHA     : "ALPHA",
-                           gtk.gdkgl.MODE_DEPTH     : "DEPTH",
-                           gtk.gdkgl.MODE_DOUBLE    : "DOUBLE",
-                           gtk.gdkgl.MODE_SINGLE    : "SINGLE"}
+    from xpra.client.gl.gtk_compat import MODE_RGB, MODE_RGBA, MODE_ALPHA, MODE_DEPTH, MODE_DOUBLE, MODE_SINGLE
+    friendly_mode_names = {MODE_RGB         : "RGB",
+                           MODE_RGBA        : "RGBA",
+                           MODE_ALPHA       : "ALPHA",
+                           MODE_DEPTH       : "DEPTH",
+                           MODE_DOUBLE      : "DOUBLE",
+                           MODE_SINGLE      : "SINGLE"}
     friendly_modes = [v for k,v in friendly_mode_names.items() if k>0 and (k&mode)==k]
     #special case for single (value is zero!)
-    if not (mode&gtk.gdkgl.MODE_DOUBLE==gtk.gdkgl.MODE_DOUBLE):
+    if not (mode&MODE_DOUBLE==MODE_DOUBLE):
         friendly_modes.append("SINGLE")
     return friendly_modes
 
@@ -140,10 +146,14 @@ def check_functions(*functions):
     else:
         log("All the required OpenGL functions are available: %s " % (", ".join(available)))
 
+
 #sanity checks: OpenGL version and fragment program support:
-def check_GL_support(gldrawable, glcontext, min_texture_size=0, force_enable=False):
-    if not gldrawable.gl_begin(glcontext):
-        raise ImportError("gl_begin failed on %s" % gldrawable)
+def check_GL_support(widget, force_enable=False):
+    from xpra.client.gl.gtk_compat import GLContextManager
+    with GLContextManager(widget):
+        return do_check_GL_support(force_enable)
+
+def do_check_GL_support(force_enable):
     props = {}
     try:
         #log redirection:
@@ -179,7 +189,7 @@ def check_GL_support(gldrawable, glcontext, min_texture_size=0, force_enable=Fal
         else:
             log("found valid OpenGL version: %s.%s", gl_major, gl_minor)
 
-	from OpenGL import version as OpenGL_version
+        from OpenGL import version as OpenGL_version
         try:
             import OpenGL_accelerate
         except:
@@ -187,8 +197,9 @@ def check_GL_support(gldrawable, glcontext, min_texture_size=0, force_enable=Fal
         props["zerocopy"] = is_pyopengl_memoryview_safe(OpenGL_version.__version__) and bool(OpenGL_accelerate)
 
         try:
-            extensions = glGetString(GL_EXTENSIONS).split(" ")
+            extensions = glGetString(GL_EXTENSIONS).decode().split(" ")
         except:
+            log("error querying extensions", exc_info=True)
             extensions = []
             gl_check_error("OpenGL could not find the list of GL extensions - does the graphics driver support OpenGL?")
         log("OpenGL extensions found: %s", ", ".join(extensions))
@@ -206,6 +217,7 @@ def check_GL_support(gldrawable, glcontext, min_texture_size=0, force_enable=Fal
                           ("shading language version", GL_SHADING_LANGUAGE_VERSION, False)):
             try:
                 v = glGetString(s)
+                v = v.decode()
                 log("%s: %s", d, v)
             except:
                 if fatal:
@@ -218,16 +230,23 @@ def check_GL_support(gldrawable, glcontext, min_texture_size=0, force_enable=Fal
         from OpenGL.GLU import gluGetString, GLU_VERSION, GLU_EXTENSIONS
         for d,s in {"GLU version": GLU_VERSION, "GLU extensions":GLU_EXTENSIONS}.items():
             v = gluGetString(s)
+            v = v.decode()
             log("%s: %s", d, v)
             props[d] = v
 
         blacklisted = None
         whitelisted = None
+        greylisted = None
         for k,vlist in BLACKLIST.items():
             v = props.get(k)
             if v in vlist:
                 log("%s '%s' found in blacklist: %s", k, v, vlist)
                 blacklisted = k, v
+        for k,vlist in GREYLIST.items():
+            v = props.get(k)
+            if v in vlist:
+                log("%s '%s' found in greylist: %s", k, v, vlist)
+                greylisted = k, v
         for k,vlist in WHITELIST.items():
             v = props.get(k)
             if v in vlist:
@@ -240,6 +259,11 @@ def check_GL_support(gldrawable, glcontext, min_texture_size=0, force_enable=Fal
                 log.warn("Warning: %s '%s' is blacklisted!", *blacklisted)
             else:
                 gl_check_error("%s '%s' is blacklisted!" % (blacklisted))
+        safe = bool(whitelisted) or not (bool(greylisted) or bool(blacklisted))
+        if safe and sys.version_info[0]>2:
+            log.warn("Warning: OpenGL python3 support is not enabled by default")
+            safe = False
+        props["safe"] = safe
 
         #check for specific functions we need:
         from OpenGL.GL import glActiveTexture, glTexSubImage2D, glTexCoord2i, \
@@ -307,20 +331,17 @@ def check_GL_support(gldrawable, glcontext, min_texture_size=0, force_enable=Fal
             try:
                 from OpenGL.GL import GL_MAX_RECTANGLE_TEXTURE_SIZE
                 rect_texture_size = glGetInteger(GL_MAX_RECTANGLE_TEXTURE_SIZE)
-            except ImportError, e:
+            except ImportError as e:
                 log("OpenGL: %s", e)
                 log("using GL_MAX_TEXTURE_SIZE=%s as default", texture_size)
-        except Exception, e:
+        except Exception as e:
             emsg = str(e)
             if hasattr(e, "description"):
                 emsg = e.description
             gl_check_error("unable to query max texture size: %s" % emsg)
             return props
 
-        if min_texture_size>texture_size or min_texture_size>rect_texture_size:
-            gl_check_error("The texture size is too small: %s" % texture_size)
-        else:
-            log("Texture size GL_MAX_RECTANGLE_TEXTURE_SIZE_ARB=%s, GL_MAX_TEXTURE_SIZE=%s", rect_texture_size, texture_size)
+            log("Texture size GL_MAX_RECTANGLE_TEXTURE_SIZE=%s, GL_MAX_TEXTURE_SIZE=%s", rect_texture_size, texture_size)
         props["texture-size-limit"] = min(rect_texture_size, texture_size)
         return props
     finally:
@@ -380,39 +401,39 @@ def check_GL_support(gldrawable, glcontext, min_texture_size=0, force_enable=Fal
         restore_logger(alogger)
         restore_logger(arlogger)
         restore_logger(clogger)
-        gldrawable.gl_end()
 
-def check_support(min_texture_size=0, force_enable=False, check_colormap=False):
+
+def check_support(force_enable=False, check_colormap=False):
     #platform checks:
     from xpra.platform.gui import gl_check
     warning = gl_check()
     if warning:
-        gl_check_error(warning)
+        if force_enable:
+            log.warn("Warning: trying to continue despite '%s'" % warning)
+        else:
+            gl_check_error(warning)
 
     props = {}
-    import gtk.gdk
-    import gtk.gdkgl, gtk.gtkgl
-    assert gtk.gdkgl is not None and gtk.gtkgl is not None
-    log("pygdkglext version=%s", gtk.gdkgl.pygdkglext_version)
-    props["pygdkglext_version"] = gtk.gdkgl.pygdkglext_version
-    log("gtkglext_version=%s", gtk.gdkgl.gdkglext_version)
-    props["gdkglext_version"] = gtk.gdkgl.gdkglext_version
-    log("pygdkglext OpenGL version=%s", gtk.gdkgl.query_version())
-    props["gdkgl_version"] = gtk.gdkgl.query_version()
+    #this will import gtk.gtkgl / gdkgl or gi.repository.GtkGLExt / GdkGLExt:
+    from xpra.client.gl.gtk_compat import get_info, gdkgl, Config_new_by_mode, MODE_DOUBLE, GLDrawingArea
+    props.update(get_info())
     display_mode = get_DISPLAY_MODE()
-    try:
-        glconfig = gtk.gdkgl.Config(mode=display_mode)
-    except gtk.gdkgl.NoMatches, e:
-        log("no match: %s, toggling double-buffering", e)
-        display_mode &= ~gtk.gdkgl.MODE_DOUBLE
-        glconfig = gtk.gdkgl.Config(mode=display_mode)
+    glconfig = Config_new_by_mode(display_mode)
+    if glconfig is None and CAN_DOUBLE_BUFFER:
+        log("trying to toggle double-buffering")
+        display_mode &= ~MODE_DOUBLE
+        glconfig = Config_new_by_mode(display_mode)
+    if not glconfig:
+        gl_check_error("cannot setup an OpenGL context")
     props["display_mode"] = get_MODE_names(display_mode)
     props["glconfig"] = glconfig
     props["has_alpha"] = glconfig.has_alpha()
     props["rgba"] = glconfig.is_rgba()
     log("GL props=%s", props)
-    assert gtk.gdkgl.query_extension()
-    glcontext, gldrawable, glext, w = None, None, None, None
+    from xpra.gtk_common.gtk_util import import_gtk, gdk_window_process_all_updates
+    gtk = import_gtk()
+    assert gdkgl.query_extension()
+    glext, w = None, None
     try:
         #ugly code for win32 and others (virtualbox broken GL drivers)
         #for getting a GL drawable and context: we must use a window...
@@ -425,7 +446,7 @@ def check_support(min_texture_size=0, force_enable=False, check_colormap=False):
         w = gtk.Window()
         w.set_decorated(False)
         vbox = gtk.VBox()
-        glarea = gtk.gtkgl.DrawingArea(glconfig)
+        glarea = GLDrawingArea(glconfig)
         glarea.set_size_request(32, 32)
         vbox.add(glarea)
         vbox.show_all()
@@ -433,25 +454,26 @@ def check_support(min_texture_size=0, force_enable=False, check_colormap=False):
         #we don't need to actually show the window!
         #w.show_all()
         glarea.realize()
-        gtk.gdk.window_process_all_updates()
-        gldrawable = glarea.get_gl_drawable()
-        glcontext = glarea.get_gl_context()
+        gdk_window_process_all_updates()
 
-        gl_props = check_GL_support(gldrawable, glcontext, min_texture_size, force_enable)
+        gl_props = check_GL_support(glarea, force_enable)
 
         if check_colormap:
-            rgb_visual = w.get_screen().get_rgb_visual()
-            rgba_visual = w.get_screen().get_rgba_visual()
-            gl_props["rgb_visual"] = visual_to_str(rgb_visual)
-            gl_props["rgba_visual"] = visual_to_str(rgba_visual)
+            s = w.get_screen()
+            for x in ("rgb_visual", "rgba_visual", "system_visual"):
+                try:
+                    visual = getattr(s, "get_%s" % x)()
+                    gl_props[x] = visual_to_str(visual)
+                except:
+                    pass
             #i = 0
-            #for v in w.get_screen().list_visuals():
+            #for v in s.list_visuals():
             #    gl_props["visual[%s]" % i] = visual_to_str(v)
             #    i += 1
     finally:
         if w:
             w.destroy()
-        del glcontext, gldrawable, glext, glconfig
+        del glext, glconfig
     props.update(gl_props)
     return props
 
@@ -466,6 +488,8 @@ def main():
         verbose = "-v" in sys.argv or "--verbose" in sys.argv
         if verbose:
             log.enable_debug()
+            from xpra.client.gl.gtk_compat import log as clog
+            clog.enable_debug()
         #replace ImportError with a log message:
         global gl_check_error
         errors = []
@@ -473,7 +497,7 @@ def main():
             log.error("ERROR: %s", msg)
             errors.append(msg)
         gl_check_error = log_error
-        props = check_support(0, True, verbose)
+        props = check_support(True, verbose)
         log.info("")
         if len(errors)>0:
             log.info("OpenGL errors:")
