@@ -1,5 +1,5 @@
 # This file is part of Xpra.
-# Copyright (C) 2010-2014 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2010-2015 Antoine Martin <antoine@devloop.org.uk>
 # Copyright (C) 2008, 2010 Nathaniel Smith <njs@pobox.com>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
@@ -15,7 +15,7 @@ import re
 from xpra.util import nonl, DONE
 from xpra.os_util import bytestostr
 from xpra.client.client_base import XpraClientBase, EXTRA_TIMEOUT, \
-    EXIT_TIMEOUT, EXIT_OK, EXIT_UNSUPPORTED, EXIT_REMOTE_ERROR
+    EXIT_TIMEOUT, EXIT_OK, EXIT_UNSUPPORTED, EXIT_REMOTE_ERROR, EXIT_FILE_TOO_BIG
 
 
 class GObjectXpraClient(XpraClientBase, gobject.GObject):
@@ -111,6 +111,11 @@ class CommandConnectClient(GObjectXpraClient):
         GObjectXpraClient.init(self, opts)
         self.connect_with_timeout(conn)
         self._protocol._log_stats  = False
+        #not used by command line clients,
+        #so don't try probing for printers, etc
+        self.file_transfer = False
+        self.printing = False
+
 
     def make_hello(self):
         capabilities = GObjectXpraClient.make_hello(self)
@@ -169,9 +174,8 @@ class ScreenshotXpraClient(CommandConnectClient):
         if len(img_data)==0:
             self.warn_and_quit(EXIT_OK, "screenshot is empty and has not been saved (maybe there are no windows or they are not currently shown)")
             return
-        f = open(self.screenshot_filename, 'wb')
-        f.write(img_data)
-        f.close()
+        with open(self.screenshot_filename, 'wb') as f:
+            f.write(img_data)
         self.warn_and_quit(EXIT_OK, "screenshot %sx%s saved to: %s" % (w, h, self.screenshot_filename))
 
     def init_packet_handlers(self):
@@ -221,7 +225,7 @@ class InfoXpraClient(CommandConnectClient):
 
     def make_hello(self):
         capabilities = GObjectXpraClient.make_hello(self)
-        log.debug("make_hello() adding info_request to %s", capabilities)
+        log("make_hello() adding info_request to %s", capabilities)
         capabilities["info_request"] = True
         return capabilities
 
@@ -252,7 +256,7 @@ class MonitorXpraClient(SendCommandConnectClient):
 
     def make_hello(self):
         capabilities = SendCommandConnectClient.make_hello(self)
-        log.debug("make_hello() adding info_request to %s", capabilities)
+        log("make_hello() adding info_request to %s", capabilities)
         capabilities["wants_features"]  = True      #so we can verify that the server supports monitor mode
         capabilities["wants_events"]    = True      #tell the server we do support server events
         capabilities["event_request"]   = True      #ask the server to enter this request mode (we're not a proper client)
@@ -276,7 +280,7 @@ class VersionXpraClient(CommandConnectClient):
 
     def make_hello(self):
         capabilities = GObjectXpraClient.make_hello(self)
-        log.debug("make_hello() adding version_request to %s", capabilities)
+        log("make_hello() adding version_request to %s", capabilities)
         capabilities["version_request"] = True
         return capabilities
 
@@ -304,11 +308,55 @@ class ControlXpraClient(CommandConnectClient):
 
     def make_hello(self):
         capabilities = GObjectXpraClient.make_hello(self)
-        log.debug("make_hello() adding command request '%s' to %s", self.command, capabilities)
+        log("make_hello() adding command request '%s' to %s", self.command, capabilities)
         capabilities["command_request"] = self.command
         return capabilities
 
 
+class PrintClient(SendCommandConnectClient):
+    """ Allows us to send a file to the server for printing.
+    """
+    def set_command_args(self, command):
+        log("set_command_args(%s)", command)
+        self.filename = command[0]
+        self.command = command[1:]
+        #FIXME: load as needed...
+        from xpra.os_util import load_binary_file
+        if self.filename=="-":
+            #replace with filename proposed
+            self.filename = command[2]
+            #read file from stdin
+            self.file_data = sys.stdin.read()
+            log("read %i bytes from stdin", len(self.file_data))
+        else:
+            self.file_data = load_binary_file(self.filename)
+        assert self.file_data, "no data found for '%s'" % self.filename
+
+    def client_type(self):
+        return "Python/GObject/Print"
+
+    def timeout(self, *args):
+        self.warn_and_quit(EXIT_TIMEOUT, "timeout: server did not respond")
+
+    def do_command(self):
+        printing = self.server_capabilities.boolget("printing")
+        if not printing:
+            self.warn_and_quit(EXIT_UNSUPPORTED, "server does not support printing")
+            return
+        #TODO: compress file data? (this should run locally most of the time anyway)
+        from xpra.net.compression import Compressed
+        blob = Compressed("print", self.file_data)
+        self.send("print", self.filename, blob, *self.command)
+        log("print: sending %s as %s for printing", self.filename, blob)
+        gobject.idle_add(self.send, "disconnect", DONE, "detaching")
+
+    def make_hello(self):
+        capabilities = SendCommandConnectClient.make_hello(self)
+        capabilities["wants_features"] = True   #so we know if printing is supported or not
+        capabilities["print_request"] = True    #marker to skip full setup
+        return capabilities
+
+
 class ExitXpraClient(SendCommandConnectClient):
     """ This client does one thing only:
         it asks the server to terminate (like stop),
