@@ -3,7 +3,7 @@
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
-from xpra.client.gtk_base.gtk_client_base import GTKXpraClient, xor_str
+from xpra.client.gtk_base.gtk_client_base import GTKXpraClient
 
 import sys
 from gi.repository import GObject               #@UnresolvedImport
@@ -29,16 +29,14 @@ class XpraClient(GTKXpraClient):
     def gtk_main(self):
         Gtk.main()
 
-    def install_signal_handlers(self):
-        #doing so interferes with gtk..
-        pass
-
     def make_hello(self):
         capabilities = GTKXpraClient.make_hello(self)
-        if xor_str is not None:
-            capabilities["encoding.supports_delta"] = [x for x in ("rgb24", "rgb32") if x in self.get_core_encodings()]
+        capabilities["encoding.supports_delta"] = [x for x in ("rgb24", "rgb32") if x in self.get_core_encodings()]
         return capabilities
 
+    def __repr__(self):
+        return "gtk3.client"
+
     def client_type(self):
         return "Python/Gtk3"
 
@@ -52,7 +50,7 @@ class XpraClient(GTKXpraClient):
             try:
                 from xpra.client.gtk3.gtk3_notifier import GTK3_Notifier
                 ncs.append(GTK3_Notifier)
-            except Exception, e:
+            except Exception as e:
                 log("failed to load the GTK3 notification class: %s", e)
         return ncs
 
@@ -61,6 +59,8 @@ class XpraClient(GTKXpraClient):
         for x in ("webp", ):
             if x in cencs:
                 cencs.remove(x)
+        #for some reason, the cairo_workaround does not work for ARGB32
+        #cencs.append("rgb32")
         return cencs
 
 
@@ -72,6 +72,11 @@ class XpraClient(GTKXpraClient):
     def make_clipboard_helper(self):
         return None
 
+    def set_windows_cursor(self, windows, cursor_data):
+        #avoid buggy win32 version:
+        if not sys.platform.startswith("win"):
+            GTKXpraClient.set_windows_cursor(self, windows, cursor_data)
+
 
     def get_root_window(self):
         return Gdk.Screen.get_default().get_root_window()
@@ -98,12 +103,4 @@ class XpraClient(GTKXpraClient):
             w, h = 1920, 1080
         return w, h
 
-    def set_windows_cursor(self, gtkwindows, new_cursor):
-        pass
-
-    def init_opengl(self, enable_opengl):
-        self.opengl_enabled = False
-        self.opengl_props = {"info" : "GTK3 does not support OpenGL"}
-
-
 GObject.type_register(XpraClient)
