@@ -11,7 +11,7 @@ from gi.repository import Gtk                   #@UnresolvedImport @UnusedImport
 from gi.repository import Gdk                   #@UnresolvedImport @UnusedImport
 from gi.repository import GdkPixbuf             #@UnresolvedImport @UnusedImport
 
-from xpra.client.gtk_base.cairo_backing import CairoBacking
+from xpra.client.gtk3.cairo_backing import CairoBacking
 from xpra.client.gtk_base.gtk_client_window_base import GTKClientWindowBase, HAS_X11_BINDINGS
 from xpra.log import Logger
 log = Logger("gtk", "window")
@@ -75,8 +75,6 @@ GTK3 version of the ClientWindow class
 """
 class ClientWindow(GTKClientWindowBase):
 
-    #WINDOW_POPUP        = Gtk.WindowType.POPUP
-    #WINDOW_TOPLEVEL     = Gtk.WindowType.TOPLEVEL
     WINDOW_EVENT_MASK   = GTK3_WINDOW_EVENT_MASK
     BUTTON_MASK         = GTK3_BUTTON_MASK
     SCROLL_MAP          = GTK3_SCROLL_MAP
@@ -86,6 +84,9 @@ class ClientWindow(GTKClientWindowBase):
     WINDOW_STATE_FULLSCREEN = Gdk.WindowState.FULLSCREEN
     WINDOW_STATE_MAXIMIZED  = Gdk.WindowState.MAXIMIZED
     WINDOW_STATE_ICONIFIED  = Gdk.WindowState.ICONIFIED
+    WINDOW_STATE_ABOVE      = Gdk.WindowState.ABOVE
+    WINDOW_STATE_BELOW      = Gdk.WindowState.BELOW
+    WINDOW_STATE_STICKY     = Gdk.WindowState.STICKY
 
 
     def init_window(self, metadata):
@@ -120,8 +121,16 @@ class ClientWindow(GTKClientWindowBase):
     def get_backing_class(self):
         return CairoBacking
 
-    def gdk_window(self):
-        return  self.get_window()
+    def enable_alpha(self):
+        screen = self.get_screen()
+        visual = screen.get_rgba_visual()
+        if visual is None or not screen.is_composited():
+            log.error("enable_alpha() cannot handle window transparency on screen %s", screen)
+            return False
+        log("enable_alpha() using rgba visual %s for wid %s", visual, self._id)
+        self.set_visual(visual)
+        return True
+
 
     def xget_u32_property(self, target, name):
         try:
@@ -133,7 +142,7 @@ class ClientWindow(GTKClientWindowBase):
                     return  None
                 log("xget_u32_property(%s, %s)=%s", target, name, prop[2][0])
                 return prop[2][0]
-        except Exception, e:
+        except Exception as e:
             log.error("xget_u32_property error on %s / %s: %s", target, name, e)
         return GTKClientWindowBase.xget_u32_property(self, target, name)
 
@@ -183,8 +192,9 @@ class ClientWindow(GTKClientWindowBase):
                 field = ASPECT_FIELDS.get(k)
                 setattr(geom, field, float(v))
                 mask |= int(name_to_hint.get(k, 0))
-        hints = Gdk.WindowHints(mask)
-        self.set_geometry_hints(None, geom, hints)
+        gdk_hints = Gdk.WindowHints(mask)
+        log("apply_geometry_hints(%s) geometry=%s, hints=%s", hints, geom, gdk_hints)
+        self.set_geometry_hints(None, geom, gdk_hints)
 
 
     def queue_draw(self, x, y, width, height):
