@@ -14,30 +14,33 @@ import os.path
 import sys
 import shlex
 import signal
+import traceback
+
+from xpra.gtk_common.gobject_compat import import_gtk, import_gdk, import_gobject, import_pango
+gobject = import_gobject()
+gobject.threads_init()
+gtk = import_gtk()
+gdk = import_gdk()
+try:
+    gdk.threads_init()
+except:
+    pass
+pango = import_pango()
 
 from xpra.platform.gui import init as gui_init
 gui_init()
 
-from xpra.gtk_common.gobject_compat import import_gtk, import_gdk, import_gobject, import_pango
-
-gtk = import_gtk()
-gdk = import_gdk()
-gobject = import_gobject()
-gobject.threads_init()
-pango = import_pango()
-
-
 from xpra.scripts.config import read_config, make_defaults_struct, validate_config, save_config, ENCRYPTION_CIPHERS
 from xpra.codecs.loader import PREFERED_ENCODING_ORDER
-from xpra.gtk_common.gtk_util import set_tooltip_text, add_close_accel, scaled_image, pixbuf_new_from_file, is_gtk3, \
+from xpra.gtk_common.gtk_util import gtk_main, add_close_accel, scaled_image, pixbuf_new_from_file, color_parse, \
                                     OptionMenu, choose_file, \
                                     WIN_POS_CENTER, STATE_NORMAL, \
+                                    DIALOG_DESTROY_WITH_PARENT, MESSAGE_INFO,  BUTTONS_CLOSE, \
                                     FILE_CHOOSER_ACTION_SAVE, FILE_CHOOSER_ACTION_OPEN
 from xpra.os_util import thread
 from xpra.client.gtk_base.gtk_tray_menu_base import make_min_auto_menu, make_encodingsmenu, set_use_tray_workaround, \
                                     MIN_QUALITY_OPTIONS, QUALITY_OPTIONS, MIN_SPEED_OPTIONS, SPEED_OPTIONS
 from xpra.client.gtk_base.about import about
-from xpra.client.client_base import SIGNAMES
 from xpra.scripts.main import connect_to, make_client
 from xpra.platform.paths import get_icon_dir
 from xpra.log import Logger, enable_debug_for
@@ -79,19 +82,19 @@ LAUNCHER_VALIDATION = {
                         "mode"              : lambda x : validate_in_list(x, MODES),
                       }
 
+PYTHON3 = sys.version_info[0]==3
 
-black   = gdk.color_parse("black")
-red     = gdk.color_parse("red")
-white   = gdk.color_parse("white")
+
+black   = color_parse("black")
+red     = color_parse("red")
+white   = color_parse("white")
 
 
 def get_active_item_index(optionmenu):
-    i = 0
     menu = optionmenu.get_menu()
-    for x in menu.get_children():
+    for i, x in enumerate(menu.get_children()):
         if hasattr(x, "get_active") and x.get_active():
             return i
-        i += 1
     return -1
 
 def set_history_from_active(optionmenu):
@@ -115,10 +118,6 @@ class ApplicationWindow:
         fixup_packetencoding(self.config)
         #what we save by default:
         self.config_keys = set(SAVED_FIELDS)
-        if is_gtk3():
-            self.config.client_toolkit = "gtk3"
-        else:
-            self.config.client_toolkit = "gtk2"
         def raise_exception(*args):
             raise Exception(*args)
         self.client = make_client(raise_exception, self.config)
@@ -148,7 +147,7 @@ class ApplicationWindow:
             settings = logo_button.get_settings()
             settings.set_property('gtk-button-images', True)
             logo_button.connect("clicked", about)
-            set_tooltip_text(logo_button, "About")
+            logo_button.set_tooltip_text("About")
             image = gtk.Image()
             image.set_from_pixbuf(icon_pixbuf)
             logo_button.set_image(image)
@@ -163,7 +162,7 @@ class ApplicationWindow:
         hbox.set_spacing(20)
         hbox.pack_start(gtk.Label("Mode: "))
         self.mode_combo = gtk.combo_box_new_text()
-        self.mode_combo.get_model().clear()
+        #self.mode_combo.get_model().clear()
         self.mode_combo.append_text("TCP")
         if "AES" in ENCRYPTION_CIPHERS:
             self.mode_combo.append_text("TCP + AES")
@@ -172,86 +171,94 @@ class ApplicationWindow:
         hbox.pack_start(self.mode_combo)
         vbox.pack_start(hbox)
 
-        # Encoding:
-        hbox = gtk.HBox(False, 20)
-        hbox.set_spacing(20)
-        hbox.pack_start(gtk.Label("Encoding: "))
-        self.encoding_combo = OptionMenu()
-        def get_current_encoding():
-            return self.config.encoding
-        def set_new_encoding(e):
-            self.config.encoding = e
-        encodings = [x for x in PREFERED_ENCODING_ORDER if x in self.client.get_encodings()]
-        server_encodings = encodings
-        es = make_encodingsmenu(get_current_encoding, set_new_encoding, encodings, server_encodings)
-        self.encoding_combo.set_menu(es)
-        set_history_from_active(self.encoding_combo)
-        hbox.pack_start(self.encoding_combo)
-        vbox.pack_start(hbox)
-        self.encoding_combo.connect("changed", self.encoding_changed)
+        self.encoding_combo = None
+        if not PYTHON3:
+            #not implemented for gtk3, where we can't use set_menu()...
 
-        # Quality
-        hbox = gtk.HBox(False, 20)
-        hbox.set_spacing(20)
-        self.quality_label = gtk.Label("Quality: ")
-        hbox.pack_start(self.quality_label)
-        self.quality_combo = OptionMenu()
-        def set_min_quality(q):
-            self.config.min_quality = q
-        def set_quality(q):
-            self.config.quality = q
-        def get_min_quality():
-            return self.config.min_quality
-        def get_quality():
-            return self.config.quality
-        sq = make_min_auto_menu("Quality", MIN_QUALITY_OPTIONS, QUALITY_OPTIONS,
-                                   get_min_quality, get_quality, set_min_quality, set_quality)
-        self.quality_combo.set_menu(sq)
-        set_history_from_active(self.quality_combo)
-        hbox.pack_start(self.quality_combo)
-        vbox.pack_start(hbox)
+            # Encoding:
+            hbox = gtk.HBox(False, 20)
+            hbox.set_spacing(20)
+            hbox.pack_start(gtk.Label("Encoding: "))
+            self.encoding_combo = OptionMenu()
+            def get_current_encoding():
+                return self.config.encoding
+            def set_new_encoding(e):
+                self.config.encoding = e
+            encodings = [x for x in PREFERED_ENCODING_ORDER if x in self.client.get_encodings()]
+            server_encodings = encodings
+            es = make_encodingsmenu(get_current_encoding, set_new_encoding, encodings, server_encodings)
+            self.encoding_combo.set_menu(es)
+            set_history_from_active(self.encoding_combo)
+            hbox.pack_start(self.encoding_combo)
+            vbox.pack_start(hbox)
+            self.encoding_combo.connect("changed", self.encoding_changed)
 
-        # Speed
-        hbox = gtk.HBox(False, 20)
-        hbox.set_spacing(20)
-        self.speed_label = gtk.Label("Speed: ")
-        hbox.pack_start(self.speed_label)
-        self.speed_combo = OptionMenu()
-        def set_min_speed(s):
-            self.config.min_speed = s
-        def set_speed(s):
-            self.config.speed = s
-        def get_min_speed():
-            return self.config.min_speed
-        def get_speed():
-            return self.config.speed
-        ss = make_min_auto_menu("Speed", MIN_SPEED_OPTIONS, SPEED_OPTIONS,
-                                   get_min_speed, get_speed, set_min_speed, set_speed)
-        self.speed_combo.set_menu(ss)
-        set_history_from_active(self.speed_combo)
-        hbox.pack_start(self.speed_combo)
-        vbox.pack_start(hbox)
+            # Quality
+            hbox = gtk.HBox(False, 20)
+            hbox.set_spacing(20)
+            self.quality_label = gtk.Label("Quality: ")
+            hbox.pack_start(self.quality_label)
+            self.quality_combo = OptionMenu()
+            def set_min_quality(q):
+                self.config.min_quality = q
+            def set_quality(q):
+                self.config.quality = q
+            def get_min_quality():
+                return self.config.min_quality
+            def get_quality():
+                return self.config.quality
+            sq = make_min_auto_menu("Quality", MIN_QUALITY_OPTIONS, QUALITY_OPTIONS,
+                                       get_min_quality, get_quality, set_min_quality, set_quality)
+            self.quality_combo.set_menu(sq)
+            set_history_from_active(self.quality_combo)
+            hbox.pack_start(self.quality_combo)
+            vbox.pack_start(hbox)
+
+            # Speed
+            hbox = gtk.HBox(False, 20)
+            hbox.set_spacing(20)
+            self.speed_label = gtk.Label("Speed: ")
+            hbox.pack_start(self.speed_label)
+            self.speed_combo = OptionMenu()
+            def set_min_speed(s):
+                self.config.min_speed = s
+            def set_speed(s):
+                self.config.speed = s
+            def get_min_speed():
+                return self.config.min_speed
+            def get_speed():
+                return self.config.speed
+            ss = make_min_auto_menu("Speed", MIN_SPEED_OPTIONS, SPEED_OPTIONS,
+                                       get_min_speed, get_speed, set_min_speed, set_speed)
+            self.speed_combo.set_menu(ss)
+            set_history_from_active(self.speed_combo)
+            hbox.pack_start(self.speed_combo)
+            vbox.pack_start(hbox)
 
         # Username@Host:Port
         hbox = gtk.HBox(False, 0)
         hbox.set_spacing(5)
-        self.username_entry = gtk.Entry(max=128)
+        self.username_entry = gtk.Entry()
+        self.username_entry.set_max_length(128)
         self.username_entry.set_width_chars(16)
         self.username_entry.connect("changed", self.validate)
-        set_tooltip_text(self.username_entry, "SSH username")
+        self.username_entry.set_tooltip_text("SSH username")
         self.username_label = gtk.Label("@")
-        self.host_entry = gtk.Entry(max=128)
+        self.host_entry = gtk.Entry()
+        self.host_entry.set_max_length(128)
         self.host_entry.set_width_chars(24)
         self.host_entry.connect("changed", self.validate)
-        set_tooltip_text(self.host_entry, "hostname")
-        self.ssh_port_entry = gtk.Entry(max=5)
+        self.host_entry.set_tooltip_text("hostname")
+        self.ssh_port_entry = gtk.Entry()
+        self.ssh_port_entry.set_max_length(5)
         self.ssh_port_entry.set_width_chars(5)
         self.ssh_port_entry.connect("changed", self.validate)
-        set_tooltip_text(self.ssh_port_entry, "SSH port")
-        self.port_entry = gtk.Entry(max=5)
+        self.ssh_port_entry.set_tooltip_text("SSH port")
+        self.port_entry = gtk.Entry()
+        self.port_entry.set_max_length(5)
         self.port_entry.set_width_chars(5)
         self.port_entry.connect("changed", self.validate)
-        set_tooltip_text(self.port_entry, "port/display")
+        self.port_entry.set_tooltip_text("port/display")
 
         hbox.pack_start(self.username_entry)
         hbox.pack_start(self.username_label)
@@ -264,7 +271,8 @@ class ApplicationWindow:
         # Password
         hbox = gtk.HBox(False, 0)
         hbox.set_spacing(20)
-        self.password_entry = gtk.Entry(max=128)
+        self.password_entry = gtk.Entry()
+        self.password_entry.set_max_length(128)
         self.password_entry.set_width_chars(30)
         self.password_entry.set_text("")
         self.password_entry.set_visibility(False)
@@ -287,12 +295,12 @@ class ApplicationWindow:
         vbox.pack_start(hbox)
         #Save:
         self.save_btn = gtk.Button("Save")
-        set_tooltip_text(self.save_btn, "Save settings to a session file")
+        self.save_btn.set_tooltip_text("Save settings to a session file")
         self.save_btn.connect("clicked", self.save_clicked)
         hbox.pack_start(self.save_btn)
         #Load:
         self.load_btn = gtk.Button("Load")
-        set_tooltip_text(self.load_btn, "Load settings from a session file")
+        self.load_btn.set_tooltip_text("Load settings from a session file")
         self.load_btn.connect("clicked", self.load_clicked)
         hbox.pack_start(self.load_btn)
         # Connect button:
@@ -337,7 +345,7 @@ class ApplicationWindow:
             self.set_widget_bg_color(w, e)
             if e:
                 err_text.append(text)
-        log.debug("validate(%s) err_text=%s, errs=%s", args, err_text, errs)
+        log("validate(%s) err_text=%s, errs=%s", args, err_text, errs)
         self.set_info_text(", ".join(err_text))
         self.set_info_color(len(err_text)>0)
         self.button.set_sensitive(len(err_text)==0)
@@ -348,8 +356,7 @@ class ApplicationWindow:
         self.window.present()
 
     def run(self):
-        from xpra.gtk_common.gtk2common import gtk2main
-        gtk2main()
+        gtk_main()
 
     def get_icon(self, icon_name):
         icon_filename = os.path.join(get_icon_dir(), icon_name)
@@ -362,14 +369,14 @@ class ApplicationWindow:
         self.port_entry.set_text("")
         if ssh:
             self.ssh_port_entry.show()
-            set_tooltip_text(self.port_entry, "Display number (optional)")
-            set_tooltip_text(self.password_entry, "SSH Password")
+            self.port_entry.set_tooltip_text("Display number (optional)")
+            self.password_entry.set_tooltip_text("SSH Password")
             self.username_entry.show()
             self.username_label.show()
         else:
             self.ssh_port_entry.hide()
-            set_tooltip_text(self.port_entry, "port number")
-            set_tooltip_text(self.password_entry, "Session Password")
+            self.port_entry.set_tooltip_text("port number")
+            self.password_entry.set_tooltip_text("Session Password")
             self.username_entry.hide()
             self.username_label.hide()
             if self.config.port>0:
@@ -385,6 +392,8 @@ class ApplicationWindow:
         self.validate()
 
     def get_selected_encoding(self, *args):
+        if not self.encoding_combo:
+            return ""
         index = get_active_item_index(self.encoding_combo)
         return self.encoding_combo.get_menu().index_to_encoding.get(index)
 
@@ -392,7 +401,6 @@ class ApplicationWindow:
         encoding = self.get_selected_encoding()
         uses_quality_option = encoding in ["jpeg", "webp", "h264"]
         log("encoding_changed(%s) uses_quality_option(%s)=%s", args, encoding, uses_quality_option)
-        #to prevent win32 crashes:
         if uses_quality_option:
             self.quality_combo.show()
             self.quality_label.show()
@@ -436,7 +444,6 @@ class ApplicationWindow:
         t = str(e)
         if self.config.debug:
             #in debug mode, include the full stacktrace:
-            import traceback
             t = traceback.format_exc()
         def ui_handle_exception():
             self.reset_client()
@@ -449,7 +456,7 @@ class ApplicationWindow:
     def do_connect(self):
         try:
             self.connect_builtin()
-        except Exception, e:
+        except Exception as e:
             log.error("cannot connect:", exc_info=True)
             self.handle_exception(e)
 
@@ -486,10 +493,6 @@ class ApplicationWindow:
                 params["username"] = username
                 full_ssh += ["-l", username]
             full_ssh += ["-T", host]
-            if sys.platform.startswith("win"):
-                #putty needs those:
-                full_ssh.append("-ssh")
-                full_ssh.append("-agent")
             if str(self.config.ssh_port)!="22":
                 if sys.platform.startswith("win"):
                     full_ssh += ["-P", str(self.config.ssh_port)]
@@ -521,7 +524,7 @@ class ApplicationWindow:
         self.set_sensitive(False)
         try:
             conn = connect_to(params, self.set_info_text, ssh_fail_cb=self.ssh_failed)
-        except Exception, e:
+        except Exception as e:
             log.error("failed to connect", exc_info=True)
             self.handle_exception(e)
             return
@@ -531,7 +534,7 @@ class ApplicationWindow:
     def start_XpraClient(self, conn):
         try:
             self.do_start_XpraClient(conn)
-        except Exception, e:
+        except Exception as e:
             log.error("failed to start client", exc_info=True)
             self.handle_exception(e)
 
@@ -594,7 +597,7 @@ class ApplicationWindow:
         self.client.quit = quit_override
         try:
             self.client.run()
-        except Exception, e:
+        except Exception as e:
             log.error("client error", exc_info=True)
             self.handle_exception(e)
 
@@ -628,7 +631,7 @@ class ApplicationWindow:
         self.config.ssh_port = self.ssh_port_entry.get_text()
         self.config.port = self.port_entry.get_text()
         self.config.username = self.username_entry.get_text()
-        self.config.encoding = self.get_selected_encoding()
+        self.config.encoding = self.get_selected_encoding() or self.config.encoding
         mode_enc = self.mode_combo.get_active_text()
         if mode_enc.startswith("TCP"):
             self.config.mode = "tcp"
@@ -647,7 +650,7 @@ class ApplicationWindow:
             self.mode_combo.set_active(1)
         else:
             self.mode_combo.set_active(2)
-        if self.config.encoding:
+        if self.config.encoding and self.encoding_combo:
             index = self.encoding_combo.get_menu().encoding_to_index.get(self.config.encoding, -1)
             log("setting encoding combo to %s / %s", self.config.encoding, index)
             if index>=0:
@@ -680,6 +683,7 @@ class ApplicationWindow:
             fn = k.replace("-", "_")
             setattr(self.config, fn, v)
         self.config_keys = self.config_keys.union(set(props.keys()))
+        log("update_options_from_file(%s) populated config with keys '%s', ssh=%s", filename, options.keys(), self.config.ssh)
 
     def choose_session_file(self, title, action, action_button, callback):
         file_filter = gtk.FileFilter()
@@ -700,7 +704,22 @@ class ApplicationWindow:
         self.choose_session_file("Load session settings from file", FILE_CHOOSER_ACTION_OPEN, gtk.STOCK_OPEN, do_load)
 
 
+#on some platforms like win32, we don't have stdout
+#and this is a GUI application, so show a dialog with the error instead
+def exception_dialog(title):
+    md = gtk.MessageDialog(None, DIALOG_DESTROY_WITH_PARENT, MESSAGE_INFO,  BUTTONS_CLOSE, title)
+    md.format_secondary_text(traceback.format_exc())
+    md.show_all()
+    def close_dialog(*args):
+        md.destroy()
+        gtk.main_quit()
+    md.connect("response", close_dialog)
+    md.connect("close", close_dialog)
+    gtk_main()
+
+
 def main():
+    from xpra.os_util import SIGNAMES
     from xpra.gtk_common.quit import gtk_main_quit_on_fatal_exceptions_enable
     gtk_main_quit_on_fatal_exceptions_enable()
 
@@ -709,42 +728,49 @@ def main():
     platform_init("Xpra-Launcher", "Xpra Connection Launcher")
     gui_init()
 
-    #logging init:
-    from xpra.scripts.main import parse_cmdline, fixup_debug_option
-    options, args = parse_cmdline(sys.argv)
-    debug = fixup_debug_option(options.debug)
-    if debug:
-        for x in debug.split(","):
-            enable_debug_for(x)
+    try:
+        from xpra.scripts.main import parse_cmdline, fixup_debug_option
+        options, args = parse_cmdline(sys.argv)
+        debug = fixup_debug_option(options.debug)
+        if debug:
+            for x in debug.split(","):
+                enable_debug_for(x)
+    except Exception:
+        exception_dialog("Error parsing command line")
+        return 1
 
-    app = ApplicationWindow()
-    def app_signal(signum, frame):
-        print("")
-        log("got signal %s" % SIGNAMES.get(signum, signum))
-        def show_signal():
-            app.show()
-            app.client.cleanup()
-            gobject.timeout_add(1000, app.set_info_text, "got signal %s" % SIGNAMES.get(signum, signum))
-            gobject.timeout_add(1000, app.set_info_color, True)
-        #call from UI thread:
-        gobject.idle_add(show_signal)
-    signal.signal(signal.SIGINT, app_signal)
-    signal.signal(signal.SIGTERM, app_signal)
-    has_file = len(args) == 1
-    if has_file:
-        app.update_options_from_file(args[0])
-    debug = fixup_debug_option(app.config.debug)
-    if debug:
-        for x in debug.split(","):
-            enable_debug_for(x)
-   #suspend tray workaround for our window widgets:
-    try:
-        set_use_tray_workaround(False)
-        app.create_window()
-    finally:
-        set_use_tray_workaround(True)
     try:
+        app = ApplicationWindow()
+        def app_signal(signum, frame):
+            print("")
+            log("got signal %s" % SIGNAMES.get(signum, signum))
+            def show_signal():
+                app.show()
+                app.client.cleanup()
+                gobject.timeout_add(1000, app.set_info_text, "got signal %s" % SIGNAMES.get(signum, signum))
+                gobject.timeout_add(1000, app.set_info_color, True)
+            #call from UI thread:
+            gobject.idle_add(show_signal)
+        signal.signal(signal.SIGINT, app_signal)
+        signal.signal(signal.SIGTERM, app_signal)
+        has_file = len(args) == 1
+        if has_file:
+            app.update_options_from_file(args[0])
+        debug = fixup_debug_option(app.config.debug)
+        if debug:
+            for x in debug.split(","):
+                enable_debug_for(x)
+        #suspend tray workaround for our window widgets:
+        try:
+            set_use_tray_workaround(False)
+            app.create_window()
+        finally:
+            set_use_tray_workaround(True)
         app.update_gui_from_config()
+    except Exception:
+        exception_dialog("Error creating launcher form")
+        return 1
+    try:
         if app.config.autoconnect:
             #file says we should connect,
             #do that only (not showing UI unless something goes wrong):
