@@ -11,10 +11,11 @@ gtk = import_gtk()
 gobject = import_gobject()
 
 from xpra.util import CLIENT_EXIT
-from xpra.gtk_common.gtk_util import set_tooltip_text, ensure_item_selected, menuitem, BUTTON_PRESS_MASK
+from xpra.gtk_common.gtk_util import ensure_item_selected, menuitem, BUTTON_PRESS_MASK
 from xpra.client.client_base import EXIT_OK
 from xpra.client.gtk_base.about import about, close_about
 from xpra.codecs.loader import PREFERED_ENCODING_ORDER, ENCODINGS_HELP, ENCODINGS_TO_NAME
+from xpra.platform.gui import get_icon_size
 from xpra.log import Logger
 log = Logger("tray")
 
@@ -86,6 +87,15 @@ def set_use_tray_workaround(enabled):
         CheckMenuItem = gtk.CheckMenuItem
 
 
+def set_sensitive(widget, sensitive):
+    if sys.platform.startswith("darwin"):
+        if sensitive:
+            widget.show()
+        else:
+            widget.hide()
+    widget.set_sensitive(sensitive)
+
+
 def make_min_auto_menu(title, min_options, options, get_current_min_value, get_current_value, set_min_value_cb, set_value_cb):
     #note: we must keep references to the parameters on the submenu
     #(closures and gtk callbacks don't mix so well!)
@@ -95,7 +105,7 @@ def make_min_auto_menu(title, min_options, options, get_current_min_value, get_c
     submenu.set_min_value_cb = set_min_value_cb
     submenu.set_value_cb = set_value_cb
     fstitle = gtk.MenuItem("Fixed %s:" % title)
-    fstitle.set_sensitive(False)
+    set_sensitive(fstitle, False)
     submenu.append(fstitle)
     submenu.menu_items = {}
     submenu.min_menu_items = {}
@@ -113,7 +123,7 @@ def make_min_auto_menu(title, min_options, options, get_current_min_value, get_c
             found_match |= candidate_match
             qi.connect('activate', set_fn, submenu)
             if s>0:
-                set_tooltip_text(qi, "%s%%" % s)
+                qi.set_tooltip_text("%s%%" % s)
             submenu.append(qi)
             items[s] = qi
         return items
@@ -140,7 +150,7 @@ def make_min_auto_menu(title, min_options, options, get_current_min_value, get_c
     submenu.menu_items.update(populate_menu(options, get_current_value(), set_value))
     submenu.append(gtk.SeparatorMenuItem())
     mstitle = gtk.MenuItem("Minimum %s:" % title)
-    mstitle.set_sensitive(False)
+    set_sensitive(mstitle, False)
     submenu.append(mstitle)
     def set_min_value(item, ss):
         if not item.get_active():
@@ -185,8 +195,7 @@ def populate_encodingsmenu(encodings_submenu, get_current_encoding, set_encoding
     encodings_submenu.index_to_encoding = {}
     encodings_submenu.encoding_to_index = {}
     NAME_TO_ENCODING = {}
-    i = 0
-    for encoding in encodings:
+    for i, encoding in enumerate(encodings):
         name = ENCODINGS_TO_NAME.get(encoding, encoding)
         descr = ENCODINGS_HELP.get(encoding)
         NAME_TO_ENCODING[name] = encoding
@@ -194,7 +203,7 @@ def populate_encodingsmenu(encodings_submenu, get_current_encoding, set_encoding
         if descr:
             if encoding not in server_encodings:
                 descr += "\n(not available on this server)"
-            set_tooltip_text(encoding_item, descr)
+            encoding_item.set_tooltip_text(descr)
         def encoding_changed(item):
             ensure_item_selected(encodings_submenu, item)
             enc = NAME_TO_ENCODING.get(item.get_label())
@@ -206,13 +215,12 @@ def populate_encodingsmenu(encodings_submenu, get_current_encoding, set_encoding
         sensitive = encoding in server_encodings
         if not sensitive and HIDE_DISABLED_MENU_ENTRIES:
             continue
-        encoding_item.set_sensitive(encoding in server_encodings)
+        set_sensitive(encoding_item, encoding in server_encodings)
         encoding_item.set_draw_as_radio(True)
         encoding_item.connect("toggled", encoding_changed)
         encodings_submenu.append(encoding_item)
         encodings_submenu.index_to_encoding[i] = encoding
         encodings_submenu.encoding_to_index[encoding] = i
-        i += 1
     encodings_submenu.show_all()
 
 
@@ -256,30 +264,31 @@ class GTKTrayMenuBase(object):
         menu.append(self.make_bellmenuitem())
         if self.client.windows_enabled:
             menu.append(self.make_cursorsmenuitem())
+        if not self.client.readonly and self.client.keyboard_helper:
+            menu.append(self.make_layoutsmenuitem())
+        if self.client.windows_enabled and not self.client.readonly:
+            menu.append(self.make_keyboardsyncmenuitem())
         menu.append(self.make_notificationsmenuitem())
         if not self.client.readonly:
             menu.append(self.make_clipboardmenuitem())
-        if self.client.opengl_enabled:
+        if self.client.client_supports_opengl:
             menu.append(self.make_openglmenuitem())
         if self.client.windows_enabled and len(self.client.get_encodings())>1:
             menu.append(self.make_encodingsmenuitem())
         menu.append(self.make_qualitymenuitem())
         menu.append(self.make_speedmenuitem())
-        if self.client.speaker_allowed and STARTSTOP_SOUND_MENU:
+        if STARTSTOP_SOUND_MENU:
             menu.append(self.make_speakermenuitem())
-        if self.client.microphone_allowed and STARTSTOP_SOUND_MENU:
+        if STARTSTOP_SOUND_MENU:
             menu.append(self.make_microphonemenuitem())
         if SHOW_COMPRESSION_MENU:
             menu.append(self.make_compressionmenu())
-        if not self.client.readonly and self.client.keyboard_helper:
-            menu.append(self.make_layoutsmenuitem())
-        if self.client.windows_enabled and not self.client.readonly:
-            menu.append(self.make_keyboardsyncmenuitem())
         if self.client.windows_enabled:
             menu.append(self.make_refreshmenuitem())
             menu.append(self.make_raisewindowsmenuitem())
         #menu.append(item("Options", "configure", None, self.options))
         menu.append(gtk.SeparatorMenuItem())
+        menu.append(self.make_startnewcommandmenuitem())
         menu.append(self.make_disconnectmenuitem())
         if show_close:
             menu.append(self.make_closemenuitem())
@@ -317,9 +326,9 @@ class GTKTrayMenuBase(object):
     def handshake_menuitem(self, *args, **kwargs):
         """ Same as menuitem() but this one will be disabled until we complete the server handshake """
         mi = self.menuitem(*args, **kwargs)
-        mi.set_sensitive(False)
+        set_sensitive(mi, False)
         def enable_menuitem(*args):
-            mi.set_sensitive(True)
+            set_sensitive(mi, True)
         self.client.connect("handshake-complete", enable_menuitem)
         return mi
 
@@ -327,7 +336,8 @@ class GTKTrayMenuBase(object):
         """ Utility method for easily creating an ImageMenuItem """
         image = None
         if icon_name:
-            image = self.get_image(icon_name, 24)
+            icon_size = get_icon_size()
+            image = self.get_image(icon_name, icon_size)
         return menuitem(title, image, tooltip, cb)
 
     def checkitem(self, title, cb=None, active=False):
@@ -369,17 +379,17 @@ class GTKTrayMenuBase(object):
                 self.client.send_bell_enabled()
             log("bell_toggled(%s) bell_enabled=%s", args, self.client.bell_enabled)
         self.bell_menuitem = self.checkitem("Bell", bell_toggled)
-        self.bell_menuitem.set_sensitive(False)
+        set_sensitive(self.bell_menuitem, False)
         def set_bell_menuitem(*args):
             log("set_bell_menuitem%s enabled=%s", args, self.client.bell_enabled)
             self.bell_menuitem.set_active(self.client.bell_enabled)
             c = self.client
             can_toggle_bell = c.toggle_cursors_bell_notify and c.server_supports_bell and c.client_supports_bell
-            self.bell_menuitem.set_sensitive(can_toggle_bell)
+            set_sensitive(self.bell_menuitem, can_toggle_bell)
             if can_toggle_bell:
-                set_tooltip_text(self.bell_menuitem, "Forward system bell")
+                self.bell_menuitem.set_tooltip_text("Forward system bell")
             else:
-                set_tooltip_text(self.bell_menuitem, "Cannot forward the system bell: the feature has been disabled")
+                self.bell_menuitem.set_tooltip_text("Cannot forward the system bell: the feature has been disabled")
         self.client.connect("handshake-complete", set_bell_menuitem)
         return  self.bell_menuitem
 
@@ -394,17 +404,17 @@ class GTKTrayMenuBase(object):
                 self.client.reset_cursor()
             log("cursors_toggled(%s) cursors_enabled=%s", args, self.client.cursors_enabled)
         self.cursors_menuitem = self.checkitem("Cursors", cursors_toggled)
-        self.cursors_menuitem.set_sensitive(False)
+        set_sensitive(self.cursors_menuitem, False)
         def set_cursors_menuitem(*args):
             log("set_cursors_menuitem%s enabled=%s", args, self.client.cursors_enabled)
             self.cursors_menuitem.set_active(self.client.cursors_enabled)
             c = self.client
             can_toggle_cursors = c.toggle_cursors_bell_notify and c.server_supports_cursors and c.client_supports_cursors
-            self.cursors_menuitem.set_sensitive(can_toggle_cursors)
+            set_sensitive(self.cursors_menuitem, can_toggle_cursors)
             if can_toggle_cursors:
-                set_tooltip_text(self.cursors_menuitem, "Forward custom mouse cursors")
+                self.cursors_menuitem.set_tooltip_text("Forward custom mouse cursors")
             else:
-                set_tooltip_text(self.cursors_menuitem, "Cannot forward mouse cursors: the feature has been disabled")
+                self.cursors_menuitem.set_tooltip_text("Cannot forward mouse cursors: the feature has been disabled")
         self.client.connect("handshake-complete", set_cursors_menuitem)
         return  self.cursors_menuitem
 
@@ -417,17 +427,17 @@ class GTKTrayMenuBase(object):
             if changed and self.client.toggle_cursors_bell_notify:
                 self.client.send_notify_enabled()
         self.notifications_menuitem = self.checkitem("Notifications", notifications_toggled)
-        self.notifications_menuitem.set_sensitive(False)
+        set_sensitive(self.notifications_menuitem, False)
         def set_notifications_menuitem(*args):
             log("set_notifications_menuitem%s enabled=%s", args, self.client.notifications_enabled)
             self.notifications_menuitem.set_active(self.client.notifications_enabled)
             c = self.client
             can_notify = c.toggle_cursors_bell_notify and c.server_supports_notifications and c.client_supports_notifications
-            self.notifications_menuitem.set_sensitive(can_notify)
+            set_sensitive(self.notifications_menuitem, can_notify)
             if can_notify:
-                set_tooltip_text(self.notifications_menuitem, "Forward system notifications")
+                self.notifications_menuitem.set_tooltip_text("Forward system notifications")
             else:
-                set_tooltip_text(self.notifications_menuitem, "Cannot forward system notifications: the feature has been disabled")
+                self.notifications_menuitem.set_tooltip_text("Cannot forward system notifications: the feature has been disabled")
         self.client.connect("handshake-complete", set_notifications_menuitem)
         return self.notifications_menuitem
 
@@ -439,17 +449,17 @@ class GTKTrayMenuBase(object):
                 self.client.clipboard_enabled = new_state
                 self.client.emit("clipboard-toggled")
         self.clipboard_menuitem = self.checkitem("Clipboard", menu_clipboard_toggled)
-        self.clipboard_menuitem.set_sensitive(False)
+        set_sensitive(self.clipboard_menuitem, False)
         def set_clipboard_menuitem(*args):
             log("set_clipboard_menuitem%s enabled=%s", args, self.client.clipboard_enabled)
             self.clipboard_menuitem.set_active(self.client.clipboard_enabled)
             c = self.client
             can_clipboard = c.server_supports_clipboard and c.client_supports_clipboard
-            self.clipboard_menuitem.set_sensitive(can_clipboard)
+            set_sensitive(self.clipboard_menuitem, can_clipboard)
             if can_clipboard:
-                set_tooltip_text(self.clipboard_menuitem, "Enable clipboard synchronization")
+                self.clipboard_menuitem.set_tooltip_text("Enable clipboard synchronization")
             else:
-                set_tooltip_text(self.clipboard_menuitem, "Clipboard synchronization cannot be enabled: disabled by server")
+                self.clipboard_menuitem.set_tooltip_text("Clipboard synchronization cannot be enabled: disabled by server")
         self.client.connect("handshake-complete", set_clipboard_menuitem)
         def clipboard_toggled(*args):
             #keep menu in sync with actual "clipboard_enabled" flag:
@@ -460,7 +470,7 @@ class GTKTrayMenuBase(object):
 
     def make_translatedclipboard_optionsmenuitem(self):
         clipboard_menu = self.menuitem("Clipboard", "clipboard.png", "Choose which remote clipboard to connect to", None)
-        clipboard_menu.set_sensitive(False)
+        set_sensitive(clipboard_menu, False)
         def set_clipboard_menu(*args):
             clipboard_submenu = gtk.Menu()
             clipboard_menu.set_submenu(clipboard_submenu)
@@ -468,7 +478,7 @@ class GTKTrayMenuBase(object):
             c = self.client
             can_clipboard = c.server_supports_clipboard and c.client_supports_clipboard and c.server_supports_clipboard
             log("set_clipboard_menu(%s) can_clipboard=%s, server=%s, client=%s", args, can_clipboard, c.server_supports_clipboard, c.client_supports_clipboard)
-            clipboard_menu.set_sensitive(can_clipboard)
+            set_sensitive(clipboard_menu, can_clipboard)
             LABEL_TO_NAME = {"Disabled"  : None,
                             "Clipboard" : "CLIPBOARD",
                             "Primary"   : "PRIMARY",
@@ -506,7 +516,7 @@ class GTKTrayMenuBase(object):
                 active = isinstance(self.client.clipboard_helper, TranslatedClipboardProtocolHelper) \
                             and self.client.clipboard_helper.remote_clipboard==remote_clipboard
                 clipboard_item.set_active(active)
-                clipboard_item.set_sensitive(can_clipboard)
+                set_sensitive(clipboard_item, can_clipboard)
                 clipboard_item.set_draw_as_radio(True)
                 clipboard_item.connect("toggled", remote_clipboard_changed)
                 clipboard_submenu.append(clipboard_item)
@@ -528,24 +538,24 @@ class GTKTrayMenuBase(object):
     def make_keyboardsyncmenuitem(self):
         def set_keyboard_sync_tooltip():
             if not self.client.keyboard_helper:
-                set_tooltip_text(self.keyboard_sync_menuitem, "Keyboard support is not loaded")
+                self.keyboard_sync_menuitem.set_tooltip_text("Keyboard support is not loaded")
             elif not self.client.toggle_keyboard_sync:
-                set_tooltip_text(self.keyboard_sync_menuitem, "This server does not support changes to keyboard synchronization")
+                self.keyboard_sync_menuitem.set_tooltip_text("This server does not support changes to keyboard synchronization")
             elif self.client.keyboard_helper.keyboard_sync:
-                set_tooltip_text(self.keyboard_sync_menuitem, "Disable keyboard synchronization (prevents spurious key repeats on high latency connections)")
+                self.keyboard_sync_menuitem.set_tooltip_text("Disable keyboard synchronization (prevents spurious key repeats on high latency connections)")
             else:
-                set_tooltip_text(self.keyboard_sync_menuitem, "Enable keyboard state synchronization")
+                self.keyboard_sync_menuitem.set_tooltip_text("Enable keyboard state synchronization")
         def keyboard_sync_toggled(*args):
             self.client.keyboard_sync = self.keyboard_sync_menuitem.get_active()
             log("keyboard_sync_toggled(%s) keyboard_sync=%s", args, self.client.keyboard_sync)
             set_keyboard_sync_tooltip()
             self.client.emit("keyboard-sync-toggled")
         self.keyboard_sync_menuitem = self.checkitem("Keyboard Synchronization", keyboard_sync_toggled)
-        self.keyboard_sync_menuitem.set_sensitive(False)
+        set_sensitive(self.keyboard_sync_menuitem, False)
         def set_keyboard_sync_menuitem(*args):
             log("set_keyboard_sync_menuitem%s enabled=%s", args, self.client.keyboard_helper.keyboard_sync)
             self.keyboard_sync_menuitem.set_active(self.client.keyboard_helper.keyboard_sync)
-            self.keyboard_sync_menuitem.set_sensitive(self.client.toggle_keyboard_sync)
+            set_sensitive(self.keyboard_sync_menuitem, self.client.toggle_keyboard_sync)
             set_keyboard_sync_tooltip()
         self.client.connect("handshake-complete", set_keyboard_sync_menuitem)
         return self.keyboard_sync_menuitem
@@ -555,9 +565,10 @@ class GTKTrayMenuBase(object):
         def gl_set(*args):
             log("gl_set(%s) opengl_enabled=%s, window_unmap=%s", args, self.client.opengl_enabled, self.client.window_unmap)
             gl.set_active(self.client.opengl_enabled)
-            gl.set_sensitive(self.client.window_unmap)
+            can_gl = self.client.window_unmap and self.client.client_supports_opengl
+            set_sensitive(gl, can_gl)
             if not self.client.window_unmap:
-                set_tooltip_text(gl, "no server support for runtime switching")
+                gl.set_tooltip_text("no server support for runtime switching")
                 return
             def opengl_toggled(*args):
                 log("opengl_toggled%s", args)
@@ -568,14 +579,14 @@ class GTKTrayMenuBase(object):
 
     def make_encodingsmenuitem(self):
         encodings = self.menuitem("Encoding", "encoding.png", "Choose picture data encoding", None)
-        encodings.set_sensitive(False)
+        set_sensitive(encodings, False)
         def set_encodingsmenuitem(*args):
             log("set_encodingsmenuitem%s", args)
-            encodings.set_sensitive(not self.client.mmap_enabled)
+            set_sensitive(encodings, not self.client.mmap_enabled)
             if self.client.mmap_enabled:
                 #mmap disables encoding and uses raw rgb24
                 encodings.set_label("Encoding")
-                set_tooltip_text(encodings, "memory mapped transfers are in use so picture encoding is disabled")
+                encodings.set_tooltip_text("memory mapped transfers are in use so picture encoding is disabled")
             else:
                 encodings.set_submenu(self.make_encodingssubmenu())
         self.client.connect("handshake-complete", set_encodingsmenuitem)
@@ -606,12 +617,12 @@ class GTKTrayMenuBase(object):
                 active = encoding==self.client.encoding
                 if active!=x.get_active():
                     x.set_active(active)
-                x.set_sensitive(encoding in self.client.server_encodings)
+                set_sensitive(x, encoding in self.client.server_encodings)
 
 
     def make_qualitymenuitem(self):
         self.quality = self.menuitem("Quality", "slider.png", "Picture quality", None)
-        self.quality.set_sensitive(False)
+        set_sensitive(self.quality, False)
         def may_enable_qualitymenu(*args):
             self.quality.set_submenu(self.make_qualitysubmenu())
             self.set_qualitymenu()
@@ -643,21 +654,21 @@ class GTKTrayMenuBase(object):
     def set_qualitymenu(self, *args):
         if self.quality:
             can_use = not self.client.mmap_enabled and self.client.encoding in self.client.server_encodings_with_quality
-            self.quality.set_sensitive(can_use)
+            set_sensitive(self.quality, can_use)
             if not can_use:
-                set_tooltip_text(self.quality, "Not supported with %s encoding" % self.client.encoding)
+                self.quality.set_tooltip_text("Not supported with %s encoding" % self.client.encoding)
                 return
-            set_tooltip_text(self.quality, "Minimum picture quality")
+            self.quality.set_tooltip_text("Minimum picture quality")
             #now check if lossless is supported:
             if self.quality.get_submenu():
                 can_lossless = self.client.encoding in self.client.server_encodings_with_lossless_mode
                 for q,item in self.quality.get_submenu().menu_items.items():
-                    item.set_sensitive(q<100 or can_lossless)
+                    set_sensitive(item, q<100 or can_lossless)
 
 
     def make_speedmenuitem(self):
         self.speed = self.menuitem("Speed", "speed.png", "Encoding latency vs size", None)
-        self.speed.set_sensitive(False)
+        set_sensitive(self.speed, False)
         def may_enable_speedmenu(*args):
             self.speed.set_submenu(self.make_speedsubmenu())
             self.set_speedmenu()
@@ -689,15 +700,15 @@ class GTKTrayMenuBase(object):
     def set_speedmenu(self, *args):
         if self.speed:
             can_use = not self.client.mmap_enabled and self.client.encoding in self.client.server_encodings_with_speed and self.client.change_speed
-            self.speed.set_sensitive(can_use)
+            set_sensitive(self.speed, can_use)
             if self.client.mmap_enabled:
-                set_tooltip_text(self.speed, "Quality is always 100% with mmap")
+                self.speed.set_tooltip_text("Quality is always 100% with mmap")
             elif not self.client.change_speed:
-                set_tooltip_text(self.speed, "Server does not support changing speed")
+                self.speed.set_tooltip_text("Server does not support changing speed")
             elif self.client.encoding!="h264":
-                set_tooltip_text(self.speed, "Not supported with %s encoding" % self.client.encoding)
+                self.speed.set_tooltip_text("Not supported with %s encoding" % self.client.encoding)
             else:
-                set_tooltip_text(self.speed, "Encoding latency vs size")
+                self.speed.set_tooltip_text("Encoding latency vs size")
 
 
     def spk_on(self, *args):
@@ -708,15 +719,19 @@ class GTKTrayMenuBase(object):
         self.client.stop_receiving_sound()
     def make_speakermenuitem(self):
         speaker = self.menuitem("Speaker", "speaker.png", "Forward sound output from the server")
-        speaker.set_sensitive(False)
+        set_sensitive(speaker, False)
         def is_speaker_on(*args):
             return self.client.speaker_enabled
         def speaker_state(*args):
-            if not self.client.server_sound_send:
-                speaker.set_sensitive(False)
-                set_tooltip_text(speaker, "Server does not support speaker forwarding")
+            if not self.client.speaker_allowed:
+                set_sensitive(speaker, False)
+                speaker.set_tooltip_text("Speaker forwarding has been disabled")
                 return
-            speaker.set_sensitive(True)
+            if not self.client.server_sound_send:
+                set_sensitive(speaker, False)
+                speaker.set_tooltip_text("Server does not support speaker forwarding")
+                return
+            set_sensitive(speaker, True)
             speaker.set_submenu(self.make_soundsubmenu(is_speaker_on, self.spk_on, self.spk_off, "speaker-changed"))
         self.client.connect("handshake-complete", speaker_state)
         return speaker
@@ -729,15 +744,19 @@ class GTKTrayMenuBase(object):
         self.client.stop_sending_sound()
     def make_microphonemenuitem(self):
         microphone = self.menuitem("Microphone", "microphone.png", "Forward sound input to the server", None)
-        microphone.set_sensitive(False)
+        set_sensitive(microphone, False)
         def is_microphone_on(*args):
             return self.client.microphone_enabled
         def microphone_state(*args):
-            if not self.client.server_sound_receive:
-                microphone.set_sensitive(False)
-                set_tooltip_text(microphone, "Server does not support microphone forwarding")
+            if not self.client.microphone_allowed:
+                set_sensitive(microphone, False)
+                microphone.set_tooltip_text("Microphone forwarding has been disabled")
                 return
-            microphone.set_sensitive(True)
+            if not self.client.server_sound_receive:
+                set_sensitive(microphone, False)
+                microphone.set_tooltip_text("Server does not support microphone forwarding")
+                return
+            set_sensitive(microphone, True)
             microphone.set_submenu(self.make_soundsubmenu(is_microphone_on, self.mic_on, self.mic_off, "microphone-changed"))
         self.client.connect("handshake-complete", microphone_state)
         return microphone
@@ -785,7 +804,7 @@ class GTKTrayMenuBase(object):
 
     def make_layoutsmenuitem(self):
         keyboard = self.menuitem("Keyboard", "keyboard.png", "Select your keyboard layout", None)
-        keyboard.set_sensitive(False)
+        set_sensitive(keyboard, False)
         self.layout_submenu = gtk.Menu()
         keyboard.set_submenu(self.layout_submenu)
         self.popup_menu_workaround(self.layout_submenu)
@@ -868,13 +887,13 @@ class GTKTrayMenuBase(object):
                 #so no need to show the user the huge layout list
                 keyboard.hide()
                 return
-            keyboard.set_sensitive(True)
+            set_sensitive(keyboard, True)
         self.client.connect("handshake-complete", set_layout_enabled)
         return keyboard
 
     def make_compressionmenu(self):
         self.compression = self.menuitem("Compression", "compressed.png", "Network packet compression", None)
-        self.compression.set_sensitive(False)
+        set_sensitive(self.compression, False)
         self.compression_submenu = gtk.Menu()
         self.compression.set_submenu(self.compression_submenu)
         self.popup_menu_workaround(self.compression_submenu)
@@ -892,7 +911,7 @@ class GTKTrayMenuBase(object):
             c.connect('activate', set_compression)
             self.compression_submenu.append(c)
         def enable_compressionmenu(self):
-            self.compression.set_sensitive(True)
+            set_sensitive(self.compression, True)
             self.compression_submenu.show_all()
         self.client.connect("handshake-complete", enable_compressionmenu)
         return self.compression
@@ -911,6 +930,19 @@ class GTKTrayMenuBase(object):
                     win.present()
         return self.handshake_menuitem("Raise Windows", "raise.png", None, raise_windows)
 
+    def make_startnewcommandmenuitem(self, handshake_done=False):
+        self.startnewcommand = self.menuitem("Run Command", "forward.png", "Run a new command on the server", self.client.show_start_new_command)
+        def enable_start_new_command(*args):
+            log("enable_start_new_command%s start_new_command=%s", args, self.client.start_new_commands)
+            set_sensitive(self.startnewcommand, self.client.start_new_commands)
+            if not self.client.start_new_commands:
+                self.startnewcommand.set_tooltip_text("Not supported by the server")
+        if not handshake_done:
+            self.client.connect("handshake-complete", enable_start_new_command)
+        else:
+            enable_start_new_command()
+        return self.startnewcommand
+
     def make_disconnectmenuitem(self):
         def menu_quit(*args):
             self.client.disconnect_and_quit(EXIT_OK, CLIENT_EXIT)
