@@ -14,12 +14,13 @@ import time
 import datetime
 
 from xpra.os_util import os_info, bytestostr
+from xpra.util import prettify_plug_name
 from xpra.gtk_common.graph import make_graph_pixmap
-from xpra.deque import maxdeque
+from collections import deque
 from xpra.simple_stats import values_to_scaled_values, values_to_diff_scaled_values, to_std_unit, std_unit_dec
 from xpra.scripts.config import python_platform
 from xpra.log import Logger
-from xpra.gtk_common.gtk_util import add_close_accel, label, title_box, set_tooltip_text, \
+from xpra.gtk_common.gtk_util import add_close_accel, label, title_box, \
                         TableBuilder, imagebutton, scaled_image, get_preferred_size, get_gtk_version_info, \
                         RELIEF_NONE, RELIEF_NORMAL, EXPAND, FILL, WIN_POS_CENTER
 from xpra.net.protocol import get_network_caps
@@ -27,6 +28,7 @@ log = Logger("info")
 
 N_SAMPLES = 20      #how many sample points to show on the graphs
 SHOW_PIXEL_STATS = True
+SHOW_SOUND_STATS = True
 
 
 def pixelstr(v):
@@ -153,19 +155,24 @@ class SessionInfo(gtk.Window):
         def client_vinfo(prop, fallback="unknown"):
             k = "%s.version" % prop
             return label(make_version_str(gtk_version_info.get(k, fallback)))
-        tb.new_row("Glib",      client_vinfo("glib"),       label(server_version_info("glib.version")))
-        tb.new_row("Gobject",   client_vinfo("gobject"),    label(server_version_info("gobject.version", "pygtk_version")))
-        tb.new_row("PyGTK",     client_vinfo("pygtk", ""),  label(server_version_info("pygtk.version", "pygtk_version")))
-        tb.new_row("GTK",       client_vinfo("gtk"),        label(server_version_info("gtk.version", "gtk_version")))
-        tb.new_row("GDK",       client_vinfo("gdk"),        label(server_version_info("gdk.version", "gdk_version")))
-        tb.new_row("Cairo",     client_vinfo("cairo"),      label(server_version_info("cairo.version", "cairo_version")))
-        tb.new_row("Pango",     client_vinfo("pango"),      label(server_version_info("pango.version", "cairo_version")))
+        def server_vinfo(prop):
+            k = "%s.version" % prop
+            fk = "%s_version" % prop
+            return label(server_version_info(k, fk))
+        tb.new_row("Glib",      client_vinfo("glib"),       server_vinfo("glib"))
+        tb.new_row("PyGlib",    client_vinfo("pyglib"),     server_vinfo("pyglib"))
+        tb.new_row("Gobject",   client_vinfo("gobject"),    server_vinfo("gobject"))
+        tb.new_row("PyGTK",     client_vinfo("pygtk", ""),  server_vinfo("pygtk"))
+        tb.new_row("GTK",       client_vinfo("gtk"),        server_vinfo("gtk"))
+        tb.new_row("GDK",       client_vinfo("gdk"),        server_vinfo("gdk"))
+        tb.new_row("Cairo",     client_vinfo("cairo"),      server_vinfo("cairo"))
+        tb.new_row("Pango",     client_vinfo("pango"),      server_vinfo("pango"))
         tb.new_row("Python", label(python_platform.python_version()), label(server_version_info("server.python.version", "python_version", "python.version")))
 
         cl_gst_v, cl_pygst_v = "", ""
         try:
             from xpra.sound.gstreamer_util import gst_version as cl_gst_v, pygst_version as cl_pygst_v
-        except Exception, e:
+        except Exception as e:
             log("cannot load gstreamer: %s", e)
         tb.new_row("GStreamer", label(make_version_str(cl_gst_v)), label(server_version_info("sound.gst.version", "gst_version")))
         tb.new_row("pygst", label(make_version_str(cl_pygst_v)), label(server_version_info("sound.pygst.version", "pygst_version")))
@@ -253,7 +260,7 @@ class SessionInfo(gtk.Window):
         tb, _ = self.table_tab("connect.png", "Connection", self.populate_connection)
         tb.new_row("Server Endpoint", label(self.connection.target))
         if self.client.server_display:
-            tb.new_row("Server Display", label(self.client.server_display))
+            tb.new_row("Server Display", label(prettify_plug_name(self.client.server_display)))
         hostname = scaps.strget("hostname")
         if hostname:
             tb.new_row("Server Hostname", label(hostname))
@@ -274,7 +281,7 @@ class SessionInfo(gtk.Window):
         self.output_bytes_label = label()
         tb.new_row("Bytes Sent", self.output_bytes_label)
         self.compression_label = label()
-        tb.new_row("Compression + Encoding", self.compression_label)
+        tb.new_row("Encoding + Compression", self.compression_label)
         self.connection_type_label = label()
         tb.new_row("Connection Type", self.connection_type_label)
         self.input_encryption_label = label()
@@ -368,9 +375,11 @@ class SessionInfo(gtk.Window):
             self.bandwidth_graph = self.add_graph_button(bandwidth_label, self.save_graphs)
             self.connect("realize", self.populate_graphs)
             self.latency_graph = self.add_graph_button(None, self.save_graphs)
-        self.pixel_in_data = maxdeque(N_SAMPLES+4)
-        self.net_in_bytecount = maxdeque(N_SAMPLES+4)
-        self.net_out_bytecount = maxdeque(N_SAMPLES+4)
+        self.pixel_in_data = deque(maxlen=N_SAMPLES+4)
+        self.net_in_bytecount = deque(maxlen=N_SAMPLES+4)
+        self.net_out_bytecount = deque(maxlen=N_SAMPLES+4)
+        self.sound_in_bytecount = deque(maxlen=N_SAMPLES+4)
+        self.sound_out_bytecount = deque(maxlen=N_SAMPLES+4)
 
         self.set_border_width(15)
         self.add(self.tab_box)
@@ -472,7 +481,7 @@ class SessionInfo(gtk.Window):
         button.connect("button_press_event", click_cb)
         button.add(graph)
         if tooltip:
-            set_tooltip_text(graph, tooltip)
+            graph.set_tooltip_text(tooltip)
         self.graph_box.add(button)
         return graph
 
@@ -491,9 +500,11 @@ class SessionInfo(gtk.Window):
         #record bytecount every second:
         self.net_in_bytecount.append(self.connection.input_bytecount)
         self.net_out_bytecount.append(self.connection.output_bytecount)
-        #pre-compute for graph:
-        self.net_in_scale, self.net_in_data = values_to_diff_scaled_values(list(self.net_in_bytecount)[1:N_SAMPLES+3], scale_unit=1000, min_scaled_value=50)
-        self.net_out_scale, self.net_out_data = values_to_diff_scaled_values(list(self.net_out_bytecount)[1:N_SAMPLES+3], scale_unit=1000, min_scaled_value=50)
+        if SHOW_SOUND_STATS and self.client.sound_sink:
+            if self.client.sound_in_bytecount>0:
+                self.sound_in_bytecount.append(self.client.sound_in_bytecount)
+            if self.client.sound_out_bytecount>0:
+                self.sound_out_bytecount.append(self.client.sound_out_bytecount)
 
         #count pixels in the last second:
         since = time.time()-1
@@ -555,11 +566,11 @@ class SessionInfo(gtk.Window):
         return not self.is_closed
 
     def init_counters(self):
-        self.avg_batch_delay = maxdeque(N_SAMPLES+4)
-        self.avg_damage_out_latency = maxdeque(N_SAMPLES+4)
-        self.avg_ping_latency = maxdeque(N_SAMPLES+4)
-        self.avg_decoding_latency = maxdeque(N_SAMPLES+4)
-        self.avg_total = maxdeque(N_SAMPLES+4)
+        self.avg_batch_delay = deque(maxlen=N_SAMPLES+4)
+        self.avg_damage_out_latency = deque(maxlen=N_SAMPLES+4)
+        self.avg_ping_latency = deque(maxlen=N_SAMPLES+4)
+        self.avg_decoding_latency = deque(maxlen=N_SAMPLES+4)
+        self.avg_total = deque(maxlen=N_SAMPLES+4)
 
     def populate_tab(self, *args):
         if self.is_closed:
@@ -594,7 +605,7 @@ class SessionInfo(gtk.Window):
             else:
                 buffering = "unknown"
         else:
-            glinfo = self.client.opengl_props.get("info", "")
+            glinfo = self.client.opengl_props.get("info", "disabled")
         self.client_opengl_label.set_text(glinfo)
         self.opengl_buffering.set_text(buffering)
 
@@ -613,6 +624,12 @@ class SessionInfo(gtk.Window):
             if state!="active":
                 return state
             s = "%s: %s" % (state, sound_pipeline.codec_description)
+            try:
+                pid = sound_pipeline.get_info().get("pid")
+                if pid:
+                    s += " (pid=%s)" % pid
+            except:
+                pass
             #if sound_pipeline.bitrate>0:
             #    s += " / %sbit/s" % std_unit(sound_pipeline.bitrate)
             return s
@@ -623,13 +640,13 @@ class SessionInfo(gtk.Window):
         def populate_speaker_info(*args):
             can = scaps.boolget("sound.send", False) and self.client.speaker_allowed
             self.bool_icon(self.server_speaker_icon, can)
-            self.speaker_codec_label.set_text(pipeline_info(can, self.client.sound_sink))
+            self.speaker_codec_label.set_text(bytestostr(pipeline_info(can, self.client.sound_sink)))
         populate_speaker_info()
         self.client.connect("speaker-changed", populate_speaker_info)
         def populate_microphone_info(*args):
             can = scaps.boolget("sound.receive", False) and self.client.microphone_allowed
             self.bool_icon(self.server_microphone_icon, can)
-            self.microphone_codec_label.set_text(pipeline_info(can, self.client.sound_source))
+            self.microphone_codec_label.set_text(bytestostr(pipeline_info(can, self.client.sound_source)))
         populate_microphone_info()
         self.client.connect("microphone-changed", populate_microphone_info)
 
@@ -687,9 +704,9 @@ class SessionInfo(gtk.Window):
             p = get_sound_info(supported, prop)
             label.set_text(p.get("state", ""))
             if details:
-                d = p.get("queue.used_pct", -1)
+                d = p.get("queue.used", -1)
                 if d>=0:
-                    details.set_text(" (buffer: %s%%)" % str(d).rjust(3))
+                    details.set_text(" (buffer: %sms)" % str(d).rjust(3))
                 else:
                     details.set_text("")
         set_sound_info(self.speaker_label, self.speaker_details, self.client.speaker_enabled, self.client.sound_sink)
@@ -725,8 +742,8 @@ class SessionInfo(gtk.Window):
             return ""
         altv = ""
         if alt:
-            altv = self.client.server_last_info.get(alt+"."+suffix, "")
-        return self.client.server_last_info.get(prefix+"."+suffix, altv)
+            altv = self.client.server_last_info.get((alt+"."+suffix).encode(), "")
+        return self.client.server_last_info.get((prefix+"."+suffix).encode(), altv)
 
     def values_from_info(self, prefix, alt=None):
         def getv(suffix):
@@ -872,10 +889,10 @@ class SessionInfo(gtk.Window):
                             pass
                 #print("window_encoder_stats=%s" % window_encoder_stats)
                 for wid, props in window_encoder_stats.items():
-                    l = label("%s (%s)" % (wid, props.get("")))
+                    l = label("%s (%s)" % (wid, bytestostr(props.get(""))))
                     l.show()
                     info = ["%s=%s" % (k,v) for k,v in props.items() if k!=""]
-                    set_tooltip_text(l, " ".join(info))
+                    l.set_tooltip_text(" ".join(info))
                     self.encoder_info_box.add(l)
         return True
 
@@ -892,7 +909,8 @@ class SessionInfo(gtk.Window):
         h = max(200, h-bh-20, rect.height-bh-20)
         w = max(360, rect.width-20)
         #bandwidth graph:
-        if self.net_in_data and self.net_out_data:
+        labels, datasets = [], []
+        if self.net_in_bytecount and self.net_out_bytecount:
             def unit(scale):
                 if scale==1:
                     return ""
@@ -901,12 +919,24 @@ class SessionInfo(gtk.Window):
                     if value==1:
                         return str(unit)
                     return "x%s%s" % (int(value), unit)
-            labels = ["recv %sB/s" % unit(self.net_in_scale), "sent %sB/s" % unit(self.net_out_scale)]
-            datasets = [self.net_in_data, self.net_out_data]
-            if SHOW_PIXEL_STATS and self.client.windows_enabled:
-                pixel_scale, in_pixels = values_to_scaled_values(list(self.pixel_in_data)[3:N_SAMPLES+4], min_scaled_value=100)
-                datasets.append(in_pixels)
-                labels.append("%s pixels/s" % unit(pixel_scale))
+            net_in_scale, net_in_data = values_to_diff_scaled_values(list(self.net_in_bytecount)[1:N_SAMPLES+3], scale_unit=1000, min_scaled_value=50)
+            net_out_scale, net_out_data = values_to_diff_scaled_values(list(self.net_out_bytecount)[1:N_SAMPLES+3], scale_unit=1000, min_scaled_value=50)
+            labels += ["recv %sB/s" % unit(net_in_scale), "sent %sB/s" % unit(net_out_scale)]
+            datasets += [net_in_data, net_out_data]
+        if SHOW_PIXEL_STATS and self.client.windows_enabled:
+            pixel_scale, in_pixels = values_to_scaled_values(list(self.pixel_in_data)[3:N_SAMPLES+4], min_scaled_value=100)
+            datasets.append(in_pixels)
+            labels.append("%s pixels/s" % unit(pixel_scale))
+        if SHOW_SOUND_STATS and self.sound_in_bytecount:
+            sound_in_scale, sound_in_data =  values_to_diff_scaled_values(list(self.sound_in_bytecount)[1:N_SAMPLES+3], scale_unit=1000, min_scaled_value=50)
+            datasets.append(sound_in_data)
+            labels.append("Speaker %sB/s" % unit(sound_in_scale))
+        if SHOW_SOUND_STATS and self.sound_out_bytecount:
+            sound_out_scale, sound_out_data =  values_to_diff_scaled_values(list(self.sound_out_bytecount)[1:N_SAMPLES+3], scale_unit=1000, min_scaled_value=50)
+            datasets.append(sound_out_data)
+            labels.append("Mic %sB/s" % unit(sound_out_scale))
+
+        if labels and datasets:
             pixmap = make_graph_pixmap(datasets, labels=labels,
                                        width=w, height=h/2,
                                        title="Bandwidth", min_y_scale=10, rounding=10,
