@@ -12,7 +12,7 @@ gtk = import_gtk()
 gdk = import_gdk()
 
 from xpra.client.tray_base import TrayBase, log
-from xpra.gtk_common.gtk_util import set_tooltip_text, get_icon_from_file, get_pixbuf_from_data, INTERP_HYPER
+from xpra.gtk_common.gtk_util import get_icon_from_file, get_pixbuf_from_data, INTERP_HYPER
 
 ORIENTATION = {}
 if not is_gtk3():
@@ -28,7 +28,7 @@ class GTKStatusIconTray(TrayBase):
     def __init__(self, *args):
         TrayBase.__init__(self, *args)
         self.tray_widget = gtk.StatusIcon()
-        set_tooltip_text(self.tray_widget, self.tooltip or "Xpra")
+        self.tray_widget.set_tooltip_text(self.tooltip or "Xpra")
         self.tray_widget.connect('activate', self.activate_menu)
         self.tray_widget.connect('popup-menu', self.popup_menu)
         if self.size_changed_cb:
@@ -60,14 +60,18 @@ class GTKStatusIconTray(TrayBase):
 
     def hide(self, *args):
         log("%s.set_visible(False)", self.tray_widget)
-        self.tray_widget.set_visible(False)
+        if self.tray_widget:
+            self.tray_widget.set_visible(False)
 
     def show(self, *args):
         log("%s.set_visible(True)", self.tray_widget)
-        self.tray_widget.set_visible(True)
+        if self.tray_widget:
+            self.tray_widget.set_visible(True)
 
 
     def get_screen(self):
+        if not self.tray_widget:
+            return -1
         ag = self.tray_widget.get_geometry()
         if ag is None:
             return -1
@@ -77,6 +81,8 @@ class GTKStatusIconTray(TrayBase):
         return screen.get_number()
 
     def get_orientation(self):
+        if not self.tray_widget:
+            return None
         ag = self.tray_widget.get_geometry()
         if ag is None:
             return None
@@ -84,6 +90,7 @@ class GTKStatusIconTray(TrayBase):
         return ORIENTATION.get(gtk_orientation)
 
     def get_geometry(self):
+        assert self.tray_widget
         ag = self.tray_widget.get_geometry()
         log("GTKStatusIconTray.get_geometry() %s.get_geometry()=%s", self.tray_widget, ag)
         if ag is None:
@@ -101,10 +108,11 @@ class GTKStatusIconTray(TrayBase):
 
 
     def set_tooltip(self, text=None):
-        set_tooltip_text(self.tray_widget, text or "Xpra")
+        if self.tray_widget:
+            self.tray_widget.set_tooltip_text(text or "Xpra")
 
     def set_blinking(self, on):
-        if hasattr(self.tray_widget, "set_blinking"):
+        if self.tray_widget and hasattr(self.tray_widget, "set_blinking"):
             self.tray_widget.set_blinking(on)
 
 
@@ -117,8 +125,7 @@ class GTKStatusIconTray(TrayBase):
         self.set_icon_from_pixbuf(tray_icon)
 
     def set_icon_from_pixbuf(self, tray_icon):
-        assert self.tray_widget
-        if not tray_icon:
+        if not tray_icon or not self.tray_widget:
             return
         tw, th = self.get_geometry()[2:]
         w = tray_icon.get_width()
