@@ -8,7 +8,7 @@ import os.path
 
 from xpra.platform.paths import get_icon_dir
 from xpra.log import Logger
-from xpra.deque import maxdeque
+from collections import deque
 log = Logger("tray")
 
 
@@ -30,7 +30,7 @@ class TrayBase(object):
         self.default_icon_name = "xpra.png"
         #some implementations need this for guessing the geometry (see recalculate_geometry):
         self.geometry_guess = None
-        self.tray_event_locations = maxdeque(512)
+        self.tray_event_locations = deque(maxlen=512)
 
     def cleanup(self):
         if self.tray_widget:
@@ -90,7 +90,7 @@ class TrayBase(object):
             with_ext = "%s.%s" % (basefilename, self.default_icon_extension)
             icon_dir = get_icon_dir()
             filename = os.path.join(icon_dir, with_ext)
-        if not os.path.exists(filename):
+        if not filename or not os.path.exists(filename):
             log.error("could not find icon '%s' for name '%s'", filename, basefilename)
             return
         abspath = os.path.abspath(filename)
