@@ -1,35 +1,28 @@
 # This file is part of Xpra.
 # Copyright (C) 2008 Nathaniel Smith <njs@pobox.com>
-# Copyright (C) 2012-2014 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2012-2015 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
 import os
 from xpra.log import Logger
 log = Logger("paint")
+deltalog = Logger("delta")
 
 from threading import Lock
 from xpra.net.mmap_pipe import mmap_read
 from xpra.net import compression
 from xpra.util import typedict
-from xpra.codecs.codec_constants import get_colorspace_from_avutil_enum, get_PIL_decodings
+from xpra.codecs.codec_constants import get_PIL_decodings
 from xpra.codecs.loader import get_codec
 from xpra.codecs.video_helper import getVideoHelper
-from xpra.os_util import BytesIOClass, bytestostr, builtins
-_memoryview = builtins.__dict__.get("memoryview")
-try:
-    from xpra.codecs.xor import xor_str
-except:
-    xor_str = None
-#for alpha unpremultiply utility function
-#(subclasses should ensure argb is present before calling it!)
-WEBP_PILLOW = os.environ.get("XPRA_WEBP_PILLOW", "0")=="1"
-try:
-    from xpra.codecs.argb.argb import unpremultiply_argb, unpremultiply_argb_in_place, byte_buffer_to_buffer   #@UnresolvedImport
-except:
-    log.warn("argb module is missing, cannot support alpha channels")
-    unpremultiply_argb, unpremultiply_argb_in_place, byte_buffer_to_buffer  = None, None, None
+from xpra.os_util import BytesIOClass, bytestostr, _buffer
+from xpra.codecs.xor.cyxor import xor_str   #@UnresolvedImport
+from xpra.codecs.argb.argb import unpremultiply_argb, unpremultiply_argb_in_place   #@UnresolvedImport
 
+DELTA_BUCKETS = int(os.environ.get("XPRA_DELTA_BUCKETS", "5"))
+
+WEBP_PILLOW = os.environ.get("XPRA_WEBP_PILLOW", "0")=="1"
 PIL = get_codec("PIL")
 
 #ie:
@@ -85,7 +78,7 @@ class WindowBackingBase(object):
         self.idle_add = idle_add
         self._alpha_enabled = window_alpha
         self._backing = None
-        self._last_pixmap_data = None
+        self._delta_pixel_data = [None for _ in range(DELTA_BUCKETS)]
         self._video_decoder = None
         self._csc_decoder = None
         self._decoder_lock = Lock()
@@ -157,17 +150,15 @@ class WindowBackingBase(object):
         return csc_modes
 
 
-    def img_data_tobytes(self, img_data):
-        if _memoryview and isinstance(img_data, _memoryview):
-            return img_data.tobytes()
-        return img_data
-
     def unpremultiply(self, img_data):
-        try:
-            unpremultiply_argb_in_place(img_data)
-            return img_data
-        except:
-            return byte_buffer_to_buffer(unpremultiply_argb(img_data))
+        if type(img_data) not in (str, _buffer):
+            try:
+                unpremultiply_argb_in_place(img_data)
+                return img_data
+            except:
+                log.warn("failed to unpremultiply %s (len=%s)" % (type(img_data), len(img_data)))
+        return unpremultiply_argb(img_data)
+
 
     def process_delta(self, raw_data, width, height, rowstride, options):
         """
@@ -186,19 +177,21 @@ class WindowBackingBase(object):
             raise Exception("expected %s bytes for %sx%s with rowstride=%s but received %s (%s compressed)" %
                                 (rowstride * height, width, height, rowstride, len(img_data), len(raw_data)))
         delta = options.intget("delta", -1)
+        bucket = options.intget("bucket", 0)
         rgb_data = img_data
         if delta>=0:
-            if not self._last_pixmap_data:
-                raise Exception("delta region references pixmap data we do not have!")
-            if xor_str is None:
-                raise Exception("received a delta region but we do not support delta encoding!")
-            lwidth, lheight, store, ldata = self._last_pixmap_data
-            assert width==lwidth and height==lheight and delta==store
+            assert bucket>=0 and bucket<DELTA_BUCKETS, "invalid delta bucket number: %s" % bucket
+            if self._delta_pixel_data[bucket] is None:
+                raise Exception("delta region bucket %s references pixmap data we do not have!" % bucket)
+            lwidth, lheight, seq, ldata = self._delta_pixel_data[bucket]
+            assert width==lwidth and height==lheight and delta==seq, "delta bucket %s data does not match: expected %s but got %s" % (bucket, (width, height, delta), (lwidth, lheight, seq))
+            deltalog("delta: xoring with bucket %i", bucket)
             rgb_data = xor_str(img_data, ldata)
         #store new pixels for next delta:
         store = options.intget("store", -1)
         if store>=0:
-            self._last_pixmap_data =  width, height, store, rgb_data
+            deltalog("delta: storing sequence %i in bucket %i", store, bucket)
+            self._delta_pixel_data[bucket] =  width, height, store, rgb_data
         return rgb_data
 
 
@@ -355,8 +348,7 @@ class WindowBackingBase(object):
     def paint_with_video_decoder(self, decoder_module, coding, img_data, x, y, width, height, options, callbacks):
         #log("paint_with_video_decoder%s", (decoder_module, coding, "%s bytes" % len(img_data), x, y, width, height, options, callbacks))
         assert decoder_module, "decoder module not found for %s" % coding
-        try:
-            self._decoder_lock.acquire()
+        with self._decoder_lock:
             if self._backing is None:
                 log("window %s is already gone!", self.wid)
                 fire_paint_callbacks(callbacks, False)
@@ -364,15 +356,9 @@ class WindowBackingBase(object):
             enc_width, enc_height = options.intpair("scaled_size", (width, height))
             input_colorspace = options.strget("csc")
             if not input_colorspace:
-                # Backwards compatibility with pre 0.10.x clients
-                # We used to specify the colorspace as an avutil PixelFormat constant
-                old_csc_fmt = options.intget("csc_pixel_format")
-                input_colorspace = get_colorspace_from_avutil_enum(old_csc_fmt)
-                if input_colorspace is None:
-                    #completely broken and out of date servers (ie: v0.3.x):
-                    log("csc was not specified and we cannot find a colorspace from csc_pixel_format=%s, assuming it is an old server and using YUV420P", old_csc_fmt)
-                    input_colorspace = "YUV420P"
-
+                log.error("csc mode is missing from the video options!")
+                fire_paint_callbacks(callbacks, False)
+                return  False
             #do we need a prep step for decoders that cannot handle the input_colorspace directly?
             decoder_colorspaces = decoder_module.get_input_colorspaces(coding)
             assert input_colorspace in decoder_colorspaces, "decoder does not support %s for %s" % (input_colorspace, coding)
@@ -385,7 +371,11 @@ class WindowBackingBase(object):
                     log("paint_with_video_decoder: window dimensions have changed from %s to %s", (self._video_decoder.get_width(), self._video_decoder.get_height()), (enc_width, enc_height))
                     self.do_clean_video_decoder()
                 elif self._video_decoder.get_colorspace()!=input_colorspace:
-                    log("paint_with_video_decoder: colorspace changed from %s to %s", self._video_decoder.get_colorspace(), input_colorspace)
+                    #this should only happen on encoder restart, which means this should be the first frame:
+                    l = log
+                    if options.get("frame", 0)>1:
+                        l = log.warn
+                    l("paint_with_video_decoder: colorspace changed from %s to %s", self._video_decoder.get_colorspace(), input_colorspace)
                     self.do_clean_video_decoder()
                 elif options.get("frame")==0:
                     log("paint_with_video_decoder: first frame of new stream")
@@ -403,10 +393,8 @@ class WindowBackingBase(object):
                       self.wid, coding, len(img_data), width, height, self._video_decoder, options)
                 return False
             self.do_video_paint(img, x, y, enc_width, enc_height, width, height, options, callbacks)
-        finally:
-            self._decoder_lock.release()
-            if self._backing is None:
-                self.close_decoder(True)
+        if self._backing is None:
+            self.close_decoder(True)
         return  False
 
     def do_video_paint(self, img, x, y, enc_width, enc_height, width, height, options, callbacks):
@@ -481,18 +469,25 @@ class WindowBackingBase(object):
     def draw_region(self, x, y, width, height, coding, img_data, rowstride, options, callbacks):
         """ dispatches the paint to one of the paint_XXXX methods """
         log("draw_region(%s, %s, %s, %s, %s, %s bytes, %s, %s, %s)", x, y, width, height, coding, len(img_data), rowstride, options, callbacks)
-        img_data = self.img_data_tobytes(img_data)
         coding = bytestostr(coding)
+        options["encoding"] = coding            #used for choosing the color of the paint box
         if coding == "mmap":
             self.idle_add(self.paint_mmap, img_data, x, y, width, height, rowstride, options, callbacks)
-        elif coding == "rgb24":
+        elif coding == "rgb24" or coding == "rgb32":
+            #avoid confusion over how many bytes-per-pixel we may have:
+            rgb_format = options.get("rgb_format")
+            if rgb_format:
+                Bpp = len(rgb_format)
+            elif coding=="rgb24":
+                Bpp = 3
+            else:
+                Bpp = 4
             if rowstride==0:
-                rowstride = width * 3
-            self.paint_rgb24(img_data, x, y, width, height, rowstride, options, callbacks)
-        elif coding == "rgb32":
-            if rowstride==0:
-                rowstride = width * 4
-            self.paint_rgb32(img_data, x, y, width, height, rowstride, options, callbacks)
+                rowstride = width * Bpp
+            if Bpp==3:
+                self.paint_rgb24(img_data, x, y, width, height, rowstride, options, callbacks)
+            else:
+                self.paint_rgb32(img_data, x, y, width, height, rowstride, options, callbacks)
         elif coding in VIDEO_DECODERS:
             self.paint_with_video_decoder(VIDEO_DECODERS.get(coding), coding, img_data, x, y, width, height, options, callbacks)
         elif coding == "webp":
