@@ -21,14 +21,13 @@ def do_get_PIL_codings(PIL, attr="SAVE"):
     if attr=="OPEN":
         try:
             assert pi.PILLOW_VERSION>='2.8', "version %s leaks memory with webp" % pi.PILLOW_VERSION
-        except Exception, e:
+        except Exception as e:
             log("Pillow support for opening webp images has been disabled: %s", e)
             test_options.remove("webp")
     for encoding in test_options:
         #strip suffix (so "png/L" -> "png")
         stripped = encoding.split("/")[0].upper()
         if stripped in avalue:
-            log("PIL.Image can %s %s", attr, encoding)
             encodings.append(encoding)
     log("do_get_PIL_codings(%s, %s)=%s", PIL, attr, encodings)
     return encodings
@@ -59,23 +58,6 @@ def get_subsampling_divs(pixel_format):
     return PIXEL_SUBSAMPLING.get(pixel_format)
 
 
-AVUTIL_ENUM_TO_COLORSPACE =  {
-            0   : "YUV420P",
-            2   : "RGB",
-            3   : "BGR",
-            4   : "YUV422P",
-            5   : "YUV444P"}
-COLORSPACE_TO_AVUTIL_ENUM = {}
-for e, s in AVUTIL_ENUM_TO_COLORSPACE.items():
-    COLORSPACE_TO_AVUTIL_ENUM[s] = e
-
-def get_colorspace_from_avutil_enum(pixfmt):
-    return AVUTIL_ENUM_TO_COLORSPACE.get(pixfmt)
-
-def get_avutil_enum_from_colorspace(pixfmt):
-    return COLORSPACE_TO_AVUTIL_ENUM.get(pixfmt)
-
-
 RGB_FORMATS = ("XRGB",
                "BGRX",
                "ARGB",
@@ -83,15 +65,6 @@ RGB_FORMATS = ("XRGB",
                "RGB")
 
 
-def get_default_csc_modes(encoding_client_options=True):
-    if not encoding_client_options:
-        #very old clients can only use 420P:
-        return ("YUV420P", )
-    #default for newer clients that don't specify "csc_modes":
-    #(0.10 onwards should have specified csc_modes)
-    return ("YUV420P", "YUV422P", "YUV444P")
-
-
 class TransientCodecException(Exception):
     pass
 
@@ -110,7 +83,7 @@ class codec_spec(object):
                     score_boost=0,
                     width_mask=0xFFFF, height_mask=0xFFFF):
         self.codec_class = codec_class          #ie: xpra.codecs.enc_x264.encoder.Encoder
-        self.codec_type = codec_type            #ie: "x264"
+        self.codec_type = codec_type            #ie: "nvenc"
         self.quality = quality
         self.speed = speed
         self.setup_cost = setup_cost
@@ -173,9 +146,6 @@ class codec_spec(object):
             return max(0, 1.0 - (1.0*ic/mi)**2)
         return 1.0
 
-    def __str__(self):
-        return "codec_spec(%s)" % self.__dict__
-
     def __repr__(self):
         return "codec_spec(%s)" % self.info()
 
@@ -204,10 +174,11 @@ class codec_spec(object):
 
 class video_codec_spec(codec_spec):
 
-    def __init__(self, encoding=None, output_colorspaces=None, **kwargs):
+    def __init__(self, encoding=None, output_colorspaces=None, has_lossless_mode=False, **kwargs):
         codec_spec.__init__(self, **kwargs)
         self.encoding = encoding                        #ie: "h264"
         self.output_colorspaces = output_colorspaces    #ie: ["YUV420P" : "YUV420P", ...]
+        self.has_lossless_mode = has_lossless_mode
         self._exported_fields += ["encoding", "output_colorspaces"]
 
 
