@@ -10,10 +10,11 @@ import time
 import os
 import warnings
 import numpy
-assert bytearray
 import pyopencl             #@UnresolvedImport
 from pyopencl import mem_flags  #@UnresolvedImport
 
+from xpra.os_util import memoryview_to_bytes, _memoryview
+
 PREFERRED_DEVICE_TYPE = os.environ.get("XPRA_OPENCL_DEVICE_TYPE", "GPU")
 PREFERRED_DEVICE_NAME = os.environ.get("XPRA_OPENCL_DEVICE_NAME", "")
 PREFERRED_DEVICE_PLATFORM = os.environ.get("XPRA_OPENCL_PLATFORM", "")
@@ -33,7 +34,7 @@ def dimdiv(dim, div):
     #when we divide a dimensions by the subsampling
     #we want to round up so as to include the last
     #pixel when we hit odd dimensions
-    return roundup(dim/div, div)
+    return roundup(dim//div, div)
 
 def device_type(d):
     try:
@@ -101,7 +102,6 @@ def select_device():
     log_platforms_info()
     #try to choose a platform and device using *our* heuristics / env options:
     options = {}
-    log("PREFERRED_DEVICE_NAME=%s, dtype==PREFERRED_DEVICE_TYPE=%s, PREFERRED_DEVICE_PLATFORM=%s", PREFERRED_DEVICE_NAME, PREFERRED_DEVICE_TYPE, PREFERRED_DEVICE_PLATFORM)
     log("select_device() environment preferred DEVICE_NAME=%s, DEVICE_TYPE=%s, DEVICE_PLATFORM=%s", PREFERRED_DEVICE_NAME, PREFERRED_DEVICE_TYPE, PREFERRED_DEVICE_PLATFORM)
     for platform in opencl_platforms:
         log("evaluating platform=%s", platform.name)
@@ -113,7 +113,6 @@ def select_device():
         is_cuda = platform.name.find("CUDA")>=0
         for d in devices:
             if d.available and d.compiler_available and d.get_info(pyopencl.device_info.IMAGE_SUPPORT):
-                log("evaluating platform=%s", platform.name)
                 if not is_supported(platform.name) and (len(PREFERRED_DEVICE_PLATFORM)==0 or str(platform.name).find(PREFERRED_DEVICE_PLATFORM)<0):
                     log("ignoring unsupported platform/device: %s / %s", platform.name, d.name)
                     continue
@@ -163,7 +162,7 @@ def select_device():
                     selected_device_setup_cost = 20
                 log("device is a %s, using CPU cost=%s, GPU cost=%s", device_type(d), selected_device_cpu_cost, selected_device_gpu_cost)
                 return
-            except Exception, e:
+            except Exception as e:
                 log.warn(" failed to use %s", platform_info(p))
                 log.warn(" with %s device %s", device_type(d), device_info(d))
                 log.warn(" Error: %s", e)
@@ -438,7 +437,7 @@ def build_kernels():
             program = pyopencl.Program(context, "\n".join(NAMES_TO_KERNELS.values()))
             program.build()
             dump_warnings(log.debug)
-        except Exception, e:
+        except Exception as e:
             log.error("cannot build the OpenCL program: %s", e, exc_info=True)
             dump_warnings(log.warn)
             raise ImportError("cannot build the OpenCL program: %s" % e)
@@ -643,15 +642,15 @@ class ColorspaceConverter(object):
         maxw_w, maxw_h = selected_device.max_work_item_sizes[:2]
         while local_w*local_h>selected_device.max_work_group_size or local_w>maxw_w or local_h>maxw_h:
             if local_w>maxw_w:
-                local_w /= 2
+                local_w //= 2
             if local_h>maxw_h:
-                local_h /= 2
+                local_h //= 2
             if local_w*local_h>selected_device.max_work_group_size:
                 #prefer h<w for local work:
                 if local_h>=local_w:
-                    local_h /= 2
+                    local_h //= 2
                 else:
-                    local_w /= 2
+                    local_w //= 2
         globalWorkSize = (roundup(wwidth, local_w), roundup(wheight, local_h))
         localWorkSize = local_w, local_h
         return globalWorkSize, localWorkSize
@@ -677,7 +676,7 @@ class ColorspaceConverter(object):
             self.init_with_device()
         try:
             return self.do_convert_image(image)
-        except pyopencl.LogicError, e:
+        except pyopencl.LogicError as e:
             if retry>0:
                 raise e
             log.warn("OpenCL error: %s", e)
@@ -717,11 +716,13 @@ class ColorspaceConverter(object):
         for i in range(3):
             _, y_div = divs[i]
             plane = pixels[i]
+            if type(plane)==_memoryview:
+                plane = memoryview_to_bytes(plane)
             if type(plane)==str:
                 flags = mem_flags.READ_ONLY | mem_flags.COPY_HOST_PTR
             else:
                 flags = mem_flags.READ_ONLY | mem_flags.USE_HOST_PTR
-            shape = strides[i], self.src_height/y_div
+            shape = strides[i], self.src_height//y_div
             iimage = pyopencl.Image(self.context, flags, iformat, shape=shape, hostbuf=plane)
             input_images.append(iimage)
 
@@ -774,9 +775,12 @@ class ColorspaceConverter(object):
 
         #input image:
         iformat = pyopencl.ImageFormat(self.channel_order, pyopencl.channel_type.UNSIGNED_INT8)
-        shape = (stride/4, self.src_height)
-        log("convert_image() input image format=%s, shape=%s, work size: local=%s, global=%s", iformat, shape, localWorkSize, globalWorkSize)
-        if type(pixels)==str:
+        shape = (stride//4, self.src_height)
+        log("convert_image() type=%s, input image format=%s, shape=%s, work size: local=%s, global=%s", type(pixels), iformat, shape, localWorkSize, globalWorkSize)
+        idata = pixels
+        if type(idata)==_memoryview:
+            idata = memoryview_to_bytes(idata)
+        if type(idata)==str:
             #str is not a buffer, so we have to copy the data
             #alternatively, we could copy it first ourselves using this:
             #pixels = numpy.fromstring(pixels, dtype=numpy.byte).data
@@ -784,7 +788,7 @@ class ColorspaceConverter(object):
             flags = mem_flags.READ_ONLY | mem_flags.COPY_HOST_PTR
         else:
             flags = mem_flags.READ_ONLY | mem_flags.USE_HOST_PTR
-        iimage = pyopencl.Image(self.context, flags, iformat, shape=shape, hostbuf=pixels)
+        iimage = pyopencl.Image(self.context, flags, iformat, shape=shape, hostbuf=idata)
 
         kernelargs = [self.queue, globalWorkSize, localWorkSize,
                       iimage, numpy.int32(self.src_width), numpy.int32(self.src_height),
@@ -797,8 +801,8 @@ class ColorspaceConverter(object):
         out_sizes = []
         for i in range(3):
             x_div, y_div = divs[i]
-            p_stride = roundup(self.dst_width / x_div, max(2, localWorkSize[0]))
-            p_height = roundup(self.dst_height / y_div, 2)
+            p_stride = roundup(self.dst_width // x_div, max(2, localWorkSize[0]))
+            p_height = roundup(self.dst_height // y_div, 2)
             p_size = p_stride * p_height
             #log("output buffer for channel %s: stride=%s, height=%s, size=%s", i, p_stride, p_height, p_size)
             out_buf = pyopencl.Buffer(self.context, mem_flags.WRITE_ONLY, p_size)
