@@ -12,7 +12,6 @@ log = Logger("csc", "cuda")
 
 import os
 import time
-assert bytearray
 import pycuda
 from pycuda import driver
 from pycuda import tools
@@ -36,6 +35,8 @@ def record_device_success(device_id):
 
 
 def device_info(d):
+    if not d:
+        return "None"
     return "%s @ %s" % (d.name(), d.pci_bus_id())
 
 def get_pycuda_version():
@@ -51,14 +52,20 @@ def get_pycuda_info():
             "driver.driver_version" : driver.get_driver_version()}
 
 
+DEVICE_INFO = {}
+def get_device_info(i):
+    global DEVICE_INFO
+    return DEVICE_INFO.get(i, None)
+
 DEVICES = None
 def init_all_devices():
-    global DEVICES
+    global DEVICES, DEVICE_INFO
     if DEVICES is not None:
         return  DEVICES
     log.info("CUDA initialization (this may take a few seconds)")
     driver.init()
     DEVICES = []
+    DEVICE_INFO = {}
     log("CUDA driver version=%s", driver.get_driver_version())
     ngpus = driver.Device.count()
     log.info("CUDA %s / PyCUDA %s, found %s device(s):", ".".join([str(x) for x in driver.get_version()]), pycuda.VERSION_TEXT, ngpus)
@@ -70,6 +77,7 @@ def init_all_devices():
         try:
             device = driver.Device(i)
             log(" + testing device %s: %s", i, device_info(device))
+            DEVICE_INFO[i] = device_info(device)
             host_mem = device.get_attribute(da.CAN_MAP_HOST_MEMORY)
             if not host_mem:
                 log.warn("skipping device %s (cannot map host memory)", device_info(device))
@@ -94,10 +102,14 @@ def init_all_devices():
                 log.info("  + %s (memory: %s%% free, compute: %s.%s)", device_info(device), 100*free/total, SMmajor, SMminor)
             finally:
                 context.pop()
-        except Exception, e:
+        except Exception as e:
             log.error("error on device %s: %s", (device or i), e)
     return DEVICES
 
+def get_devices():
+    global DEVICES
+    return DEVICES
+
 def check_devices():
     devices = init_all_devices()
     assert len(devices)>0, "no valid CUDA devices found!"
@@ -138,7 +150,10 @@ def select_device(preferred_device_id=-1, preferred_device_name=CUDA_DEVICE_NAME
                 if compute<min_compute:
                     log("ignoring device %s: compute capability %#x (minimum %#x required)", device_info(device), compute, min_compute)
                 elif device_id==preferred_device_id:
-                    log.info("device matches preferred device id %s: %s", preferred_device_id, device_info(device))
+                    l = log
+                    if len(device_list)>1:
+                        l = log.info
+                    l("device matches preferred device id %s: %s", preferred_device_id, device_info(device))
                     return device_id, device
                 elif preferred_device_name and device_info(device).find(preferred_device_name)>=0:
                     log("device matches preferred device name: %s", preferred_device_name)
@@ -152,72 +167,47 @@ def select_device(preferred_device_id=-1, preferred_device_name=CUDA_DEVICE_NAME
                     context.pop()
                     context.detach()
         if selected_device_id>=0 and selected_device:
-            log("select device: %s / %s", device_id, device)
+            l = log
+            if len(devices)>1:
+                l = log.info
+            l("selected device %s: %s", device_id, device_info(device))
             return selected_device_id, selected_device
     return -1, None
 
 
-#cache pre-compiled kernel cubins per device:
-KERNEL_cubins = {}
-def get_CUDA_function(device_id, function_name, kernel_source):
+#cache kernel fatbin files:
+KERNELS = {}
+def get_CUDA_function(device_id, function_name):
     """
         Returns the compiled kernel for the given device
         and kernel key.
-        Kernels may be pre-compiled with compile_all.
     """
-    global KERNEL_cubins
-    cubin = KERNEL_cubins.get((device_id, function_name))
-    if cubin is None:
-        start = time.time()
-        log("compiling for device %s: %s=%s", device_id, function_name, kernel_source)
-        cubin = compile(kernel_source)
-        KERNEL_cubins[(device_id, function_name)] = cubin
-        end = time.time()
-        log("compilation of %s took %.1fms", function_name, 1000.0*(end-start))
+    global KERNELS
+    data = KERNELS.get(function_name)
+    if data is None:
+        from xpra.platform.paths import default_get_app_dir
+        from xpra.os_util import load_binary_file
+        cubin_file = os.path.join(default_get_app_dir(), "cuda", "%s.fatbin" % function_name)
+        log("get_CUDA_function(%s, %s) cubin file=%s", device_id, function_name, cubin_file)
+        data = load_binary_file(cubin_file)
+        if not data:
+            log.error("failed to load CUDA bin file %s", cubin_file)
+            return None
+        log(" loaded %s bytes", len(data))
+        KERNELS[function_name] = data
     #now load from cubin:
     start = time.time()
-    mod = driver.module_from_buffer(cubin)
-    CUDA_function = mod.get_function(function_name)
+    mod = driver.module_from_buffer(data)
+    log("get_CUDA_function(%s, %s) module=%s", device_id, function_name, mod)
+    try:
+        CUDA_function = mod.get_function(function_name)
+    except driver.LogicError as e:
+        raise Exception("failed to load '%s' from %s: %s" % (function_name, mod, e))
     end = time.time()
     log("loading function %s from pre-compiled cubin took %.1fms", function_name, 1000.0*(end-start))
     return CUDA_function
 
 
-def recompile_all(function_name, kernel_src, device_ids=None):
-    global KERNEL_cubins
-    KERNEL_cubins = {}
-    tools.clear_context_caches()
-    compile_all(function_name, kernel_src, device_ids)
-
-def compile_all(function_name, kernel_src, device_ids=None):
-    """
-        Pre-compiles kernel source on the given devices,
-        so we can then call get_CUDA_function quickly
-        to get the function to call.
-    """
-    global KERNEL_cubins
-    if device_ids is None:
-        device_ids = init_all_devices()
-    cf = driver.ctx_flags
-    for device_id in device_ids:
-        device = None
-        context = None
-        try:
-            device = driver.Device(device_id)
-            context = device.make_context(flags=cf.SCHED_YIELD | cf.MAP_HOST)
-            cubin = KERNEL_cubins.get((device_id, function_name))
-            if cubin is None:
-                start = time.time()
-                log("compiling for device %s: %s=%s", device_id, function_name, kernel_src)
-                cubin = compile(kernel_src)
-                end = time.time()
-                log("compilation of %s took %.1fms", function_name, 1000.0*(end-start))
-                KERNEL_cubins[(device_id, function_name)] = cubin
-        finally:
-            if context:
-                context.pop()
-
-
 def main():
     import sys
     if "-v" in sys.argv or "--verbose" in sys.argv:
