@@ -94,17 +94,21 @@ class ImageWrapper(object):
     def set_pixels(self, pixels):
         self.pixels = pixels
 
+    def allocate_buffer(self, buf_len, free_existing=1):
+        #only defined for XImage wrappers:
+        return 0
+
     def clone_pixel_data(self):
-        if not self.freed:
-            if self.planes == 0:
-                #no planes, simple buffer:
-                assert self.pixels, "no pixels!"
-                self.pixels = self.pixels[:]
-            else:
-                assert self.planes>0
-                for i in range(self.planes):
-                    self.pixels[i] = self.pixels[i][:]
-            self.thread_safe = True
+        assert not self.freed, "image has already been freed!"
+        if self.planes == 0:
+            #no planes, simple buffer:
+            assert self.pixels, "no pixels!"
+            self.pixels = self.pixels[:]
+        else:
+            assert self.planes>0
+            for i in range(self.planes):
+                self.pixels[i] = self.pixels[i][:]
+        self.thread_safe = True
 
     def __del__(self):
         #print("ImageWrapper.__del__() calling %s" % self.free)
