@@ -14,7 +14,7 @@ if sys.version > '3':
 
 #these codecs may well not load because we
 #do not require the libraries to be installed
-NOWARN = ["opencl"]
+NOWARN = ["nvenc3", "nvenc4", "nvenc5", "opencl"]
 
 codec_errors = {}
 codecs = {}
@@ -24,31 +24,40 @@ def codec_import_check(name, description, top_module, class_module, *classnames)
     try:
         try:
             __import__(top_module, {}, {}, [])
-        except ImportError, e:
+        except ImportError as e:
             log(" cannot import %s (%s): %s", name, description, e)
             log("", exc_info=True)
             codec_errors[name] = e
             return None
+    except Exception as e:
+        log.warn(" cannot load %s (%s):", name, description, exc_info=True)
+        codec_errors[name] = e
+        return None
+    try:
         #module is present
         try:
             log(" %s found, will check for %s in %s", top_module, classnames, class_module)
             for classname in classnames:
                 ic =  __import__(class_module, {}, {}, classname)
+                selftest = getattr(ic, "selftest", None)
+                log("%s.selftest=%s", name, selftest)
+                if selftest:
+                    selftest()
                 #log.warn("codec_import_check(%s, ..)=%s" % (name, ic))
                 log(" found %s : %s", name, ic)
                 codecs[name] = ic
                 return ic
-        except ImportError, e:
+        except ImportError as e:
             codec_errors[name] = e
             l = log.warn
             if name in NOWARN:
                 l = log.debug
             l(" cannot import %s (%s): %s", name, description, e)
             log("", exc_info=True)
-    except Exception, e:
+    except Exception as e:
         codec_errors[name] = e
         log.warn(" cannot load %s (%s): %s missing from %s: %s", name, description, classname, class_module, e)
-        log("", exc_info=True)
+        log.warn("", exc_info=True)
     return None
 codec_versions = {}
 def add_codec_version(name, top_module, version="get_version()", alt_version="__version__"):
@@ -69,21 +78,43 @@ def add_codec_version(name, top_module, version="get_version()", alt_version="__
             #optional info:
             if hasattr(module, "get_info"):
                 info = getattr(module, "get_info")
-                log(" %s info(%s)=%s", name, top_module, info())
+                log(" %s %s.%s=%s", name, top_module, info, info())
             return v
         if name in codecs:
             log.warn(" cannot find %s in %s", " or ".join(fieldnames), module)
         else:
             log(" no version information for missing codec %s", name)
-    except ImportError, e:
+    except ImportError as e:
         #not present
         log(" cannot import %s: %s", name, e)
         log("", exc_info=True)
-    except Exception, e:
+    except Exception as e:
         log.warn("error during codec import: %s", e)
         log("", exc_info=True)
     return None
 
+def PIL_logging_workaround():
+    import logging
+    PIL_DEBUG = os.environ.get("XPRA_PIL_DEBUG", "0")=="1"
+    if PIL_DEBUG:
+        from xpra.log import Logger
+        log = Logger("util")
+        log.info("enabling PIL.DEBUG")
+        level = logging.DEBUG
+    else:
+        level = logging.INFO
+
+    #newer versions use this logger,
+    #we must initialize it before we load the class:
+    for x in ("Image", "PngImagePlugin", "WebPImagePlugin", "JpegImagePlugin"):
+        logger = logging.getLogger("PIL.%s" % x)
+        logger.setLevel(level)
+    import PIL
+    from PIL import Image
+    assert PIL and Image
+    if hasattr(Image, "DEBUG"):
+        #for older versions (pre 3.0), use Image.DEBUG flag:
+        Image.DEBUG = int(PIL_DEBUG)
 
 loaded = False
 def load_codecs():
@@ -92,6 +123,10 @@ def load_codecs():
         return
     loaded = True
     log("loading codecs")
+    try:
+        PIL_logging_workaround()
+    except:
+        log("error in PIL logging workaround", exc_info=True)
     codec_import_check("PIL", "Python Imaging Library", "PIL", "PIL", "Image")
     add_codec_version("PIL", "PIL.Image", "PILLOW_VERSION", "VERSION")
 
@@ -105,6 +140,10 @@ def load_codecs():
     codec_import_check("enc_x265", "x265 encoder", "xpra.codecs.enc_x265", "xpra.codecs.enc_x265.encoder", "Encoder")
     add_codec_version("x265", "xpra.codecs.enc_x265.encoder")
 
+    for v in (4, 3, 5):
+        codec_import_check("nvenc%s" % v, "nvenc encoder", "xpra.codecs.nvenc%s" % v, "xpra.codecs.nvenc%s.encoder" % v, "Encoder")
+        add_codec_version("nvenc%s" % v, "xpra.codecs.nvenc%s.encoder" % v)
+
     codec_import_check("csc_swscale", "swscale colorspace conversion", "xpra.codecs.csc_swscale", "xpra.codecs.csc_swscale.colorspace_converter", "ColorspaceConverter")
     add_codec_version("swscale", "xpra.codecs.csc_swscale.colorspace_converter")
 
@@ -114,15 +153,9 @@ def load_codecs():
     codec_import_check("csc_opencl", "OpenCL colorspace conversion", "xpra.codecs.csc_opencl", "xpra.codecs.csc_opencl.colorspace_converter", "ColorspaceConverter")
     add_codec_version("opencl", "xpra.codecs.csc_opencl.colorspace_converter")
 
-    #ffmpeg v1:
-    codec_import_check("dec_avcodec", "avcodec decoder", "xpra.codecs.dec_avcodec", "xpra.codecs.dec_avcodec.decoder", "Decoder")
-    add_codec_version("avcodec", "xpra.codecs.dec_avcodec.decoder")
-
-    #ffmpeg v2:
     codec_import_check("dec_avcodec2", "avcodec2 decoder", "xpra.codecs.dec_avcodec2", "xpra.codecs.dec_avcodec2.decoder", "Decoder")
     add_codec_version("avcodec2", "xpra.codecs.dec_avcodec2.decoder")
 
-    #webp via cython:
     codec_import_check("enc_webp", "webp encoder", "xpra.codecs.webp", "xpra.codecs.webp.encode", "compress")
     add_codec_version("enc_webp", "xpra.codecs.webp.encode")
 
@@ -158,19 +191,17 @@ def has_codec(name):
     return name in codecs
 
 OLD_ENCODING_NAMES_TO_NEW = {"x264" : "h264", "vpx" : "vp8"}
-NEW_ENCODING_NAMES_TO_OLD = {"h264" : "x264", "vp8" : "vpx"}
 ALL_OLD_ENCODING_NAMES_TO_NEW = {"x264" : "h264", "vpx" : "vp8", "rgb24" : "rgb"}
-ALL_NEW_ENCODING_NAMES_TO_OLD = {"h264" : "x264", "vp8" : "vpx", "rgb" : "rgb24"}
 
-ALL_CODECS = "PIL", "enc_vpx", "dec_vpx", "enc_x264", "enc_x265", \
+ALL_CODECS = "PIL", "enc_vpx", "dec_vpx", "enc_x264", "enc_x265", "nvenc3", "nvenc4", "nvenc5", \
             "csc_swscale", "csc_cython", "csc_opencl", \
-            "dec_avcodec", "dec_avcodec2", \
+            "dec_avcodec2", \
             "enc_webp", \
             "dec_webp"
 
 #note: this is just for defining the order of encodings,
 #so we have both core encodings (rgb24/rgb32) and regular encodings (rgb) in here:
-PREFERED_ENCODING_ORDER = ["h264", "vp8", "png", "png/P", "png/L", "webp", "rgb", "rgb24", "rgb32", "jpeg", "h265", "vp9"]
+PREFERED_ENCODING_ORDER = ["h264", "vp9", "vp8", "png", "png/P", "png/L", "webp", "rgb", "rgb24", "rgb32", "jpeg", "h265"]
 #encoding order for edges (usually one pixel high or wide):
 EDGE_ENCODING_ORDER = ["rgb24", "rgb32", "jpeg", "png", "webp", "png/P", "png/L", "rgb"]
 
@@ -196,7 +227,7 @@ ENCODINGS_HELP = {
       "h264"    : "H.264 video codec",
       "h265"    : "H.265 (HEVC) video codec (slow and buggy - do not use!)",
       "vp8"     : "VP8 video codec",
-      "vp9"     : "VP9 video codec (very slow - do not use!)",
+      "vp9"     : "VP9 video codec",
       "png"     : "Portable Network Graphics (lossless, 24bpp or 32bpp for transparency)",
       "png/P"   : "Portable Network Graphics (lossy, 8bpp colour)",
       "png/L"   : "Portable Network Graphics (lossy, 8bpp grayscale)",
@@ -208,7 +239,7 @@ ENCODINGS_HELP = {
 HELP_ORDER = ("h264", "h265", "vp8", "vp9", "png", "png/P", "png/L", "webp", "rgb", "jpeg")
 
 #those are currently so useless that we don't want the user to select them by mistake
-PROBLEMATIC_ENCODINGS = ("h265", "vp9")
+PROBLEMATIC_ENCODINGS = ("h265", )
 
 
 def encodings_help(encodings):
@@ -262,8 +293,12 @@ def main():
                     elif name.find("enc")>=0 or name.find("dec")>=0:
                         encodings = mod.get_encodings()
                         print("                         %s" % ", ".join(encodings))
-                except:
-                    e = sys.exc_info()[1]
+                    try:
+                        i = mod.get_info()
+                        print("                         %s" % i)
+                    except:
+                        pass
+                except Exception as e:
                     print("error getting extra information on %s: %s" % (name, e))
         print("")
         print("codecs versions:")
