@@ -15,14 +15,15 @@ from xpra.codecs.loader import get_codec, get_codec_error, load_codecs
 
 #the codec loader uses the names...
 #but we need the module name to be able to probe without loading the codec:
-CODEC_TO_MODULE = {"vpx"        : "vpx",
-                   "x264"       : "enc_x264",
-                   "x265"       : "enc_x265",
-                   "swscale"    : "csc_swscale",
-                   "cython"     : "csc_cython",
-                   "opencl"     : "csc_opencl",
-                   "avcodec"    : "dec_avcodec",
-                   "avcodec2"   : "dec_avcodec2"}
+CODEC_TO_MODULE = {"vpx"        : ["vpx"],
+                   "x264"       : ["enc_x264"],
+                   "x265"       : ["enc_x265"],
+                   "nvenc"      : ["nvenc4", "nvenc3", "nvenc5"],
+                   "swscale"    : ["csc_swscale"],
+                   "cython"     : ["csc_cython"],
+                   "opencl"     : ["csc_opencl"],
+                   "avcodec"    : ["dec_avcodec"],
+                   "avcodec2"   : ["dec_avcodec2"]}
 
 def has_codec_module(module_name):
     top_module = "xpra.codecs.%s" % module_name
@@ -30,26 +31,28 @@ def has_codec_module(module_name):
         __import__(top_module, {}, {}, [])
         log("codec module %s is installed", module_name)
         return True
-    except Exception, e:
+    except Exception as e:
         log("codec module %s cannot be loaded: %s", module_name, e)
         return False
 
 def try_import_modules(codec_names):
     names = []
     for codec_name in codec_names:
-        module_name = CODEC_TO_MODULE[codec_name]
-        if has_codec_module(module_name):
-            names.append(codec_name)
+        module_names = CODEC_TO_MODULE[codec_name]
+        for module_name in module_names:
+            if has_codec_module(module_name):
+                names.append(codec_name)
+                break
     return names
 
 #all the codecs we know about:
 #try to import the module that contains them (cheap check):
-ALL_VIDEO_ENCODER_OPTIONS = try_import_modules(["x264", "vpx", "x265"])
+ALL_VIDEO_ENCODER_OPTIONS = try_import_modules(["x264", "vpx", "x265", "nvenc"])
 ALL_CSC_MODULE_OPTIONS = try_import_modules(["swscale", "cython", "opencl"])
 NO_GFX_CSC_OPTIONS = [x for x in ALL_CSC_MODULE_OPTIONS if x not in ("opencl", )]
 ALL_VIDEO_DECODER_OPTIONS = try_import_modules(["avcodec", "avcodec2", "vpx"])
 
-PREFERRED_ENCODER_ORDER = ["x264", "vpx", "x265"]
+PREFERRED_ENCODER_ORDER = ["nvenc", "x264", "vpx", "x265"]
 PREFERRED_DECODER_ORDER = ["avcodec", "avcodec2", "vpx"]
 log("video_helper: ALL_VIDEO_ENCODER_OPTIONS=%s", ALL_VIDEO_ENCODER_OPTIONS)
 log("video_helper: ALL_CSC_MODULE_OPTIONS=%s", ALL_CSC_MODULE_OPTIONS)
@@ -60,10 +63,12 @@ log("video_helper: ALL_VIDEO_DECODER_OPTIONS=%s", ALL_VIDEO_DECODER_OPTIONS)
 #on top of that, there are compatibility problems with gtk at times: OpenCL AMD and TLS don't mix well
 
 
-def get_encoder_module_name(x):
-        if x.find("enc")>=0:
-            return x            #ie: "enc_vpx"
-        return "enc_"+x         #ie: "enc_x264"
+def get_encoder_module_names(x):
+    if x=="nvenc":
+        return ["nvenc4", "nvenc3", "nvenc5"]
+    elif x.find("enc")>=0:
+        return [x]              #ie: "nvenc" or "enc_vpx"
+    return ["enc_"+x]           #ie: "enc_x264"
 
 def get_decoder_module_name(x):
         return "dec_"+x         #ie: "dec_vpx"
@@ -77,10 +82,12 @@ def get_DEFAULT_VIDEO_ENCODERS():
     """ returns all the video encoders installed """
     encoders = []
     for x in list(ALL_VIDEO_ENCODER_OPTIONS):
-        mod = get_encoder_module_name(x)
-        c = get_codec(mod)
-        if c:
-            encoders.append(x)
+        mods = get_encoder_module_names(x)
+        for mod in mods:
+            c = get_codec(mod)
+            if c:
+                encoders.append(x)
+                break
     return encoders
 
 def get_DEFAULT_CSC_MODULES():
@@ -141,8 +148,7 @@ class VideoHelper(object):
             video_encoders, csc_modules, video_decoders, self.video_encoders, self.csc_modules, self.video_decoders)
 
     def cleanup(self):
-        try:
-            self._lock.acquire()
+        with self._lock:
             #check again with lock held (in case of race):
             if not self._initialized:
                 return
@@ -159,8 +165,6 @@ class VideoHelper(object):
             self.csc_modules = []
             self.video_decoders = []
             self._initialized = False
-        finally:
-            self._lock.release()
 
     def clone(self):
         if not self._initialized:
@@ -184,15 +188,15 @@ class VideoHelper(object):
         for encoding, encoder_specs in self._video_encoder_specs.items():
             for in_csc, specs in encoder_specs.items():
                 for spec in specs:
-                    d.setdefault("encoding."+in_csc+"_to_"+encoding, []).append(spec.codec_type)
+                    d.setdefault("encoding.%s_to_%s" % (in_csc, encoding), []).append(spec.codec_type)
         for in_csc, specs in self._csc_encoder_specs.items():
             for out_csc, specs in specs.items():
-                d["csc."+in_csc+"_to_"+out_csc] = [spec.codec_type for spec in specs]
+                d["csc.%s_to_%s" % (in_csc, out_csc)] = [spec.codec_type for spec in specs]
         for encoding, decoder_specs in self._video_decoder_specs.items():
             for out_csc, decoders in decoder_specs.items():
                 for decoder in decoders:
                     decoder_name, _ = decoder
-                    d.setdefault("decoding."+encoding+"_to_"+out_csc, []).append(decoder_name)
+                    d.setdefault("decoding.%s_to_%s" % (encoding, out_csc), []).append(decoder_name)
         def modstatus(x, def_list, active_list):
             #the module is present
             if x in active_list:
@@ -210,8 +214,7 @@ class VideoHelper(object):
     def init(self):
         log("VideoHelper.init()")
         load_codecs()
-        try:
-            self._lock.acquire()
+        with self._lock:
             #check again with lock held (in case of race):
             log("VideoHelper.init() initialized=%s", self._initialized)
             if self._initialized:
@@ -220,8 +223,6 @@ class VideoHelper(object):
             self.init_csc_options()
             self.init_video_decoders_options()
             self._initialized = True
-        finally:
-            self._lock.release()
         log("VideoHelper.init() done")
 
     def get_encodings(self):
@@ -248,10 +249,16 @@ class VideoHelper(object):
         log("init_video_encoders_options() will try video encoders: %s", self.video_encoders)
         for x in self.video_encoders:
             try:
-                mod = get_encoder_module_name(x)
-                self.init_video_encoder_option(mod)
-            except:
-                log.warn("init_video_encoders_options() cannot add %s encoder", x, exc_info=True)
+                mods = get_encoder_module_names(x)
+                log("init_video_encoders_options() modules for %s: %s", x, mods)
+                for mod in mods:
+                    try:
+                        self.init_video_encoder_option(mod)
+                        break
+                    except:
+                        log.warn("init_video_encoders_options() cannot add %s encoder", mod, exc_info=True)
+            except Exception as e:
+                log.warn("init_video_encoders_options() cannot add %s encoder: %s", x, e)
         log("init_video_encoders_options() video encoder specs: %s", self._video_encoder_specs)
 
     def init_video_encoder_option(self, encoder_name):
@@ -265,17 +272,15 @@ class VideoHelper(object):
         try:
             encoder_module.init_module()
             self._cleanup_modules.append(encoder_module)
-        except Exception, e:
-            log("exception in %s module initialization %s: %s", encoder_type, encoder_module.init_module, e, exc_info=True)
+        except Exception as e:
+            log("exception in %s module %s initialization %s: %s", encoder_type, encoder_module.__name__, encoder_module.init_module, e, exc_info=True)
             log.warn("Warning: %s video encoder failed: %s", encoder_type, e)
-            return
-        colorspaces = encoder_module.get_input_colorspaces()
-        log("init_video_encoder_option(%s) %s input colorspaces=%s", encoder_module, encoder_type, colorspaces)
+            raise e
         encodings = encoder_module.get_encodings()
         log("init_video_encoder_option(%s) %s encodings=%s", encoder_module, encoder_type, encodings)
         for encoding in encodings:
-            #we assume that all encodings support all colorspaces here...
-            #which may not be the case in the future!
+            colorspaces = encoder_module.get_input_colorspaces(encoding)
+            log("init_video_encoder_option(%s) %s input colorspaces(%s)=%s", encoder_module, encoder_type, encoding, colorspaces)
             for colorspace in colorspaces:
                 spec = encoder_module.get_spec(encoding, colorspace)
                 self.add_encoder_spec(encoding, colorspace, spec)
@@ -308,7 +313,7 @@ class VideoHelper(object):
         try:
             csc_module.init_module()
             self._cleanup_modules.append(csc_module)
-        except Exception, e:
+        except Exception as e:
             log("exception in %s module initialization %s: %s", csc_type, csc_module.init_module, e, exc_info=True)
             log.warn("cannot use %s module %s: %s", csc_type, csc_module, e)
             return
@@ -344,7 +349,7 @@ class VideoHelper(object):
         try:
             decoder_module.init_module()
             self._cleanup_modules.append(decoder_module)
-        except Exception, e:
+        except Exception as e:
             log("exception in %s module initialization %s: %s", encoder_type, decoder_module.init_module, e, exc_info=True)
             log.warn("cannot use %s module %s: %s", encoder_type, decoder_module, e, exc_info=True)
             return
@@ -359,7 +364,7 @@ class VideoHelper(object):
                 try:
                     assert decoder_module.Decoder
                     self.add_decoder(encoding, colorspace, decoder_name, decoder_module)
-                except Exception, e:
+                except Exception as e:
                     log.warn("failed to add decoder %s: %s", decoder_module, e)
 
     def add_decoder(self, encoding, colorspace, decoder_name, decoder_module):
@@ -412,8 +417,8 @@ def getVideoHelper():
 
 def main():
     from xpra.codecs.loader import log as loader_log
-    loader_log.enable_debug()
     if "-v" in sys.argv or "--verbose" in sys.argv:
+        loader_log.enable_debug()
         log.enable_debug()
     vh = getVideoHelper()
     vh.set_modules(ALL_VIDEO_ENCODER_OPTIONS, ALL_CSC_MODULE_OPTIONS, ALL_VIDEO_DECODER_OPTIONS)
