@@ -1,6 +1,6 @@
 # This file is part of Xpra.
 # Copyright (C) 2008, 2009 Nathaniel Smith <njs@pobox.com>
-# Copyright (C) 2012-2014 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2012-2015 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -13,6 +13,8 @@ import weakref
 logging.basicConfig(format="%(asctime)s %(message)s")
 logging.root.setLevel(logging.INFO)
 
+LOG_PREFIX = os.environ.get("XPRA_LOG_PREFIX", "")
+
 #so we can keep a reference to all the loggers in use
 #we may have multiple loggers for the same key, so use a dict
 #but we don't want to prevent garbage collection so use a list of weakrefs
@@ -21,14 +23,15 @@ def add_logger(categories, logger):
     global all_loggers
     categories = list(categories)
     categories.append("all")
+    l = weakref.ref(logger)
     for cat in categories:
-        all_loggers.setdefault(cat, []).append(weakref.ref(logger))
+        all_loggers.setdefault(cat, set()).add(l)
 
 def get_all_loggers():
     global all_loggers
     a = set()
     for loggers in all_loggers.values():
-        for l in loggers:
+        for l in list(loggers):
             #weakref:
             v = l()
             if v:
@@ -37,40 +40,50 @@ def get_all_loggers():
 
 debug_enabled_categories = set()
 debug_disabled_categories = set()
-def add_debug_category(cat):
-    remove_disabled_category(cat)
-    debug_enabled_categories.add(cat)
+def add_debug_category(*cat):
+    remove_disabled_category(*cat)
+    for c in cat:
+        debug_enabled_categories.add(c)
 
-def remove_debug_category(cat):
-    if cat in debug_enabled_categories:
-        debug_enabled_categories.remove(cat)
+def remove_debug_category(*cat):
+    for c in cat:
+        if c in debug_enabled_categories:
+            debug_enabled_categories.remove(c)
 
-def add_disabled_category(cat):
-    remove_debug_category(cat)
-    debug_disabled_categories.add(cat)
+def add_disabled_category(*cat):
+    remove_debug_category(*cat)
+    for c in cat:
+        debug_disabled_categories.add(c)
 
-def remove_disabled_category(cat):
-    if cat in debug_disabled_categories:
-        debug_disabled_categories.remove(cat)
+def remove_disabled_category(*cat):
+    for c in cat:
+        if c in debug_disabled_categories:
+            debug_disabled_categories.remove(c)
 
 
-def enable_debug_for(cat):
-    loggers = all_loggers.get(cat)
-    if loggers:
-        for l in loggers:
-            #weakref:
-            v = l()
-            if v:
-                v.enable_debug()
+def get_loggers_for_categories(*cat):
+    if not cat:
+        return  []
+    cset = set(cat)
+    matches = set()
+    for l in get_all_loggers():
+        if set(l.categories).issuperset(cset):
+            matches.add(l)
+    return list(matches or [])
 
-def disable_debug_for(cat):
-    loggers = all_loggers.get(cat)
-    if loggers:
-        for l in loggers:
-            #weakref:
-            v = l()
-            if v:
-                v.disable_debug()
+def enable_debug_for(*cat):
+    loggers = get_loggers_for_categories(*cat)
+    for l in loggers:
+        if not l.is_debug_enabled():
+            l.enable_debug()
+    return loggers
+
+def disable_debug_for(*cat):
+    loggers = get_loggers_for_categories(*cat)
+    for l in loggers:
+        if l.is_debug_enabled():
+            l.disable_debug()
+    return loggers
 
 
 default_level = logging.DEBUG
@@ -79,25 +92,42 @@ def set_default_level(level):
     default_level = level
 
 
+#this allows us to capture all logging and redirect it:
+def standard_logging(log, level, msg, *args, **kwargs):
+    #this is just the regular logging:
+    log(level, msg, *args, **kwargs)
+
+global_logging_handler = standard_logging
+
+def set_global_logging_handler(h):
+    global global_logging_handler
+    saved = global_logging_handler
+    global_logging_handler = h
+    return saved
+
+
 KNOWN_FILTERS = ["auth", "cairo", "client", "clipboard", "codec", "loader", "video",
-                 "score", "encoding", "scaling", "subregion", "regiondetect", "regionrefresh", "refresh", "compress", "mouse",
+                 "score", "encoding", "scaling", "delta",
+                 "subregion", "regiondetect", "regionrefresh", "refresh", "compress", "mouse",
                  "error", "verbose",
                  #codecs:
                  "csc", "cuda", "cython", "opencl", "swscale",
-                 "decoder", "avcodec", "vpx", "proxy",
+                 "decoder", "avcodec", "vpx", "nvenc", "proxy",
                  "x264", "webp",
+                 "x265",
                  "gobject", "gtk", "main", "util", "dbus",
                  "window", "icon", "info", "launcher", "mdns", "cursor",
                  "mmap", "network", "protocol", "crypto", "encoder", "stats",
                  "notify", "xsettings", "grab", "xshm", "workspace",
-                 "sound", "events",
+                 "sound", "printing", "file", "events",
                  "opengl",
                  "osx", "win32",
                  "paint", "platform", "import",
-                 "posix", "qt",
-                 "keyboard", "pointer", "focus",
-                 "server", "command",
+                 "posix",
+                 "keyboard", "pointer", "focus", "metadata", "state", "screen",
+                 "server", "command", "timeout",
                  "shadow",
+                 "test",
                  "x11", "bindings", "core", "randr", "ximage", "focus", "tray", "xor"]
 
 
@@ -155,7 +185,10 @@ class Logger(object):
     def log(self, level, msg, *args, **kwargs):
         if kwargs.get("exc_info") is True:
             kwargs["exc_info"] = sys.exc_info()
-        self.logger.log(level, msg, *args, **kwargs)
+        global global_logging_handler
+        if LOG_PREFIX:
+            msg = LOG_PREFIX+msg
+        global_logging_handler(self.logger.log, level, msg, *args, **kwargs)
 
     def __call__(self, msg, *args, **kwargs):
         if self.debug_enabled:
