@@ -3,7 +3,6 @@
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
-import sys
 import struct
 
 
@@ -17,21 +16,12 @@ LZO_FLAG        = 0x20
 FLAGS_NOHEADER  = 0x40
 #0x80 is free
 
-if sys.version_info[:2]>=(2,5):
-    def unpack_header(buf):
-        return struct.unpack_from('!cBBBL', buf)
-else:
-    def unpack_header(buf):
-        return struct.unpack('!cBBBL', "".join(buf))
-
+_header_unpack_struct = struct.Struct('!cBBBL')
+def unpack_header(buf):
+    return _header_unpack_struct.unpack_from(buf)
 
 #'P' + protocol-flags + compression_level + packet_index + data_size
+_header_pack_struct = struct.Struct('!BBBBL')
+assert ord("P") == 80
 def pack_header(proto_flags, level, index, payload_size):
-    return struct.pack('!BBBBL', ord("P"), proto_flags, level, index, payload_size)
-
-if sys.version_info[0]<3:
-    #before v3, python does the right thing without hassle:
-    def pack_header_and_data(actual_size, proto_flags, level, index, payload_size, data):
-        return struct.pack('!BBBBL%ss' % actual_size, ord("P"), proto_flags, level, index, payload_size, data)
-else:
-    pack_header_and_data = None
+    return _header_pack_struct.pack(80, proto_flags, level, index, payload_size)
