@@ -53,8 +53,8 @@ def do_get_bind_IPs():
 					ips.append(ip)
 				if ipmask not in if_ipmasks:
 					if_ipmasks.append(ipmask)
-		except:
-			log.error("error on %s: %s", iface, sys.exc_info()[1])
+		except Exception as e:
+			log.error("error on %s: %s", iface, e)
 		iface_ipmasks[iface] = if_ipmasks
 	log("do_get_bind_IPs()=%s", ips)
 	return ips
@@ -71,8 +71,8 @@ def do_get_bind_ifacemask(iface):
 					try:
 						socket.inet_aton(addr)
 						ipmasks.append((addr,mask))
-					except:
-						log.error("do_get_bind_ifacemask(%s) error on %s", iface, addr, sys.exc_info()[1])
+					except Exception as e:
+						log.error("do_get_bind_ifacemask(%s) error on %s", iface, addr, e)
 	log("do_get_bind_ifacemask(%s)=%s", iface, ipmasks)
 	return ipmasks
 
@@ -110,8 +110,8 @@ def get_iface(ip):
 						break
 				if match:
 					best_match = iface
-			except:
-				log.error("error parsing ip (%s) or its mask (%s): %s", test_ip, mask, sys.exc_info()[1])
+			except Exception as e:
+				log.error("error parsing ip (%s) or its mask (%s): %s", test_ip, mask, e)
 	log("get_iface(%s)=%s", ip, best_match)
 	return	best_match
 
@@ -136,10 +136,10 @@ if not sys.platform.startswith("win"):
 		#<CDLL 'libc.so.6', handle 7fcac419b000 at 7fcac1ab0c10>
 		_libc = CDLL(library)
 		log("successfully loaded socket C library from %s", library)
-	except ImportError:
-		log.error("library %s not found: %s", library, sys.exc_info()[1])
-	except OSError:
-		log.error("error loading %s: %s", library, sys.exc_info()[1])
+	except ImportError as e:
+		log.error("library %s not found: %s", library, e)
+	except OSError as e:
+		log.error("error loading %s: %s", library, e)
 	else:
 		_libc.if_indextoname.restype = c_char_p
 		_libc.if_indextoname.argtypes = [c_uint, c_char_p]
