@@ -18,22 +18,24 @@ from threading import Lock
 
 from xpra.log import Logger
 log = Logger("network", "protocol")
-debug = log.debug
 from xpra.os_util import Queue, strtobytes
-from xpra.util import repr_ellipsized
+from xpra.util import repr_ellipsized, updict
 from xpra.net.bytestreams import ABORT
 from xpra.net import compression
 from xpra.net import packet_encoding
-from xpra.net.compression import get_compression_caps, decompress, InvalidCompressionException, Compressed, LevelCompressed, Uncompressed
-from xpra.net.packet_encoding import get_packet_encoding_caps, decode, InvalidPacketEncodingException
-from xpra.net.header import unpack_header, pack_header, pack_header_and_data, FLAGS_CIPHER, FLAGS_NOHEADER
+from xpra.net.compression import get_compression_caps, decompress, sanity_checks as compression_sanity_checks,\
+        InvalidCompressionException, Compressed, LevelCompressed, Uncompressed
+from xpra.net.packet_encoding import get_packet_encoding_caps, decode, sanity_checks as packet_encoding_sanity_checks, InvalidPacketEncodingException
+from xpra.net.header import unpack_header, pack_header, FLAGS_CIPHER, FLAGS_NOHEADER
 from xpra.net.crypto import get_crypto_caps, get_cipher
 
 
 #stupid python version breakage:
+JOIN_TYPES = (str, bytes)
 if sys.version > '3':
     long = int              #@ReservedAssignment
     unicode = str           #@ReservedAssignment
+    JOIN_TYPES = (bytes, )
 
 
 USE_ALIASES = os.environ.get("XPRA_USE_ALIASES", "1")=="1"
@@ -70,6 +72,11 @@ def get_network_caps(legacy=True):
     caps.update(get_packet_encoding_caps())
     return caps
 
+def sanity_checks():
+    """ warns the user if important modules are missing """
+    compression_sanity_checks()
+    packet_encoding_sanity_checks()
+
 
 class ConnectionClosedException(Exception):
     pass
@@ -190,19 +197,19 @@ class Protocol(object):
         return  [x for x in [self._write_thread, self._read_thread, self._read_parser_thread, self._write_format_thread] if x is not None]
 
 
-    def get_info(self):
+    def get_info(self, alias_info=True):
         info = {
-            "input.count"           : self.input_stats,
             "input.packetcount"     : self.input_packetcount,
             "input.raw_packetcount" : self.input_raw_packetcount,
             "input.cipher"          : self.cipher_in_name or "",
-            "output.count"          : self.output_stats,
             "output.packetcount"    : self.output_packetcount,
             "output.raw_packetcount": self.output_raw_packetcount,
             "output.cipher"         : self.cipher_out_name or "",
             "large_packets"         : self.large_packets,
             "compression_level"     : self.compression_level,
             "max_packet_size"       : self.max_packet_size}
+        updict(info, "input.count", self.input_stats)
+        updict(info, "output.count", self.output_stats)
         c = self._compress
         if c:
             info["compressor"] = compression.get_compressor_name(self._compress)
@@ -212,18 +219,23 @@ class Protocol(object):
                 info["encoder"] = "noencode"
             else:
                 info["encoder"] = packet_encoding.get_encoder_name(self._encoder)
-        for k,v in self.send_aliases.items():
-            info["send_alias." + str(k)] = v
-            info["send_alias." + str(v)] = k
-        for k,v in self.receive_aliases.items():
-            info["receive_alias." + str(k)] = v
-            info["receive_alias." + str(v)] = k
+        if alias_info:
+            for k,v in self.send_aliases.items():
+                info["send_alias." + str(k)] = v
+                info["send_alias." + str(v)] = k
+            for k,v in self.receive_aliases.items():
+                info["receive_alias." + str(k)] = v
+                info["receive_alias." + str(v)] = k
         c = self._conn
         if c:
             try:
                 info.update(self._conn.get_info())
             except:
                 log.error("error collecting connection information on %s", self._conn, exc_info=True)
+        info["has_more"] = self._source_has_more.is_set()
+        for t in (self._write_thread, self._read_thread, self._read_parser_thread, self._write_format_thread):
+            if t:
+                info["thread.%s" % t.name] = t.is_alive()
         return info
 
 
@@ -241,7 +253,7 @@ class Protocol(object):
             log("send_now(%s ...) connection is closed already, not sending", packet[0])
             return
         log("send_now(%s ...)", packet[0])
-        assert self._get_packet_cb==None, "cannot use send_now when a packet source exists!"
+        assert self._get_packet_cb==None, "cannot use send_now when a packet source exists! (set to %s)" % self._get_packet_cb
         def packet_cb():
             self._get_packet_cb = None
             return (packet, )
@@ -270,13 +282,10 @@ class Protocol(object):
             return
         log("add_packet_to_queue(%s ...)", packet[0])
         chunks, proto_flags = self.encode(packet)
-        try:
-            self._write_lock.acquire()
+        with self._write_lock:
             if self._closed:
                 return
             self._add_chunks_to_queue(chunks, proto_flags, start_send_cb, end_send_cb)
-        finally:
-            self._write_lock.release()
 
     def _add_chunks_to_queue(self, chunks, proto_flags, start_send_cb=None, end_send_cb=None):
         """ the write_lock must be held when calling this function """
@@ -308,10 +317,10 @@ class Protocol(object):
             if proto_flags & FLAGS_NOHEADER:
                 #for plain/text packets (ie: gibberish response)
                 items.append((data, scb, ecb))
-            elif pack_header_and_data is not None and actual_size<PACKET_JOIN_SIZE:
-                if type(data)==unicode:
-                    data = str(data)
-                header_and_data = pack_header_and_data(actual_size, proto_flags, level, index, payload_size, data)
+            elif actual_size<PACKET_JOIN_SIZE:
+                if type(data) not in JOIN_TYPES:
+                    data = bytes(data)
+                header_and_data = pack_header(proto_flags, level, index, payload_size) + data
                 items.append((header_and_data, scb, ecb))
             else:
                 header = pack_header(proto_flags, level, index, payload_size)
@@ -339,10 +348,8 @@ class Protocol(object):
         if packet is None:
             return err("None value")
         if type(packet)==list:
-            i = 0
-            for x in packet:
+            for i, x in enumerate(packet):
                 self.do_verify_packet(new_tree("[%s]" % i), x)
-                i += 1
         elif type(packet)==dict:
             for k,v in packet.items():
                 self.do_verify_packet(new_tree("key for value='%s'" % str(v)), k)
@@ -356,8 +363,9 @@ class Protocol(object):
 
     def enable_encoder_from_caps(self, caps):
         opts = packet_encoding.get_enabled_encoders(order=packet_encoding.PERFORMANCE_ORDER)
+        log("enable_encoder_from_caps(..) options=%s", opts)
         for e in opts:
-            if caps.boolget(e):
+            if caps.boolget(e, e=="bencode"):
                 self.enable_encoder(e)
                 return True
         log.error("no matching packet encoder found!")
@@ -366,6 +374,7 @@ class Protocol(object):
     def enable_encoder(self, e):
         self._encoder = packet_encoding.get_encoder(e)
         self.encoder = e
+        log("enable_encoder(%s): %s", e, self._encoder)
 
 
     def enable_default_compressor(self):
@@ -380,9 +389,9 @@ class Protocol(object):
             self.enable_compressor("none")
             return
         opts = compression.get_enabled_compressors(order=compression.PERFORMANCE_ORDER)
-        defaults = {"zlib" : True}
+        log("enable_compressor_from_caps(..) options=%s", opts)
         for c in opts:      #ie: [zlib, lz4, lzo]
-            if caps.boolget(c, defaults.get(c)):
+            if caps.boolget(c):
                 self.enable_compressor(c)
                 return
         log.warn("compression disabled: no matching compressor found")
@@ -396,7 +405,16 @@ class Protocol(object):
 
     def noencode(self, data):
         #just send data as a string for clients that don't understand xpra packet format:
-        return ": ".join([str(x) for x in data])+"\n", FLAGS_NOHEADER
+        if sys.version_info[0] >= 3:
+            import codecs
+            def b(x):
+                if type(x)==bytes:
+                    return x
+                return codecs.latin_1_encode(x)[0]
+        else:
+            def b(x):               #@DuplicatedSignature
+                return x
+        return b(": ".join(str(x) for x in data)+"\n"), FLAGS_NOHEADER
 
 
     def encode(self, packet_in):
@@ -447,14 +465,14 @@ class Protocol(object):
                     packet[i] = item.data
                     min_comp_size += l
                     size_check += l
-            elif ti==str and level>0 and l>LARGE_PACKET_SIZE:
+            elif ti in (str, bytes) and level>0 and l>LARGE_PACKET_SIZE:
                 log.warn("found a large uncompressed item in packet '%s' at position %s: %s bytes", packet[0], i, len(item))
                 #add new binary packet with large item:
                 cl, cdata = self._compress(item, level)
                 packets.append((i, cl, cdata))
                 #replace this item with an empty string placeholder:
                 packet[i] = ''
-            elif ti!=str:
+            elif ti not in (str, bytes):
                 log.warn("unexpected data type %s in %s packet: %s", ti, packet[0], repr_ellipsized(item))
         #now the main packet (or what is left of it):
         packet_type = packet[0]
@@ -464,7 +482,7 @@ class Protocol(object):
             packet[0] = self.send_aliases[packet_type]
         try:
             main_packet, proto_version = self._encoder(packet)
-        except Exception, e:
+        except Exception as e:
             if self._closed:
                 return [], 0
             log.error("failed to encode packet: %s", packet, exc_info=True)
@@ -494,12 +512,10 @@ class Protocol(object):
             while not self._closed:
                 callback()
             log("io_thread_loop(%s, %s) loop ended, closed=%s", name, callback, self._closed)
-        except KeyboardInterrupt, e:
-            raise e
-        except ConnectionClosedException, e:
+        except ConnectionClosedException as e:
             if not self._closed:
                 self._internal_error("%s connection %s closed: %s" % (name, self._conn, e))
-        except (OSError, IOError, socket_error), e:
+        except (OSError, IOError, socket_error) as e:
             if not self._closed:
                 self._internal_error("%s connection %s reset: %s" % (name, self._conn, e), exc_info=e.args[0] not in ABORT)
         except:
@@ -577,9 +593,7 @@ class Protocol(object):
     #(so this can more easily be intercepted and overriden
     # see tcp-proxy)
     def _invalid_header(self, data):
-        #call via idle_add gives the client time
-        #to disconnect (and so we don't bother with it)
-        self.idle_add(self.invalid_header, self, data)
+        self.invalid_header(self, data)
 
     def invalid_header(self, proto, data):
         err = "invalid packet header: '%s'" % binascii.hexlify(data[:8])
@@ -631,14 +645,13 @@ class Protocol(object):
                 if bl<=0:
                     break
                 if payload_size<0:
-                    head = read_buffer[:8]
                     if read_buffer[0] not in ("P", ord("P")):
                         self._invalid_header(read_buffer)
                         return
                     if bl<8:
                         break   #packet still too small
                     #packet format: struct.pack('cBBBL', ...) - 8 bytes
-                    _, protocol_flags, compression_level, packet_index, data_size = unpack_header(head)
+                    _, protocol_flags, compression_level, packet_index, data_size = unpack_header(read_buffer[:8])
 
                     #sanity check size (will often fail if not an xpra client):
                     if data_size>self.abs_max_packet_size:
@@ -706,10 +719,10 @@ class Protocol(object):
                 if compression_level>0:
                     try:
                         data = decompress(data, compression_level)
-                    except InvalidCompressionException, e:
+                    except InvalidCompressionException as e:
                         self.invalid("invalid compression: %s" % e, data)
                         return
-                    except Exception, e:
+                    except Exception as e:
                         ctype = compression.get_compression_type(compression_level)
                         log("%s packet decompression failed", ctype, exc_info=True)
                         msg = "%s packet decompression failed" % ctype
@@ -737,10 +750,10 @@ class Protocol(object):
                 #final packet (packet_index==0), decode it:
                 try:
                     packet = decode(data, protocol_flags)
-                except InvalidPacketEncodingException, e:
+                except InvalidPacketEncodingException as e:
                     self.invalid("invalid packet encoding: %s" % e, data)
                     return
-                except ValueError, e:
+                except ValueError as e:
                     etype = packet_encoding.get_packet_encoding_type(protocol_flags)
                     log.error("failed to parse %s packet: %s", etype, e, exc_info=not self._closed)
                     if self._closed:
