@@ -4,9 +4,6 @@
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
-from xpra.platform.darwin.osx_menu import getOSXMenuHelper
-from xpra.platform.paths import get_icon
-
 from xpra.log import Logger
 log = Logger("osx", "events")
 
@@ -46,10 +43,15 @@ except:
 
 def do_init():
     osxapp = get_OSXApplication()
+    log("do_init() osxapp=%s", osxapp)
+    from xpra.platform.paths import get_icon
     icon = get_icon("xpra.png")
+    log("do_init() icon=%s", icon)
     if icon:
         osxapp.set_dock_icon_pixbuf(icon)
+    from xpra.platform.darwin.osx_menu import getOSXMenuHelper
     mh = getOSXMenuHelper(None)
+    log("do_init() menu helper=%s", mh)
     osxapp.set_dock_menu(mh.build_dock_menu())
     osxapp.set_menu_bar(mh.rebuild())
 
@@ -60,6 +62,7 @@ def do_ready():
 
 
 def get_native_tray_menu_helper_classes():
+    from xpra.platform.darwin.osx_menu import getOSXMenuHelper
     return [getOSXMenuHelper]
 
 def get_native_tray_classes():
@@ -72,6 +75,24 @@ def system_bell(*args):
     Snd.SysBeep(1)
     return True
 
+#if there is an easier way of doing this, I couldn't find it:
+try:
+    import ctypes
+    Carbon_ctypes = ctypes.CDLL("/System/Library/Frameworks/Carbon.framework/Carbon")
+except:
+    Carbon_ctypes = None
+
+def get_double_click_time():
+    try:
+        #what are ticks? just an Apple retarded way of measuring elapsed time.
+        #They must have considered gigaparsecs divided by teapot too, which is just as useful.
+        #(but still call it "Time" you see)
+        MS_PER_TICK = 1000/60
+        return int(Carbon_ctypes.GetDblTime() * MS_PER_TICK)
+    except:
+        return -1
+
+
 try:
     import Quartz.CoreGraphics as CG    #@UnresolvedImport
     ALPHA = {
@@ -110,7 +131,7 @@ def get_CG_imagewrapper():
 
 def take_screenshot():
     log("grabbing screenshot")
-    from PIL import Image
+    from PIL import Image                       #@UnresolvedImport
     from xpra.os_util import StringIOClass
     image = get_CG_imagewrapper()
     w = image.get_width()
@@ -129,7 +150,7 @@ try:
     import PyObjCTools.AppHelper as AppHelper   #@UnresolvedImport
     NSObject = Foundation.NSObject
     NSWorkspace = AppKit.NSWorkspace
-except Exception, e:
+except Exception as e:
     log.warn("failed to load critical modules for sleep notification support: %s", e)
     NSObject = object
     NSWorkspace = None
@@ -139,11 +160,19 @@ class NotificationHandler(NSObject):
 
     def handleSleepNotification_(self, aNotification):
         log("handleSleepNotification(%s)", aNotification)
+        if self.sleep_callback:
+            try:
         self.sleep_callback()
+            except:
+                log.error("Error in sleep callback %s", self.sleep_callback, exc_info=True)
 
     def handleWakeNotification_(self, aNotification):
         log("handleWakeNotification(%s)", aNotification)
+        if self.wake_callback:
+            try:
         self.wake_callback()
+            except:
+                log.error("Error in wake callback %s", self.wake_callback, exc_info=True)
 
 
 class ClientExtras(object):
