@@ -5,7 +5,7 @@
 # later version. See the file COPYING for details.
 
 
-import gtk.gdk
+from xpra.gtk_common.gtk_util import META_MASK
 from xpra.platform.keyboard_base import KeyboardBase, log
 from xpra.platform.darwin.osx_menu import getOSXMenuHelper
 
@@ -34,12 +34,19 @@ class Keyboard(KeyboardBase):
         self.num_lock_keycode = NUM_LOCK_KEYCODE
         self.key_translations = {}
 
+    def get_keymap_modifiers(self):
+        """
+            Override superclass so we can tell the server
+            that 'control' will also be missing from non key events modifiers
+        """
+        return  {}, [], ["lock", "control"]
+
     def set_modifier_mappings(self, mappings):
         KeyboardBase.set_modifier_mappings(self, mappings)
         self.meta_modifier = self.modifier_keys.get("Meta_L") or self.modifier_keys.get("Meta_R")
         self.control_modifier = self.modifier_keys.get("Control_L") or self.modifier_keys.get("Control_R")
         self.num_lock_modifier = self.modifier_keys.get("Num_Lock")
-        log("set_modifier_mappings(..) meta=%s, control=%s, numlock=%s", mappings, self.meta_modifier, self.control_modifier, self.num_lock_modifier)
+        log("set_modifier_mappings(%s) meta=%s, control=%s, numlock=%s", mappings, self.meta_modifier, self.control_modifier, self.num_lock_modifier)
         #find the keysyms and keycodes to use for each key we may translate:
         for orig_keysym in KEYS_TRANSLATION_OPTIONS.keys():
             new_def = self.find_translation(orig_keysym)
@@ -76,9 +83,10 @@ class Keyboard(KeyboardBase):
     def mask_to_names(self, mask):
         names = KeyboardBase.mask_to_names(self, mask)
         if self.swap_keys and self.meta_modifier is not None and self.control_modifier is not None:
-            meta_on = bool(mask & gtk.gdk.META_MASK)
+            meta_on = bool(mask & META_MASK)
             meta_set = self.meta_modifier in names
             control_set = self.control_modifier in names
+            log("mask_to_names names=%s, meta_on=%s, meta_set=%s, control_set=%s", names, meta_on, meta_set, control_set)
             if meta_on and not control_set:
                 log("mask_to_names swapping meta for control: %s for %s", self.meta_modifier, self.control_modifier)
                 names.append(self.control_modifier)
