@@ -15,7 +15,7 @@ log = Logger("tray", "osx")
 
 
 #control which menus are shown in the OSX global menu:
-SHOW_SOUND_MENU = False
+SHOW_SOUND_MENU = True
 SHOW_ENCODINGS_MENU = True
 SHOW_ACTIONS_MENU = True
 SHOW_ABOUT_XPRA = True
@@ -108,6 +108,7 @@ class OSXMenuHelper(GTKTrayMenuBase):
         features_menu.add(self.make_notificationsmenuitem())
         features_menu.add(self.make_swapkeysmenuitem())
         features_menu.add(self.make_numlockmenuitem())
+        features_menu.add(self.make_openglmenuitem())
         if SHOW_SOUND_MENU:
             sound_menu = self.make_osxmenu("Sound")
             if self.client.speaker_allowed and len(self.client.speaker_codecs)>0:
@@ -125,6 +126,12 @@ class OSXMenuHelper(GTKTrayMenuBase):
             actions_menu = self.make_osxmenu("Actions")
             actions_menu.add(self.make_refreshmenuitem())
             actions_menu.add(self.make_raisewindowsmenuitem())
+            #set_sensitive(bool) does not work on OSX,
+            #so we only add the menu item if it does something
+            def addsnc(*args):
+                if self.client.start_new_commands:
+                    actions_menu.add(self.make_startnewcommandmenuitem(True))
+            self.client.connect("handshake-complete", addsnc)
         self.menu_bar.show_all()
 
     #these methods are called by the superclass
