@@ -1,9 +1,11 @@
 # This file is part of Xpra.
 # Copyright (C) 2010 Nathaniel Smith <njs@pobox.com>
-# Copyright (C) 2011-2014 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2011-2015 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
+import os.path
+
 #defaults which may be overriden by platform_import:
 LOCAL_SERVERS_SUPPORTED = False
 SHADOW_SUPPORTED = False
@@ -18,6 +20,10 @@ CLIPBOARD_GREEDY = False
 CLIPBOARD_NATIVE_CLASS = None
 CAN_DAEMONIZE = False
 UI_THREAD_POLLING = 0
+OPEN_COMMAND = "xdg-open"
+DOWNLOAD_PATH = "~/Downloads"
+if not os.path.exists(os.path.expanduser(DOWNLOAD_PATH)):
+    DOWNLOAD_PATH = "~"
 
 DEFAULT_PULSEAUDIO_COMMAND = "pulseaudio --start --daemonize=false --system=false " + \
                                     "--exit-idle-time=-1 -n --load=module-suspend-on-idle " + \
@@ -36,6 +42,7 @@ _features_list_ = ["LOCAL_SERVERS_SUPPORTED",
                 "DEFAULT_PULSEAUDIO_COMMAND",
                 "DEFAULT_XVFB_COMMAND",
                 "GOT_PASSWORD_PROMPT_SUGGESTION",
+                "DOWNLOAD_PATH",
                 "CLIPBOARDS",
                 "CLIPBOARD_WANT_TARGETS",
                 "CLIPBOARD_GREEDY",
