@@ -157,8 +157,7 @@ def fix_unicode_out():
                     if self._hConsole is None:
                         try:
                             self._stream.flush()
-                        except Exception:
-                            e = sys.exc_info()[1]
+                        except Exception as e:
                             _complain("%s.flush: %r from %r" % (self.name, e, self._stream))
                             raise
 
@@ -184,8 +183,7 @@ def fix_unicode_out():
                                 if not remaining:
                                     break
                                 text = text[n.value:]
-                    except:
-                        e = sys.exc_info()[1]
+                    except Exception as e:
                         _complain("%s.write: %r" % (self.name, e))
                         raise
 
@@ -193,8 +191,7 @@ def fix_unicode_out():
                     try:
                         for line in lines:
                             self.write(line)
-                    except:
-                        e = sys.exc_info()[1]
+                    except Exception as e:
                         _complain("%s.writelines: %r" % (self.name, e))
                         raise
 
@@ -207,8 +204,7 @@ def fix_unicode_out():
                 sys.stderr = UnicodeOutput(hStderr, None, STDERR_FILENO, '<Unicode console stderr>')
             else:
                 sys.stderr = UnicodeOutput(None, sys.stderr, old_stderr_fileno, '<Unicode redirected stderr>')
-    except Exception:
-        e = sys.exc_info()[1]
+    except Exception as e:
         _complain("exception %r while fixing up sys.stdout and sys.stderr" % (e,))
 
 _wait_for_input = False
