@@ -9,7 +9,7 @@ from xpra.platform.keyboard_base import KeyboardBase
 from xpra.keyboard.layouts import WIN32_LAYOUTS
 from xpra.gtk_common.keymap import KEY_TRANSLATIONS
 from xpra.log import Logger
-log = Logger("keyboard", "win32")
+log = Logger("keyboard")
 
 
 class Keyboard(KeyboardBase):
@@ -94,7 +94,7 @@ class Keyboard(KeyboardBase):
                 log("unknown keyboard layout for kbid: %s", kbid)
             else:
                 layouts.append(layout)
-        except Exception, e:
+        except Exception as e:
             log.error("failed to detect keyboard layout: %s", e)
         return layout,layouts,variant,variants
 
@@ -110,9 +110,9 @@ class Keyboard(KeyboardBase):
             #0=1000/30, 31=1000/2.5
             _speed = min(31, max(0, _speed))
             speed = int(1000/(2.5+27.5*_speed/31))
-            log.debug("keyboard repeat speed(%s)=%s, delay(%s)=%s", _speed, speed, _delay, delay)
+            log("keyboard repeat speed(%s)=%s, delay(%s)=%s", _speed, speed, _delay, delay)
             return  delay,speed
-        except Exception, e:
+        except Exception as e:
             log.error("failed to get keyboard rate: %s", e)
         return None
 
