@@ -4,8 +4,6 @@
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
-import sys
-
 from xpra.platform.keyboard_base import KeyboardBase
 from xpra.keyboard.mask import MODIFIER_MAP
 from xpra.keyboard.layouts import parse_xkbmap_query, xkbmap_query_tostring
@@ -15,7 +13,7 @@ log = Logger("keyboard", "posix")
 try:
     from xpra.x11.bindings.keyboard_bindings import X11KeyboardBindings   #@UnresolvedImport
     keyboard_bindings = X11KeyboardBindings()
-except Exception, e:
+except Exception as e:
     log.warn("failed load posix keyboard bindings: %s", e)
     keyboard_bindings = None
 
@@ -30,7 +28,7 @@ class Keyboard(KeyboardBase):
             if returncode==0:
                 return out.decode('utf-8')
             log.error("'%s' failed with exit code %s", cmd, returncode)
-        except Exception, e:
+        except Exception as e:
             log.error("error running '%s': %s", cmd, e)
         return None
 
@@ -40,7 +38,7 @@ class Keyboard(KeyboardBase):
                 mod_mappings = keyboard_bindings.get_modifier_mappings()
                 if mod_mappings:
                     #ie: {"shift" : ["Shift_L", "Shift_R"], "mod1" : "Meta_L", ...]}
-                    log.debug("modifier mappings=%s", mod_mappings)
+                    log("modifier mappings=%s", mod_mappings)
                     meanings = {}
                     for modifier,keys in mod_mappings.items():
                         for _,keyname in keys:
@@ -51,7 +49,7 @@ class Keyboard(KeyboardBase):
                     if numlock_mod:
                         mod_missing.append(numlock_mod)
                     return  meanings, [], mod_missing
-            except Exception, e:
+            except Exception as e:
                 log.error("failed to use native get_modifier_mappings: %s", e, exc_info=True)
         return self.modifiers_fallback()
 
@@ -72,14 +70,14 @@ class Keyboard(KeyboardBase):
                 for x in nohex:
                     #ie: meanings['Shift_L']=shift
                     meanings[x] = parts[0]
-        log.debug("get_keymap_modifiers parsed: meanings=%s", meanings)
+        log("get_keymap_modifiers parsed: meanings=%s", meanings)
         return  meanings, [], []
 
     def get_x11_keymap(self):
         if keyboard_bindings:
             try:
                 return keyboard_bindings.get_keycode_mappings()
-            except Exception, e:
+            except Exception as e:
                 log.error("failed to use raw x11 keymap: %s", e)
         return  {}
 
@@ -103,6 +101,7 @@ class Keyboard(KeyboardBase):
             return None
         _query_struct = keyboard_bindings.getXkbProperties()
         _query = xkbmap_query_tostring(_query_struct)
+        log("get_keymap_spec_from_xkb() query tostring(%s)=%s", _query_struct, _query)
         xkbmap_print = self.exec_get_keyboard_data(["setxkbmap", "-print"])
         return xkbmap_print or "", _query, _query_struct
 
@@ -120,13 +119,14 @@ class Keyboard(KeyboardBase):
         #FIXME: a bit ugly to call gtk here...
         #but otherwise we have to call XGetWindowProperty and deal with X11 errors..
         xkb_rules_names = ""
-        if "gtk" in sys.modules:
-            import gtk.gdk
-            prop = gtk.gdk.get_default_root_window().property_get("_XKB_RULES_NAMES", "STRING")
-            #ie:('STRING', 8, 'evdev\x00pc104\x00gb,us\x00,\x00\x00')
-            if prop and len(prop)==3:
-                xkb_rules_names = prop[2].split("\0")
-                #ie: ['evdev', 'pc104', 'gb,us', ',', '', '']
+        from xpra.platform.xposix.gui import _get_X11_root_property
+        prop = _get_X11_root_property("_XKB_RULES_NAMES", "STRING")
+        log("get_xkb_rules_names_property() _XKB_RULES_NAMES=%s", prop)
+        #ie: 'evdev\x00pc104\x00gb,us\x00,\x00\x00'
+        if prop:
+            xkb_rules_names = prop.split("\0")
+            #ie: ['evdev', 'pc104', 'gb,us', ',', '', '']
+        log("get_xkb_rules_names_property()=%s", xkb_rules_names)
         return xkb_rules_names
 
     def get_layout_spec(self):
@@ -152,14 +152,18 @@ class Keyboard(KeyboardBase):
             try:
                 delay, interval = keyboard_bindings.get_key_repeat_rate()
                 return delay,interval
-            except Exception, e:
+            except Exception as e:
                 log.error("failed to get keyboard repeat rate: %s", e)
         return None
 
     def update_modifier_map(self, display, xkbmap_mod_meanings):
+        try:
+            dn = "%s %s" % (type(display).__name__, display.get_name())
+        except Exception as e:
+            dn = str(display)
         try:
             from xpra.x11.gtk_x11.keys import grok_modifier_map
             self.modifier_map = grok_modifier_map(display, xkbmap_mod_meanings)
         except ImportError:
             self.modifier_map = MODIFIER_MAP
-        log.debug("update_modifier_map(%s, %s) modifier_map=%s", display, xkbmap_mod_meanings, self.modifier_map)
+        log("update_modifier_map(%s, %s) modifier_map=%s", dn, xkbmap_mod_meanings, self.modifier_map)
