@@ -1,5 +1,6 @@
+#!/usr/bin/env python
 # This file is part of Xpra.
-# Copyright (C) 2010-2014 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2010-2015 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -48,7 +49,7 @@ def OpenGL_safety_check():
         try:
             rnum = [int(x) for x in ur.split(".")]  #ie: [12, 4]
             if rnum<=[12, 4]:
-                return False, "Ubuntu %s is too buggy" % ur
+                return "Ubuntu %s is too buggy" % ur
         except:
             pass
     #try to detect VirtualBox:
@@ -58,7 +59,7 @@ def OpenGL_safety_check():
     try:
         from ctypes import cdll
         if cdll.LoadLibrary("VBoxHook.dll"):
-            return True, "VirtualBox is present (VBoxHook.dll)"
+            return "VirtualBox is present (VBoxHook.dll)"
     except:
         pass
     try:
@@ -69,18 +70,16 @@ def OpenGL_safety_check():
             if f:
                 f.close()
                 return True, "VirtualBox is present (VBoxMiniRdrDN)"
-    except Exception, e:
+    except Exception as e:
         import errno
         if e.args[0]==errno.EACCES:
-            return True, "VirtualBox is present (VBoxMiniRdrDN)"
-    return True, None
+            return "VirtualBox is present (VBoxMiniRdrDN)"
+    return None
 OPENGL_DEFAULT = None       #will auto-detect by probing
-if not OpenGL_safety_check()[0]:
+if OpenGL_safety_check() is not None:
     OPENGL_DEFAULT = False
 
 
-
-
 def get_build_info():
     info = []
     try:
@@ -93,7 +92,7 @@ def get_build_info():
             info.append("revision %s" % REVISION)
         else:
             info.append("revision %s with %s local changes" % (REVISION, LOCAL_MODIFICATIONS))
-    except Exception, e:
+    except Exception as e:
         warn("Error: could not find the source information: %s" % e)
     try:
         from xpra.build_info import BUILT_BY, BUILT_ON, BUILD_DATE, CYTHON_VERSION, COMPILER_VERSION    #@UnresolvedImport
@@ -105,25 +104,27 @@ def get_build_info():
         if CYTHON_VERSION!="unknown":
             info.append("built with Cython %s" % CYTHON_VERSION)
         if COMPILER_VERSION!="unknown":
-            info.append(COMPILER_VERSION)
-    except Exception, e:
+            cv = COMPILER_VERSION.replace("Optimizing Compiler Version", "Optimizing Compiler\nVersion")
+            info += cv.splitlines()
+    except Exception as e:
         warn("Error: could not find the build information: %s" % e)
     return info
 
 
+def name_to_field(name):
+    return name.replace("-", "_")
+
 def save_config(conf_file, config, keys, extras_types={}):
-    f = open(conf_file, "w")
-    option_types = OPTION_TYPES.copy()
-    option_types.update(extras_types)
-    saved = {}
-    for key in keys:
-        assert key in option_types, "invalid configuration key: %s" % key
-        fn = key.replace("-", "_")
-        v = getattr(config, fn)
-        saved[key] = v
-        f.write("%s=%s%s" % (key, v, os.linesep))
-    debug("save_config: saved %s to %s", saved, conf_file)
-    f.close()
+    with open(conf_file, "w") as f:
+        option_types = OPTION_TYPES.copy()
+        option_types.update(extras_types)
+        saved = {}
+        for key in keys:
+            assert key in option_types, "invalid configuration key: %s" % key
+            v = getattr(config, name_to_field(key))
+            saved[key] = v
+            f.write("%s=%s%s" % (key, v, os.linesep))
+        debug("save_config: saved %s to %s", saved, conf_file)
 
 def read_config(conf_file):
     """
@@ -135,21 +136,20 @@ def read_config(conf_file):
     if not os.path.isfile(conf_file):
         debug("read_config(%s) is not a file!", conf_file)
         return d
-    f = open(conf_file, "rU")
-    lines = []
-    no = 0
-    for line in f:
-        sline = line.strip().rstrip('\r\n').strip()
-        no += 1
-        if len(sline) == 0:
-            debug("%4s empty line", no)
-            continue
-        if sline[0] in ( '!', '#' ):
-            debug("%4s skipping comments   : %s", no, sline[:16]+"..")
-            continue
-        debug("%4s loaded              : %s", no, sline)
-        lines.append(sline)
-    f.close()
+    with open(conf_file, "rU") as f:
+        lines = []
+        no = 0
+        for line in f:
+            sline = line.strip().rstrip('\r\n').strip()
+            no += 1
+            if len(sline) == 0:
+                debug("%4s empty line", no)
+                continue
+            if sline[0] in ( '!', '#' ):
+                debug("%4s skipping comments   : %s", no, sline[:16]+"..")
+                continue
+            debug("%4s loaded              : %s", no, sline)
+            lines.append(sline)
     debug("loaded %s lines", len(lines))
     #aggregate any lines with trailing backslash
     agg_lines = []
@@ -245,10 +245,10 @@ OPTION_TYPES = {
                     "title"             : str,
                     "username"          : str,
                     "auth"              : str,
+                    "tcp-auth"          : str,
                     "wm-name"           : str,
                     "remote-xpra"       : str,
                     "session-name"      : str,
-                    "client-toolkit"    : str,
                     "dock-icon"         : str,
                     "tray-icon"         : str,
                     "window-icon"       : str,
@@ -263,11 +263,19 @@ OPTION_TYPES = {
                     "socket-dir"        : str,
                     "log-file"          : str,
                     "border"            : str,
+                    "max-size"          : str,
                     "window-layout"     : str,
                     "display"           : str,
                     "tcp-proxy"         : str,
+                    "download-path"     : str,
+                    "open-command"      : str,
+                    "lpadmin"           : str,
                     "debug"             : str,
                     "input-method"      : str,
+                    "microphone"        : str,
+                    "speaker"           : str,
+                    "sound-source"      : str,
+                    "socket-permissions": str,
                     #int options:
                     "quality"           : int,
                     "min-quality"       : int,
@@ -275,8 +283,10 @@ OPTION_TYPES = {
                     "min-speed"         : int,
                     "compression_level" : int,
                     "dpi"               : int,
+                    "scaling"           : int,
+                    "file-size-limit"   : int,
+                    "idle-timeout"      : int,
                     #float options:
-                    "max-bandwidth"     : float,
                     "auto-refresh-delay": float,
                     #boolean options:
                     "daemon"            : bool,
@@ -289,8 +299,6 @@ OPTION_TYPES = {
                     "dbus-proxy"        : bool,
                     "mmap"              : bool,
                     "mmap-group"        : bool,
-                    "speaker"           : bool,
-                    "microphone"        : bool,
                     "readonly"          : bool,
                     "keyboard-sync"     : bool,
                     "pings"             : bool,
@@ -307,7 +315,12 @@ OPTION_TYPES = {
                     "exit-ssh"          : bool,
                     "opengl"            : bool,
                     "mdns"              : bool,
+                    "file-transfer"     : bool,
+                    "printing"          : bool,
+                    "open-files"        : bool,
                     "swap-keys"         : bool,
+                    "start-new-commands": bool,
+                    "remote-logging"    : bool,
                     #arrays of strings:
                     "encodings"         : list,
                     "video-encoders"    : list,
@@ -318,8 +331,10 @@ OPTION_TYPES = {
                     "compressors"       : list,
                     "packet-encoders"   : list,
                     "key-shortcut"      : list,
+                    "start"             : list,
                     "start-child"       : list,
                     "bind-tcp"          : list,
+                    "env"               : list,
                }
 
 GLOBAL_DEFAULTS = None
@@ -329,7 +344,7 @@ def get_defaults():
     global GLOBAL_DEFAULTS
     if GLOBAL_DEFAULTS is not None:
         return GLOBAL_DEFAULTS
-    from xpra.platform.features import DEFAULT_SSH_CMD, DEFAULT_PULSEAUDIO_COMMAND, DEFAULT_XVFB_COMMAND
+    from xpra.platform.features import DEFAULT_SSH_CMD, DOWNLOAD_PATH, OPEN_COMMAND, DEFAULT_PULSEAUDIO_COMMAND, DEFAULT_XVFB_COMMAND
     try:
         from xpra.platform.info import get_username
         username = get_username()
@@ -340,10 +355,10 @@ def get_defaults():
                     "title"             : "@title@ on @client-machine@",
                     "username"          : username,
                     "auth"              : "",
+                    "tcp-auth"          : "",
                     "wm-name"           : DEFAULT_NET_WM_NAME,
                     "remote-xpra"       : "~/.xpra/run-xpra",
                     "session-name"      : "",
-                    "client-toolkit"    : "",
                     "dock-icon"         : "",
                     "tray-icon"         : "",
                     "window-icon"       : "",
@@ -357,18 +372,27 @@ def get_defaults():
                     "socket-dir"        : "",
                     "log-file"          : "$DISPLAY.log",
                     "border"            : "auto,0",
+                    "max-size"          : "",
                     "window-layout"     : "",
                     "display"           : "",
                     "tcp-proxy"         : "",
+                    "download-path"     : DOWNLOAD_PATH,
+                    "open-command"      : OPEN_COMMAND,
+                    "lpadmin"           : "lpadmin",
                     "debug"             : "",
                     "input-method"      : "none",
+                    "sound-source"      : "",
+                    "html"              : "",
+                    "socket-permissions": "600",
                     "quality"           : 0,
                     "min-quality"       : 30,
                     "speed"             : 0,
                     "min-speed"         : 0,
                     "compression_level" : 1,
-                    "dpi"               : 96,
-                    "max-bandwidth"     : 0.0,
+                    "dpi"               : 0,
+                    "scaling"           : 1,
+                    "file-size-limit"   : 10,
+                    "idle-timeout"      : 0,
                     "auto-refresh-delay": 0.25,
                     "daemon"            : True,
                     "use-display"       : False,
@@ -380,8 +404,8 @@ def get_defaults():
                     "dbus-proxy"        : os.name=="posix" and not sys.platform.startswith("darwin"),
                     "mmap"              : True,
                     "mmap-group"        : False,
-                    "speaker"           : has_sound_support,
-                    "microphone"        : has_sound_support,
+                    "speaker"           : ["disabled", "on"][has_sound_support],
+                    "microphone"        : ["disabled", "off"][has_sound_support],
                     "readonly"          : False,
                     "keyboard-sync"     : True,
                     "pings"             : False,
@@ -395,9 +419,14 @@ def get_defaults():
                     "windows"           : True,
                     "exit-with-children": False,
                     "exit-with-client"  : False,
+                    "start-new-commands": False,
+                    "remote-logging"    : sys.platform.startswith("win") or sys.platform.startswith("darwin"),
                     "exit-ssh"          : True,
                     "opengl"            : OPENGL_DEFAULT,
                     "mdns"              : False,
+                    "file-transfer"     : True,
+                    "printing"          : True,
+                    "open-files"        : False,
                     "swap-keys"         : sys.platform.startswith("darwin"),    #only used on osx
                     "encodings"         : ["all"],
                     "video-encoders"    : ["all"],
@@ -407,33 +436,40 @@ def get_defaults():
                     "microphone-codec"  : [],
                     "compressors"       : ["all"],
                     "packet-encoders"   : ["all"],
-                    "key-shortcut"      : ["Meta+Shift+F4:quit", "Meta+Shift+F8:magic_key", "Meta+Shift+F11:show_session_info"],
+                    "key-shortcut"      : [
+                                           "Meta+Shift+F2:show_start_new_command",
+                                           "Meta+Shift+F4:quit",
+                                           "Meta+Shift+F8:magic_key",
+                                           "Meta+Shift+F11:show_session_info"
+                                           ],
                     "bind-tcp"          : None,
+                    "start"             : None,
                     "start-child"       : None,
+                    "env"               : None,
                     }
     return GLOBAL_DEFAULTS
 #fields that got renamed:
-CLONES = {
-            "quality"       : "jpeg-quality",
-          }
+CLONES = {}
 
 #these options should not be specified in config files:
 NO_FILE_OPTIONS = ["daemon"]
 
 
-
+TRUE_OPTIONS = ("yes", "true", "1", "on", True)
+FALSE_OPTIONS = ("no", "false", "0", "off", False)
 def parse_bool(k, v):
     if type(v)==str:
         v = v.lower()
-    if v in ["yes", "true", "1", "on", True]:
+    if v in TRUE_OPTIONS:
         return True
-    elif v in ["no", "false", "0", "off", False]:
+    elif v in FALSE_OPTIONS:
         return False
     elif v in ["auto", None]:
         #keep default - which may be None!
         return None
     else:
         warn("Warning: cannot parse value '%s' for '%s' as a boolean" % (v, k))
+        return None
 
 def print_bool(k, v, true_str='yes', false_str='no'):
     if type(v)==type(None):
@@ -444,6 +480,19 @@ def print_bool(k, v, true_str='yes', false_str='no'):
         return false_str
     warn("Warning: cannot print value '%s' for '%s' as a boolean" % (v, k))
 
+def parse_bool_or_int(k, v):
+    return parse_bool_or_number(int, k, v)
+
+def parse_bool_or_number(numtype, k, v, auto=0):
+    if type(v)==str:
+        v = v.lower()
+    if v in TRUE_OPTIONS:
+        return 1
+    elif v in FALSE_OPTIONS:
+        return 0
+    else:
+        return parse_number(numtype, k, v, auto)
+
 def parse_number(numtype, k, v, auto=0):
     if type(v)==str:
         v = v.lower()
@@ -451,10 +500,15 @@ def parse_number(numtype, k, v, auto=0):
         return auto
     try:
         return numtype(v)
-    except Exception, e:
+    except Exception as e:
         warn("Warning: cannot parse value '%s' for '%s' as a type %s: %s" % (v, k, numtype, e))
         return None
 
+def print_number(i, auto_value=0):
+    if i==auto_value:
+        return "auto"
+    return str(i)
+
 def validate_config(d={}, discard=NO_FILE_OPTIONS, extras_types={}, extras_validation={}):
     """
         Validates all the options given in a dict with fields as keys and
@@ -480,7 +534,7 @@ def validate_config(d={}, discard=NO_FILE_OPTIONS, extras_types={}, extras_valid
                 warn("invalid value for '%s': %s (string required)" % (k, type(v)))
                 continue
         elif vt==int:
-            v = parse_number(int, k, v)
+            v = parse_bool_or_number(int, k, v)
             if v==None:
                 continue
         elif vt==float:
@@ -529,8 +583,7 @@ def dict_to_validated_config(d, extras_defaults={}, extras_types={}, extras_vali
             options[v] = options[k]
     config = AdHocStruct()
     for k,v in options.items():
-        attr_name = k.replace("-", "_")
-        setattr(config, attr_name, v)
+        setattr(config, name_to_field(k), v)
     return config
 
 
@@ -538,8 +591,7 @@ def main():
     from xpra.util import nonl
     def print_options(o):
         for k,ot in sorted(OPTION_TYPES.items()):
-            attr_name = k.replace("-", "_")
-            v = getattr(o, attr_name, "")
+            v = getattr(o, name_to_field(k), "")
             if ot==bool and v is None:
                 v = "Auto"
             if type(v)==list:
