@@ -5,6 +5,7 @@
 
 import subprocess
 
+from xpra.util import nonl
 from xpra.log import Logger
 log = Logger("util")
 
@@ -15,6 +16,6 @@ def safe_exec(cmd, stdin=None, log_errors=True, **kwargs):
     l=log.debug
     if code!=0 and log_errors:
         l=log.error
-    l("signal_safe_exec(%s, %s, %s) stdout='%s'", cmd, stdin, kwargs, out)
-    l("signal_safe_exec(%s, %s, %s) stderr='%s'", cmd, stdin, kwargs, err)
+    l("signal_safe_exec(%s, %s, %s) stdout='%s'", cmd, stdin, kwargs, nonl(out))
+    l("signal_safe_exec(%s, %s, %s) stderr='%s'", cmd, stdin, kwargs, nonl(err))
     return  code, out, err
