@@ -8,14 +8,12 @@
 #  http://lists.partiwm.org/pipermail/parti-discuss/2008-September/000041.html
 #  http://lists.partiwm.org/pipermail/parti-discuss/2008-September/000042.html
 # (also do not import anything that imports gtk)
-import gobject
 import subprocess
 import sys
 import os.path
 import atexit
 import signal
 import socket
-import getpass
 import select
 import time
 import traceback
@@ -23,10 +21,6 @@ import traceback
 from xpra.scripts.main import TCP_NODELAY, warn
 from xpra.dotxpra import DotXpra, ServerSockInUse
 
-o0117 = 79
-o0177 = 127
-o0666 = 438
-o0700 = 448
 
 # use process polling with python versions older than 2.7 and 3.0, (because SIGCHLD support is broken)
 # or when the user requests it with the env var:
@@ -66,102 +60,6 @@ def deadly_signal(signum, frame):
     #kill(os.getpid(), signum)
     os._exit(128 + signum)
 
-# Note that this class has async subtleties -- e.g., it is possible for a
-# child to exit and us to receive the SIGCHLD before our fork() returns (and
-# thus before we even know the pid of the child).  So be careful:
-class ChildReaper(object):
-    def __init__(self, quit_cb):
-        self._quit = quit_cb
-        self._children_pids = {}
-        self._dead_pids = set()
-        self._ignored_pids = set()
-        from xpra.log import Logger
-        self._logger = Logger("server", "util")
-        if USE_PROCESS_POLLING:
-            POLL_DELAY = int(os.environ.get("XPRA_POLL_DELAY", 2))
-            self._logger.warn("Warning: outdated/buggy version of Python: %s", ".".join(str(x) for x in sys.version_info))
-            self._logger.warn("switching to process polling every %s seconds to support 'exit-with-children'", POLL_DELAY)
-            gobject.timeout_add(POLL_DELAY*1000, self.check)
-        else:
-            #with a less buggy python, we can just check the list of pids
-            #whenever we get a SIGCHLD
-            #however.. subprocess.Popen will no longer work as expected
-            #see: http://bugs.python.org/issue9127
-            #so we must ensure certain things that exec happen first:
-            from xpra.version_util import get_platform_info
-            get_platform_info()
-
-            signal.signal(signal.SIGCHLD, self.sigchld)
-            # Check once after the mainloop is running, just in case the exit
-            # conditions are satisfied before we even enter the main loop.
-            # (Programming with unix the signal API sure is annoying.)
-            def check_once():
-                self.check()
-                return False # Only call once
-            gobject.timeout_add(0, check_once)
-
-    def add_process(self, process, command, ignore=False):
-        process.command = command
-        assert process.pid>0
-        self._children_pids[process.pid] = process
-        if ignore:
-            self._ignored_pids.add(process.pid)
-        self._logger("add_process(%s, %s, %s) pid=%s", process, command, ignore, process.pid)
-
-    def check(self):
-        pids = set(self._children_pids.keys()) - self._ignored_pids
-        self._logger("check() pids=%s", pids)
-        if pids:
-            for pid, proc in self._children_pids.items():
-                if proc.poll() is not None:
-                    self.add_dead_pid(pid)
-            self._logger("check() pids=%s, dead_pids=%s", pids, self._dead_pids)
-            if pids.issubset(self._dead_pids):
-                self._quit()
-                return False
-        return True
-
-    def sigchld(self, signum, frame):
-        gobject.idle_add(self._logger, "sigchld(%s, %s)", signum, frame)
-        gobject.idle_add(self.reap)
-
-    def add_dead_pid(self, pid):
-        self._logger("add_dead_pid(%s)", pid)
-        if pid not in self._dead_pids:
-            proc = self._children_pids.get(pid)
-            if proc:
-                self._logger.info("child '%s' with pid %s has terminated", proc.command, pid)
-            self._dead_pids.add(pid)
-            self.check()
-
-    def reap(self):
-        if not self.check():
-            return
-        while True:
-            try:
-                pid, _ = os.waitpid(-1, os.WNOHANG)
-            except OSError:
-                break
-            self._logger("reap() waitpid=%s", pid)
-            if pid == 0:
-                break
-            self.add_dead_pid(pid)
-
-    def get_info(self):
-        d = dict(self._children_pids)
-        info = {"children"          : len(d),
-                "children.dead"     : len(self._dead_pids),
-                "children.ignored"  : len(self._ignored_pids)}
-        i = 0
-        for pid in sorted(d.keys()):
-            proc = d[pid]
-            info["child[%i].live" % i]  = pid not in self._dead_pids
-            info["child[%i].pid" % i]   = pid
-            info["child[%i].command" % i]   = proc.command
-            info["child[%i].ignored" % i] = pid in self._ignored_pids
-            i += 1
-        return info
-
 
 def save_xvfb_pid(pid):
     import gtk
@@ -252,16 +150,12 @@ def write_runner_shell_script(dotxpra, contents, overwrite=True):
         return
     # Write out a shell-script so that we can start our proxy in a clean
     # environment:
-    scriptfile = open(scriptpath, "w")
-    # Unix is a little silly sometimes:
-    umask = os.umask(0)
-    os.umask(umask)
-    if hasattr(os, "fchmod"):
-        os.fchmod(scriptfile.fileno(), o0700 & ~umask)
-    else:
-        os.chmod(scriptpath, o0700 & ~umask)
-    scriptfile.write(contents)
-    scriptfile.close()
+    with open(scriptpath, "w") as scriptfile:
+        # Unix is a little silly sometimes:
+        umask = os.umask(0)
+        os.umask(umask)
+        os.fchmod(scriptfile.fileno(), 0o700 & ~umask)
+        scriptfile.write(contents)
 
 
 def display_name_check(display_name):
@@ -290,7 +184,7 @@ MDNS_WARNING = False
 def mdns_publish(display_name, mode, listen_on, text_dict={}):
     try:
         from xpra.net.avahi_publisher import AvahiPublishers
-    except Exception, e:
+    except Exception as e:
         global MDNS_WARNING
         if not MDNS_WARNING:
             MDNS_WARNING = True
@@ -306,14 +200,27 @@ def mdns_publish(display_name, mode, listen_on, text_dict={}):
     _cleanups.append(ap.stop)
 
 
-def create_unix_domain_socket(sockpath, mmap_group):
+def create_unix_domain_socket(sockpath, mmap_group, socket_permissions):
+    if mmap_group:
+        #when using the mmap group option, use '660'
+        umask = 0o117
+    else:
+        #parse octal mode given as config option:
+        try:
+            if type(socket_permissions)==int:
+                sperms = socket_permissions
+            else:
+                #assume octal string:
+                sperms = int(socket_permissions, 8)
+            assert sperms>=0 and sperms<=0o777
+        except ValueError:
+            raise ValueError("invalid socket permissions (must be an octal number): '%s'" % socket_permissions)
+        #now convert this to a umask!
+        umask = 0o777-sperms
     listener = socket.socket(socket.AF_UNIX)
     listener.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
     #bind the socket, using umask to set the correct permissions
-    if mmap_group:
-        orig_umask = os.umask(o0117) #660
-    else:
-        orig_umask = os.umask(o0177) #600
+    orig_umask = os.umask(umask)
     listener.bind(sockpath)
     os.umask(orig_umask)
     return listener
@@ -362,7 +269,7 @@ def parse_bind_tcp(bind_tcp):
     return tcp_sockets
 
 
-def setup_local_socket(dotxpra, display_name, clobber, mmap_group):
+def setup_local_socket(dotxpra, display_name, clobber, mmap_group, socket_permissions):
     if sys.platform.startswith("win"):
         return None, None
     from xpra.log import Logger
@@ -375,9 +282,9 @@ def setup_local_socket(dotxpra, display_name, clobber, mmap_group):
         raise Exception("You already have an xpra server running at %s\n"
                      "  (did you want 'xpra upgrade'?)"
                      % (display_name,))
-    except Exception, e:
+    except Exception as e:
         raise Exception("socket path error: %s" % e)
-    sock = create_unix_domain_socket(sockpath, mmap_group)
+    sock = create_unix_domain_socket(sockpath, mmap_group, socket_permissions)
     def cleanup_socket():
         log.info("removing socket %s", sockpath)
         try:
@@ -387,6 +294,81 @@ def setup_local_socket(dotxpra, display_name, clobber, mmap_group):
     _cleanups.append(cleanup_socket)
     return ("unix-domain", sock), cleanup_socket
 
+
+def get_free_tcp_port():
+    s = socket.socket()
+    s.bind(('', 0))
+    port = s.getsockname()[1]
+    s.close()
+    return port
+
+def start_websockify(child_reaper, opts, tcp_sockets):
+    # start websockify?
+    if not opts.html:
+        return
+    html = opts.html
+    if type(html)==str:
+        html = html.lower()
+    from xpra.scripts.config import FALSE_OPTIONS, TRUE_OPTIONS
+    if html in FALSE_OPTIONS:
+        #html disabled
+        return
+    if opts.tcp_proxy:
+        raise Exception("cannot use tcp-proxy mode with html, use one or the other")
+        return 1
+    from xpra.platform.paths import get_resources_dir
+    www_dir = os.path.abspath(os.path.join(get_resources_dir(), "www"))
+    if not os.path.exists(www_dir):
+        raise Exception("cannot find xpra's html directory (not found in '%s')" % www_dir)
+    import websockify           #@UnresolvedImport
+    assert websockify
+    html_port = -1
+    html_host = "127.0.0.1"
+    if html not in TRUE_OPTIONS:
+        #we expect either HOST:PORT, or just PORT
+        if html.find(":")>=0:
+            html_host, html = html.split(":", 1)
+        try:
+            html_port = int(html)
+        except Exception as e:
+            raise Exception("invalid html port: %s" % e)
+        #we now have the host and port (port may be -1 to mean auto..)
+    if html_port==-1:
+        #try to find a free port and hope that websockify can then use it..
+        html_port = get_free_tcp_port()
+    if len(tcp_sockets)<1:
+        raise Exception("html web server requires at least one tcp socket, see 'bind-tcp'")
+    #use the first tcp socket for websockify to talk back to us:
+    _, xpra_tcp_port = list(tcp_sockets)[0]
+    from xpra.log import Logger
+    log = Logger("server")
+    websockify_command = ["websockify", "--web", www_dir, "%s:%s" % (html_host, html_port), "127.0.0.1:%s" % xpra_tcp_port]
+    log("websockify_command: %s", websockify_command)
+    websockify_proc = subprocess.Popen(websockify_command, close_fds=True)
+    websockify_proc._closed = False
+    start_time = time.time()
+    def websockify_ended(proc):
+        elapsed = time.time()-start_time
+        log("websockify_ended(%s) after %i seconds", elapsed)
+        if not websockify_proc._closed:
+            log.warn("Warning: websockify has terminated, the html web server will not be available.")
+            log.warn(" command used: %s", " ".join(websockify_command))
+        return False
+    child_reaper.add_process(websockify_proc, "websockify", websockify_command, ignore=True, callback=websockify_ended)
+    log.info("websockify started, serving %s on %s:%s", www_dir, html_host, html_port)
+    def cleanup_websockify():
+        log("cleanup_websockify() process.poll()=%s, pid=%s", websockify_proc.poll(), websockify_proc.pid)
+        if websockify_proc.poll() is None and not websockify_proc._closed:
+            log.info("stopping websockify with pid %s", websockify_proc.pid)
+            try:
+                websockify_proc._closed = True
+                websockify_proc.terminate()
+            except:
+                log.warn("error trying to stop websockify", exc_info=True)
+    _cleanups.append(cleanup_websockify)
+    opts.tcp_proxy = "%s:%s" % (html_host, html_port)
+
+
 def close_all_fds(exceptions=[]):
     fd_dirs = ["/dev/fd", "/proc/self/fd"]
     for fd_dir in fd_dirs:
@@ -416,7 +398,7 @@ def open_log_file(logpath):
     """
     if os.path.exists(logpath):
         os.rename(logpath, logpath + ".old")
-    return os.open(logpath, os.O_WRONLY | os.O_CREAT | os.O_TRUNC, o0666)
+    return os.open(logpath, os.O_WRONLY | os.O_CREAT | os.O_TRUNC, 0o666)
 
 def select_log_file(dotxpra, log_file, display_name):
     """ returns the log file path we should be using given the parameters,
@@ -521,43 +503,13 @@ def imsettings_env(disabled, gtk_im_module, qt_im_module, imsettings_module, xmo
     os.environ.update(v)
     return v
 
-def start_pulseaudio(child_reaper, pulseaudio_command):
-    from xpra.log import Logger
-    log = Logger("sound")
-    log("pulseaudio_command=%s", pulseaudio_command)
-    pa_proc = subprocess.Popen(pulseaudio_command, stdin=subprocess.PIPE, shell=True, close_fds=True)
-    child_reaper.add_process(pa_proc, "pulseaudio", ignore=True)
-    log.info("pulseaudio server started with pid %s", pa_proc.pid)
-    def check_pa_start():
-        if pa_proc.poll() is not None or pa_proc.pid in child_reaper._dead_pids:
-            log.warn("Warning: pulseaudio has terminated. Either fix the pulseaudio command line or use --no-pulseaudio to avoid this warning.")
-            log.warn(" usually, only a single pulseaudio instance can be running for each user account, and one may be running already")
-        return False
-    gobject.timeout_add(1000*2, check_pa_start)
-    def cleanup_pa():
-        log("cleanup_pa() process.poll()=%s, pid=%s, dead_pids=%s", pa_proc.poll(), pa_proc.pid, child_reaper._dead_pids)
-        if pa_proc.poll() is None and pa_proc.pid not in child_reaper._dead_pids:
-            log.info("stopping pulseaudio with pid %s", pa_proc.pid)
-            try:
-                #first we try pactl (required on Ubuntu):
-                from xpra.scripts.exec_util import safe_exec
-                r, _, _ = safe_exec(["pactl", "exit"])
-                #warning: pactl will return 0 whether it succeeds or not...
-                #but we can't kill the process because Ubuntu starts a new one
-                if r!=0:
-                    #fallback to using SIGINT:
-                    pa_proc.terminate()
-            except:
-                log.warn("error trying to stop pulseaudio", exc_info=True)
-    _cleanups.append(cleanup_pa)
-
 def start_Xvfb(xvfb_str, display_name):
     # We need to set up a new server environment
     xauthority = os.environ.get("XAUTHORITY", os.path.expanduser("~/.Xauthority"))
     if not os.path.exists(xauthority):
         try:
             open(xauthority, 'wa').close()
-        except Exception, e:
+        except Exception as e:
             #trying to continue anyway!
             sys.stderr.write("Error trying to create XAUTHORITY file %s: %s\n" % (xauthority, e))
 
@@ -584,6 +536,8 @@ def start_Xvfb(xvfb_str, display_name):
             os.setsid()
 
     xvfb_cmd = xvfb_str.split()
+    if not xvfb_cmd:
+        raise Exception("cannot start Xvfb, command definition is missing!")
     xvfb_executable = xvfb_cmd[0]
     if display_name[0]=='S':
         # 'S' means that we allocate the display automatically
@@ -598,7 +552,7 @@ def start_Xvfb(xvfb_str, display_name):
                 MAXFD = os.sysconf("SC_OPEN_MAX")
             except:
                 MAXFD = 256
-            for i in xrange(3, MAXFD):
+            for i in range(3, MAXFD):
                 if i not in (r_pipe, w_pipe):
                     try:
                         os.close(i)
@@ -652,7 +606,7 @@ def start_Xvfb(xvfb_str, display_name):
         code = subprocess.call(xauth_cmd)
         if code != 0:
             raise OSError("non-zero exit code: %s" % code)
-    except OSError, e:
+    except OSError as e:
         #trying to continue anyway!
         sys.stderr.write("Error running \"%s\": %s\n" % (" ".join(xauth_cmd), e))
     return xvfb, display_name
@@ -679,7 +633,7 @@ def verify_display_ready(xvfb, display_name, shadowing):
     # starting.  First wait for it to start up:
     try:
         wait_for_x_server(display_name, 3) # 3s timeout
-    except Exception, e:
+    except Exception as e:
         sys.stderr.write("%s\n" % e)
         return  None
     # Now we can safely load gtk and connect:
@@ -707,53 +661,6 @@ def verify_display_ready(xvfb, display_name, shadowing):
         return  None
     return display
 
-def find_lib(libname):
-    #it would be better to rely on dlopen to find the paths
-    #but I cannot find a way of getting ctypes to tell us the path
-    #it found the library in
-    libpaths = os.environ.get("LD_LIBRARY_PATH", "").split(":")
-    libpaths.append("/usr/lib64")
-    libpaths.append("/usr/lib")
-    for libpath in libpaths:
-        if not libpath or not os.path.exists(libpath):
-            continue
-        libname_so = os.path.join(libpath, libname)
-        if os.path.exists(libname_so):
-            return libname_so
-    return None
-
-def find_fakeXinerama():
-    return find_lib("libfakeXinerama.so.1")
-
-
-def start_children(child_reaper, commands, fake_xinerama):
-    assert os.name=="posix"
-    from xpra.log import Logger
-    log = Logger("server")
-    env = os.environ.copy()
-    #add fake xinerama:
-    if fake_xinerama:
-        libfakeXinerama_so = find_fakeXinerama()
-        if libfakeXinerama_so:
-            env["LD_PRELOAD"] = libfakeXinerama_so
-    #disable ubuntu's global menu using env vars:
-    env.update({
-        "UBUNTU_MENUPROXY"          : "",
-        "QT_X11_NO_NATIVE_MENUBAR"  : "1",
-        "MWNOCAPTURE"               : "true",
-        "MWNO_RIT"                  : "true",
-        "MWWM"                      : "allwm",
-    })
-    for child_cmd in commands:
-        if not child_cmd:
-            continue
-        try:
-            proc = subprocess.Popen(child_cmd, stdin=subprocess.PIPE, env=env, shell=True, close_fds=True)
-            child_reaper.add_process(proc, child_cmd)
-            log.info("started child '%s' with pid %s", child_cmd, proc.pid)
-        except OSError, e:
-            sys.stderr.write("Error spawning child '%s': %s\n" % (child_cmd, e))
-
 
 def run_server(error_cb, opts, mode, xpra_file, extra_args):
     if opts.encoding and opts.encoding=="help":
@@ -790,6 +697,10 @@ def run_server(error_cb, opts, mode, xpra_file, extra_args):
     proxying  = mode == "proxy"
     clobber   = upgrading or opts.use_display
 
+    if upgrading or shadowing:
+        #there should already be one running
+        opts.pulseaudio = False
+
     #get the display name:
     if shadowing and len(extra_args)==0:
         if sys.platform.startswith("win") or sys.platform.startswith("darwin"):
@@ -857,9 +768,10 @@ def run_server(error_cb, opts, mode, xpra_file, extra_args):
             #this can happen if stderr is closed by the caller already
             pass
 
-    # Write out a shell-script so that we can start our proxy in a clean
-    # environment:
-    write_runner_shell_script(dotxpra, script)
+    if os.name=="posix":
+        # Write out a shell-script so that we can start our proxy in a clean
+        # environment:
+        write_runner_shell_script(dotxpra, script)
 
     from xpra.log import Logger
     log = Logger("server")
@@ -876,7 +788,7 @@ def run_server(error_cb, opts, mode, xpra_file, extra_args):
             sockets.append(socket)
         if opts.mdns:
             mdns_recs.append(("tcp", bind_tcp))
-    except Exception, e:
+    except Exception as e:
         log.error("cannot start server: failed to setup sockets: %s", e)
         return 1
 
@@ -892,7 +804,7 @@ def run_server(error_cb, opts, mode, xpra_file, extra_args):
     if not shadowing and not proxying and not clobber:
         try:
             xvfb, display_name = start_Xvfb(opts.xvfb, display_name)
-        except OSError, e:
+        except OSError as e:
             log.error("Error starting Xvfb: %s\n", e)
             return  1
         xvfb_pid = xvfb.pid
@@ -913,7 +825,7 @@ def run_server(error_cb, opts, mode, xpra_file, extra_args):
         return  1
 
     #setup unix domain socket:
-    socket, cleanup_socket = setup_local_socket(dotxpra, display_name, clobber, opts.mmap_group)
+    socket, cleanup_socket = setup_local_socket(dotxpra, display_name, clobber, opts.mmap_group, opts.socket_permissions)
     if socket:      #win32 returns None!
         sockets.append(socket)
         if opts.mdns:
@@ -923,8 +835,9 @@ def run_server(error_cb, opts, mode, xpra_file, extra_args):
 
     #publish mdns records:
     if opts.mdns:
+        from xpra.platform.info import get_username
         mdns_info = {"display" : display_name,
-                     "username": getpass.getuser()}
+                     "username": get_username()}
         if opts.session_name:
             mdns_info["session"] = opts.session_name
         for mode, listen_on in mdns_recs:
@@ -947,17 +860,11 @@ def run_server(error_cb, opts, mode, xpra_file, extra_args):
     if shadowing:
         from xpra.platform.shadow_server import ShadowServer
         app = ShadowServer()
-        app.init(opts)
+        info = "shadow"
     elif proxying:
         from xpra.server.proxy_server import ProxyServer
         app = ProxyServer()
-        try:
-            app.init(opts)
-        except Exception, e:
-            log.error("Error: cannot start the proxy server")
-            log.error(str(e))
-            log.info("")
-            return 1
+        info = "proxy"
     else:
         assert starting or upgrading
         from xpra.x11.gtk_x11 import gdk_display_source
@@ -980,14 +887,43 @@ def run_server(error_cb, opts, mode, xpra_file, extra_args):
             from xpra.x11.server import XpraServer
             from xpra.x11.bindings.window_bindings import X11WindowBindings     #@UnresolvedImport
             X11Window = X11WindowBindings()
-        except ImportError, e:
+        except ImportError as e:
             log.error("Failed to load Xpra server components, check your installation: %s" % e)
             return 1
         if not X11Window.displayHasXComposite():
             log.error("Xpra is a compositing manager, it cannot use a display which lacks the XComposite extension!")
             return 1
-        app = XpraServer()
-        app.init(clobber, opts)
+        app = XpraServer(clobber)
+        info = "xpra"
+
+    try:
+        app.init(opts)
+    except Exception as e:
+        log.error("Error: cannot start the %s server", info, exc_info=True)
+        log.error(str(e))
+        log.info("")
+        return 1
+
+    #honour start child, html webserver, and setup child reaper
+    if os.name=="posix" and not proxying and not upgrading and not shadowing:
+        # start websockify?
+        try:
+            start_websockify(app.child_reaper, opts, bind_tcp)
+            #websockify overrides the tcp proxy, so we must re-set it:
+            app._tcp_proxy = opts.tcp_proxy
+        except Exception as e:
+            error_cb("failed to setup websockify html server: %s" % e)
+        if opts.exit_with_children:
+            assert opts.start_child, "exit-with-children was specified but start-child is missing!"
+        if opts.start:
+            for x in opts.start:
+                if x:
+                    app.start_child(x, x, True)
+        if opts.start_child:
+            for x in opts.start_child:
+                if x:
+                    app.start_child(x, x, False)
+
     log("%s(%s)", app.init_sockets, sockets)
     app.init_sockets(sockets)
     log("%s(%s)", app.init_when_ready, _when_ready)
@@ -1006,21 +942,6 @@ def run_server(error_cb, opts, mode, xpra_file, extra_args):
     if xvfb_pid is not None and not opts.use_display and not shadowing:
         _cleanups.append(kill_xvfb)
 
-    if os.name=="posix" and not proxying and not upgrading and not shadowing:
-        def reaper_quit():
-            if opts.exit_with_children:
-                log.info("all children have exited and --exit-with-children was specified, exiting")
-                gobject.idle_add(app.reaper_quit)
-        child_reaper = ChildReaper(reaper_quit)
-        if not upgrading and not shadowing and opts.pulseaudio and len(opts.pulseaudio_command)>0:
-            start_pulseaudio(child_reaper, opts.pulseaudio_command)
-        if opts.exit_with_children:
-            assert opts.start_child, "exit-with-children was specified but start-child is missing!"
-        if opts.start_child:
-            assert os.name=="posix", "start-child cannot be used on %s" % os.name
-            start_children(child_reaper, opts.start_child, (opts.fake_xinerama and not shadowing))
-        app.child_reaper = child_reaper
-
     try:
         log("running %s", app.run)
         e = app.run()
@@ -1028,7 +949,7 @@ def run_server(error_cb, opts, mode, xpra_file, extra_args):
     except KeyboardInterrupt:
         log.info("stopping on KeyboardInterrupt")
         e = 0
-    except Exception, e:
+    except Exception as e:
         log.error("server error", exc_info=True)
         e = -128
     if e>0:
