@@ -10,6 +10,9 @@ assert init
 
 class Authenticator(SysAuthenticator):
 
+    def __repr__(self):
+        return "Allow Authenticator"
+
     def get_challenge(self):
         return get_hex_uuid(), "hmac"
 
