@@ -10,3 +10,6 @@ def init(opts):
 class Authenticator(object):
     def __init__(self, username):
         raise Exception("failing")
+
+    def __repr__(self):
+        return "Fail Authenticator"
