@@ -4,21 +4,16 @@
 # later version. See the file COPYING for details.
 
 from xpra.server.auth.sys_auth_base import SysAuthenticator, init
-import os
+from xpra.platform.info import get_username
 
+assert init
 
 class Authenticator(SysAuthenticator):
 
     def __init__(self, username):
         self.salt = None
         self.pw = None
-        try:
-            import pwd
-            self.pw = pwd.getpwuid(os.getuid())
-            self.username = self.pw.pw_name
-        except:
-            import getpass
-            self.username = getpass.getuser()
+        self.username = get_username()
 
     def requires_challenge(self):
         return False
@@ -31,3 +26,6 @@ class Authenticator(SysAuthenticator):
 
     def authenticate(self, challenge_response, client_salt):
         return True
+
+    def __repr__(self):
+        return "None Authenticator"
