@@ -49,7 +49,7 @@ try:
                         resp.append(('', 0))
                     else:
                         log.error("pam_conf_password unknown type: '%s'", pam_type)
-            except Exception, e:
+            except Exception as e:
                 log.error("pam_conv_password error: %s", e)
             return    resp
 
@@ -64,13 +64,13 @@ try:
             auth.authenticate()
             return    True
             #auth.acct_mgmt()
-        except PAM.error, resp:
+        except PAM.error as resp:
             log.error("PAM.authenticate() error: %s", resp)
             return    False
-        except Exception, e:
+        except Exception as e:
             log.error("PAM.authenticate() internal error: %s", e)
             return    False
-except Exception, e:
+except Exception as e:
     log("PAM module not available: %s", e)
 
 try:
@@ -94,7 +94,7 @@ class Authenticator(SysAuthenticator):
             return False
         return check(self.username, password)
 
-    def __str__(self):
+    def __repr__(self):
         return "PAM Authenticator"
 
 
