@@ -23,5 +23,5 @@ class Authenticator(SysAuthenticator):
     def check(self, password):
         win32security.LogonUser(self.username, '', password, win32security.LOGON32_LOGON_NETWORK, win32security.LOGON32_PROVIDER_DEFAULT)
 
-    def __str__(self):
+    def __repr__(self):
         return "Win32 Authenticator"
