@@ -26,6 +26,9 @@ class Worker_Thread(Thread):
         self.exit = False
         self.setDaemon(True)
 
+    def __repr__(self):
+        return "Worker_Thread(items=%s, exit=%s)" % (self.items.qsize(), self.exit)
+
     def stop(self, force=False):
         if self.exit:
             return
@@ -65,22 +68,22 @@ lock = Lock()
 
 def get_worker(create=True):
     global singleton
-    #fast path:
+    #fast path (no lock):
     if singleton is not None or not create:
         return singleton
-    try:
-        lock.acquire()
+    with lock:
         if not singleton:
             singleton = Worker_Thread()
             singleton.start()
-    finally:
-        lock.release()
     return singleton
 
 def add_work_item(item):
-    get_worker().add(item)
+    w = get_worker(True)
+    debug("add_work_item(%s) worker=%s", item, w)
+    w.add(item)
 
 def stop_worker(force=False):
     w = get_worker(False)
+    debug("stop_worker(%s) worker=%s", force, w)
     if w:
         w.stop(force)
