@@ -12,13 +12,13 @@ gobject.threads_init()
 
 from xpra.log import Logger
 log = Logger("server", "gtk")
+screenlog = Logger("server", "screen")
 
 from xpra.gtk_common.quit import (gtk_main_quit_really,
                            gtk_main_quit_on_fatal_exceptions_enable,
                            gtk_main_quit_on_fatal_exceptions_disable)
 from xpra.server.server_base import ServerBase
-from xpra.gtk_common.gtk_util import get_gtk_version_info
-from xpra.gtk_common.gtk2common import gtk2main
+from xpra.gtk_common.gtk_util import get_gtk_version_info, gtk_main
 from xpra.util import updict
 
 
@@ -53,8 +53,8 @@ class GTKServerBase(ServerBase):
 
     def do_run(self):
         gtk_main_quit_on_fatal_exceptions_enable()
-        log("do_run() calling %s", gtk2main)
-        gtk2main()
+        log("do_run() calling %s", gtk_main)
+        gtk_main()
         log("do_run() end of gtk.main()")
 
     def add_listen_socket(self, socktype, sock):
@@ -82,8 +82,7 @@ class GTKServerBase(ServerBase):
 
     def do_get_info(self, proto, *args):
         info = ServerBase.do_get_info(self, proto, *args)
-        ss = self._server_sources.get(proto)
-        updict(info, "server", get_gtk_version_info(new_namespace=(ss is not None) and ss.namespace))
+        updict(info, "server", get_gtk_version_info())
         info.update({
                      "server.type"      : "Python/gtk-x11",
                      "features.randr"   : self.randr})
@@ -101,11 +100,11 @@ class GTKServerBase(ServerBase):
         return root_w, root_h
 
     def calculate_workarea(self, maxw, maxh):
-        log("calculate_workarea(%s, %s)", maxw, maxh)
+        screenlog("calculate_workarea(%s, %s)", maxw, maxh)
         workarea = gtk.gdk.Rectangle(0, 0, maxw, maxh)
         for ss in self._server_sources.values():
             screen_sizes = ss.screen_sizes
-            log("calculate_workarea() screen_sizes(%s)=%s", ss, screen_sizes)
+            screenlog("calculate_workarea() screen_sizes(%s)=%s", ss, screen_sizes)
             if not screen_sizes:
                 continue
             for display in screen_sizes:
@@ -116,11 +115,11 @@ class GTKServerBase(ServerBase):
                 if len(display)>=10:
                     work_x, work_y, work_w, work_h = display[6:10]
                     display_workarea = gtk.gdk.Rectangle(work_x, work_y, work_w, work_h)
-                    log("calculate_workarea() found %s for display %s", display_workarea, display[0])
+                    screenlog("calculate_workarea() found %s for display %s", display_workarea, display[0])
                     workarea = workarea.intersect(display_workarea)
         #sanity checks:
         if workarea.width==0 or workarea.height==0:
-            log.warn("failed to calculate a common workarea - using the full display area")
+            screenlog.warn("failed to calculate a common workarea - using the full display area")
             workarea = gtk.gdk.Rectangle(0, 0, maxw, maxh)
         self.set_workarea(workarea)
 
