@@ -11,16 +11,11 @@ from xpra.log import Logger
 log = Logger("window", "encoding")
 
 from xpra.net import compression
-try:
-    from xpra.codecs.argb.argb import bgra_to_rgb, bgra_to_rgba, argb_to_rgb, argb_to_rgba   #@UnresolvedImport
-except Exception, e:
-    log("cannot load argb module: %s", e)
-    bgra_to_rgb, bgra_to_rgba, argb_to_rgb, argb_to_rgba = (None,)*4
-from xpra.codecs.codec_constants import get_PIL_encodings
-from xpra.os_util import StringIOClass, memoryview_to_bytes
+from xpra.codecs.argb.argb import bgra_to_rgb, bgra_to_rgba, argb_to_rgb, argb_to_rgba, restride_image   #@UnresolvedImport
+from xpra.os_util import StringIOClass
 from xpra.codecs.loader import get_codec, get_codec_version
-from xpra.os_util import builtins
-_memoryview = builtins.__dict__.get("memoryview")
+from xpra.codecs.codec_constants import get_PIL_encodings
+from xpra.os_util import memoryview_to_bytes
 try:
     from xpra.net.mmap_pipe import mmap_write
 except:
@@ -84,9 +79,9 @@ def webp_encode(coding, image, rgb_formats, supports_transparency, quality, spee
 def roundup(n, m):
     return (n + m - 1) & ~(m - 1)
 
-def rgb_encode(coding, image, rgb_formats, supports_transparency, speed, rgb_zlib=True, rgb_lz4=True, rgb_lzo=False, encoding_client_options=True, supports_rgb24zlib=True):
+def rgb_encode(coding, image, rgb_formats, supports_transparency, speed, rgb_zlib=True, rgb_lz4=True, rgb_lzo=False):
     pixel_format = image.get_pixel_format()
-    #log("rgb_encode%s pixel_format=%s, rgb_formats=%s", (coding, image, rgb_formats, supports_transparency, speed, rgb_zlib, rgb_lz4, encoding_client_options, supports_rgb24zlib), pixel_format, rgb_formats)
+    #log("rgb_encode%s pixel_format=%s, rgb_formats=%s", (coding, image, rgb_formats, supports_transparency, speed, rgb_zlib, rgb_lz4), pixel_format, rgb_formats)
     if pixel_format not in rgb_formats:
         if not rgb_reformat(image, rgb_formats, supports_transparency):
             raise Exception("cannot find compatible rgb format to use for %s! (supported: %s)" % (pixel_format, rgb_formats))
@@ -99,75 +94,60 @@ def rgb_encode(coding, image, rgb_formats, supports_transparency, speed, rgb_zli
             coding = "rgb24"
     #always tell client which pixel format we are sending:
     options = {"rgb_format" : pixel_format}
+
+    #we may want to re-stride:
+    restride_image(image)
+
     #compress here and return a wrapper so network code knows it is already zlib compressed:
     pixels = image.get_pixels()
-
-    #special case for when rowstride is so much bigger than the width
-    #that we would end up sending large chunks of padding with each row of pixels
-    #this happens with XShm pixel data (the default)
-    stride = image.get_rowstride()
     width = image.get_width()
-    rstride = roundup(width*len(pixel_format), 4)   #a reasonable stride: rounded up to 4
     height = image.get_height()
-    if stride>8 and rstride<stride:
-        al = len(pixels)                    #current buffer size
-        el = rstride*height                 #desirable size we could have
-        if al-el>1024 and el*110/100<al:    #is it worth re-striding to save space?
-            #we'll save at least 1KB and 10%, do it
-            #Note: we could also change the pixel format whilst we're at it
-            # and convert BGRX to RGB for example (assuming RGB is also supported by the client)
-            rows = []
-            for y in range(height):
-                rows.append(memoryview_to_bytes(pixels[stride*y:stride*y+rstride]))
-            pixels = "".join(rows)
-            log("rgb_encode: %s pixels re-stride saving %i%% from %s (%s bytes) to %s (%s bytes)", pixel_format, 100-100*el/al, stride, al, rstride, el)
-            stride = rstride
+    stride = image.get_rowstride()
 
-    #compression
+    #compression stage:
     #by default, wire=raw:
-    raw_data = str(pixels)
-    wire_data = raw_data
+    raw_data = pixels
     level = 0
     algo = "not"
     if len(pixels)>=256 and (rgb_zlib and compression.use_zlib) or (rgb_lz4 and compression.use_lz4) or (rgb_lzo and compression.use_lzo):
         level = max(0, min(5, int(115-speed)/20))
         if len(pixels)<1024:
             #fewer pixels, make it more likely we won't bother compressing:
-            level = level / 2
+            level = level // 2
     if level>0:
         if rgb_lz4 and compression.use_lz4:
-            wire_data = compression.compressed_wrapper(coding, pixels, lz4=True)
+            cwrapper = compression.compressed_wrapper(coding, raw_data, lz4=True)
             algo = "lz4"
             level = 1
         elif rgb_lzo and compression.use_lzo:
-            wire_data = compression.compressed_wrapper(coding, pixels, lzo=True)
+            cwrapper = compression.compressed_wrapper(coding, raw_data, lzo=True)
             algo = "lzo"
             level = 1
-        else:
-            assert rgb_zlib and compression.use_zlib
-            wire_data = compression.compressed_wrapper(coding, pixels, zlib=True, level=level)
+        elif rgb_zlib and compression.use_zlib:
+            cwrapper = compression.compressed_wrapper(coding, raw_data, zlib=True, level=level)
             algo = "zlib"
-        raw_data = wire_data.data
-        #log("%s/%s data compressed from %s bytes down to %s (%s%%) with lz4=%s",
-        #         coding, pixel_format, len(pixels), len(raw_data), int(100.0*len(raw_data)/len(pixels)), self.rgb_lz4)
-        if len(raw_data)>=(len(pixels)-32):
-            #compressed is actually bigger! (use uncompressed)
+        else:
+            cwrapper = None
+        if cwrapper is None or len(cwrapper)>=(len(raw_data)-32):
+            #compressed is actually bigger! (fall through to uncompressed)
             level = 0
-            wire_data = str(pixels)
-            raw_data = wire_data
         else:
             #add compressed marker:
             options[algo] = level
+            #remove network layer compression marker
+            #so that this data will be decompressed by the decode thread client side:
+            cwrapper.level = 0
+    if level==0:
+        #can't pass a raw buffer to bencode / rencode:
+        cwrapper = compression.Compressed(coding, memoryview_to_bytes(raw_data), True)
     if pixel_format.upper().find("A")>=0 or pixel_format.upper().find("X")>=0:
         bpp = 32
     else:
         bpp = 24
-    log("rgb_encode using level=%s, %s compressed %sx%s in %s/%s: %s bytes down to %s", level, algo, image.get_width(), image.get_height(), coding, pixel_format, len(pixels), len(raw_data))
-    if not encoding_client_options or not supports_rgb24zlib:
-        return  coding, wire_data, {}, width, height, stride, bpp
+    log("rgb_encode using level=%s, %s compressed %sx%s in %s/%s: %s bytes down to %s", level, algo, image.get_width(), image.get_height(), coding, pixel_format, len(pixels), len(cwrapper.data))
     #wrap it using "Compressed" so the network layer receiving it
     #won't decompress it (leave it to the client's draw thread)
-    return coding, compression.Compressed(coding, raw_data, True), options, width, height, stride, bpp
+    return coding, cwrapper, options, width, height, stride, bpp
 
 
 def PIL_encode(coding, image, quality, speed, supports_transparency):
@@ -184,19 +164,17 @@ def PIL_encode(coding, image, quality, speed, supports_transparency):
     bpp = 32
     #remove transparency if it cannot be handled:
     try:
+        #PIL cannot use the memoryview directly:
+        pixels = memoryview_to_bytes(image.get_pixels())
         #it is safe to use frombuffer() here since the convert()
         #calls below will not convert and modify the data in place
         #and we save the compressed data then discard the image
-        pixels = image.get_pixels()
-        if _memoryview and isinstance(pixels, _memoryview):
-            #PIL cannot use the memoryview directly:
-            pixels = pixels.tobytes()
         im = PIL.Image.frombuffer(rgb, (w, h), pixels, "raw", pixel_format, image.get_rowstride())
         if coding.startswith("png") and not supports_transparency and rgb=="RGBA":
             im = im.convert("RGB")
             rgb = "RGB"
             bpp = 24
-    except Exception, e:
+    except Exception as e:
         log.error("PIL_encode(%s) converting to %s failed", (w, h, coding, "%s bytes" % image.get_size(), pixel_format, image.get_rowstride()), rgb, exc_info=True)
         raise e
     buf = StringIOClass()
@@ -269,10 +247,6 @@ def PIL_encode(coding, image, quality, speed, supports_transparency):
 def argb_swap(image, rgb_formats, supports_transparency):
     """ use the argb codec to do the RGB byte swapping """
     pixel_format = image.get_pixel_format()
-    if None in (bgra_to_rgb, bgra_to_rgba, argb_to_rgb, argb_to_rgba):
-        warn_encoding_once("argb-module-missing", "no argb module, cannot convert %s to one of: %s" % (pixel_format, rgb_formats))
-        return False
-
     #try to fallback to argb module
     #if we have one of the target pixel formats:
     pixels = image.get_pixels()
@@ -344,9 +318,8 @@ def rgb_reformat(image, rgb_formats, supports_transparency):
     start = time.time()
     w = image.get_width()
     h = image.get_height()
-    if _memoryview and isinstance(pixels, _memoryview):
-        #PIL cannot use the memoryview directly:
-        pixels = pixels.tobytes()
+    #PIL cannot use the memoryview directly:
+    pixels = memoryview_to_bytes(pixels)
     img = PIL.Image.frombuffer(target_format, (w, h), pixels, "raw", input_format, image.get_rowstride())
     rowstride = w*len(target_format)    #number of characters is number of bytes per pixel!
     #use tobytes() if present, fallback to tostring():
