@@ -22,6 +22,7 @@ from xpra.net import compression
 from xpra.net.compression import Compressed, compressed_wrapper
 from xpra.net.protocol import Protocol, get_network_caps
 from xpra.net.crypto import new_cipher_caps
+from xpra.codecs.loader import load_codecs
 from xpra.codecs.image_wrapper import ImageWrapper
 from xpra.codecs.video_helper import getVideoHelper, PREFERRED_ENCODER_ORDER
 from xpra.os_util import Queue, SIGNAMES
@@ -175,7 +176,7 @@ class ProxyInstanceProcess(Process):
         try:
             try:
                 self.run_queue()
-            except KeyboardInterrupt, e:
+            except KeyboardInterrupt as e:
                 self.stop(str(e))
         finally:
             log("ProxyProcess.run() ending %s", os.getpid())
@@ -219,7 +220,7 @@ class ProxyInstanceProcess(Process):
             if x not in order:
                 order.append(x)
         self.video_encoder_types = [x for x in order if x in encoder_types]
-        log.info("proxy video encoders: %s", self.video_encoder_types)
+        log.info("proxy video encoders: %s", ", ".join(self.video_encoder_types))
 
 
     def create_control_socket(self):
@@ -231,10 +232,10 @@ class ProxyInstanceProcess(Process):
             log.warn("You already have a proxy server running at %s, the control socket will not be created!", sockpath)
             return False
         try:
-            sock = create_unix_domain_socket(sockpath, None)
+            sock = create_unix_domain_socket(sockpath, None, 0o600)
             sock.listen(5)
-        except Exception, e:
-            log.warn("failed to setup control socket %s: %s", sockpath, e)
+        except:
+            log.warn("failed to setup control socket %s", sockpath, exc_info=True)
             return False
         self.control_socket = sock
         self.control_socket_path = sockpath
@@ -278,7 +279,7 @@ class ProxyInstanceProcess(Process):
     def process_control_packet(self, proto, packet):
         try:
             self.do_process_control_packet(proto, packet)
-        except Exception, e:
+        except Exception as e:
             log.error("error processing control packet", exc_info=True)
             self.send_disconnect(proto, CONTROL_COMMAND_ERROR, str(e))
 
@@ -347,7 +348,7 @@ class ProxyInstanceProcess(Process):
                 log("trying to add %s=%s using %s", k, v, parser)
                 try:
                     d[k] = parser(k, v)
-                except Exception, e:
+                except Exception as e:
                     log.warn("failed to parse value %s for %s using %s: %s", v, k, parser, e)
         return d
 
@@ -480,6 +481,23 @@ class ProxyInstanceProcess(Process):
         log("sending to server: %s", p[0])
         return p, None, None, self.server_packets.qsize()>0
 
+
+    def _packet_recompress(self, packet, index, name):
+        if len(packet)>index:
+            data = packet[index]
+            if len(data)<512:
+                packet[8] = str(data)
+                return
+            #FIXME: this is ugly and not generic!
+            zlib = compression.use_zlib and self.caps.get("zlib", True)
+            lz4 = compression.use_lz4 and self.caps.get("lz4", False)
+            lzo = compression.use_lzo and self.caps.get("lzo", False)
+            if zlib or lz4 or lzo:
+                packet[index] = compressed_wrapper(name, data, zlib=zlib, lz4=lz4, lzo=lzo, can_inline=False)
+            else:
+                #prevent warnings about large uncompressed data
+                packet[index] = Compressed("raw %s" % name, data, can_inline=True)
+
     def process_server_packet(self, proto, packet):
         packet_type = packet[0]
         log("process_server_packet: %s", packet_type)
@@ -527,20 +545,9 @@ class ProxyInstanceProcess(Process):
             #packet = ["cursor", x, y, width, height, xhot, yhot, serial, pixels, name]
             #or:
             #packet = ["cursor", ""]
-            if len(packet)>=9:
-                pixels = packet[8]
-                if len(pixels)<512:
-                    packet[8] = str(pixels)
-                else:
-                    #FIXME: this is ugly and not generic!
-                    zlib = compression.use_zlib and self.caps.get("zlib", True) 
-                    lz4 = compression.use_lz4 and self.caps.get("lz4", False) 
-                    lzo = compression.use_lzo and self.caps.get("lzo", False)
-                    if zlib or lz4 or lzo:
-                        packet[8] = compressed_wrapper("cursor", pixels, zlib=zlib, lz4=lz4, lzo=lzo, can_inline=False)
-                    else:
-                        #prevent warnings about large uncompressed data
-                        packet[8] = Compressed("raw cursor", pixels, can_inline=True)
+            self._packet_recompress(packet, 8, "cursor")
+        elif packet_type=="window-icon":
+            self._packet_recompress(packet, 5, "icon")
         self.queue_client_packet(packet)
 
 
@@ -594,12 +601,12 @@ class ProxyInstanceProcess(Process):
         rgb_format = client_options.get("rgb_format", "")
         log("proxy draw: client_options=%s", client_options)
 
-        def send_updated(encoding, compressed_data, client_options):
+        def send_updated(encoding, compressed_data, updated_client_options):
             #update the packet with actual encoding data used:
             packet[6] = encoding
             packet[7] = compressed_data
-            packet[10] = client_options
-            log("returning %s bytes from %s", len(compressed_data), len(pixels))
+            packet[10] = updated_client_options
+            log("returning %s bytes from %s, options=%s", len(compressed_data), len(pixels), updated_client_options)
             return (wid not in self.lost_windows)
 
         def passthrough(strip_alpha=True):
@@ -694,16 +701,16 @@ class ProxyInstanceProcess(Process):
             ve.init_context(width, height, rgb_format, dst_formats, encoding, quality, speed, scaling, {})
             self.video_encoders[wid] = ve
             self.video_encoders_last_used_time[wid] = time.time()       #just to make sure this is always set
-        else:
-            if quality>=0:
-                ve.set_encoding_quality(quality)
-            if speed>=0:
-                ve.set_encoding_speed(speed)
         #actual video compression:
         log("proxy compression using %s with quality=%s, speed=%s", ve, quality, speed)
-        data, client_options = ve.compress_image(image, encoder_options)
+        data, out_options = ve.compress_image(image, quality, speed, encoder_options)
+        #pass through some options if we don't have them from the encoder
+        #(maybe we should also use the "pts" from the real server?)
+        for k in ("timestamp", "rgb_format", "depth", "csc"):
+            if k not in out_options and k in client_options:
+                out_options[k] = client_options[k]
         self.video_encoders_last_used_time[wid] = time.time()
-        return send_updated(ve.get_encoding(), Compressed(encoding, data), client_options)
+        return send_updated(ve.get_encoding(), Compressed(encoding, data), out_options)
 
     def timeout_video_encoders(self):
         #have to be careful as another thread may come in...
