@@ -1,7 +1,7 @@
 # coding=utf8
 # This file is part of Xpra.
 # Copyright (C) 2011 Serviware (Arthur Huillet, <ahuillet@serviware.com>)
-# Copyright (C) 2010-2014 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2010-2015 Antoine Martin <antoine@devloop.org.uk>
 # Copyright (C) 2008 Nathaniel Smith <njs@pobox.com>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
@@ -18,6 +18,8 @@ import thread
 
 from xpra.log import Logger
 log = Logger("server")
+netlog = Logger("network")
+proxylog = Logger("proxy")
 commandlog = Logger("command")
 
 import xpra
@@ -29,14 +31,15 @@ from xpra.net.bytestreams import SocketConnection
 from xpra.platform import set_application_name
 from xpra.os_util import load_binary_file, get_machine_id, get_user_uuid, SIGNAMES, Queue
 from xpra.version_util import version_compat_check, get_version_info, get_platform_info, get_host_info, local_version
-from xpra.net.protocol import Protocol, get_network_caps
+from xpra.net.protocol import Protocol, get_network_caps, sanity_checks
 from xpra.net.crypto import new_cipher_caps
-from xpra.server.background_worker import stop_worker
+from xpra.server.background_worker import stop_worker, get_worker
 from xpra.daemon_thread import make_daemon_thread
 from xpra.server.proxy import XpraProxy
 from xpra.util import typedict, updict, repr_ellipsized, \
         SERVER_SHUTDOWN, SERVER_EXIT, LOGIN_TIMEOUT, DONE, PROTOCOL_ERROR, SERVER_ERROR, VERSION_ERROR
 
+main_thread = threading.current_thread()
 
 MAX_CONCURRENT_CONNECTIONS = 20
 
@@ -51,30 +54,59 @@ def get_server_info():
     up("build",     get_version_info())
     return info
 
-def get_thread_info(proto=None):
+def get_thread_info(proto=None, protocols=[]):
     #threads:
     info_threads = proto.get_threads()
     info = {
-            "count"        : threading.activeCount() - len(info_threads),
+            "count"        : threading.active_count() - len(info_threads),
             "info.count"   : len(info_threads)
             }
-    i = 0
+    thread_ident = {
+            threading.current_thread().ident    : "info",
+            main_thread.ident                   : "main",
+            }
+    w = get_worker(False)
+    if w:
+        thread_ident[w.ident] = "worker"
+
     #threads used by the "info" client:
-    for t in info_threads:
+    for i, t in enumerate(info_threads):
         info["info[%s]" % i] = t.getName()
-        i += 1
-    i = 0
+        thread_ident[t.ident] = t.getName()
+    for p in protocols:
+        try:
+            threads = p.get_threads()
+            for t in threads:
+                thread_ident[t.ident] = t.getName()
+        except:
+            pass
     #all non-info threads:
-    for t in threading.enumerate():
-        if t not in info_threads:
-            info[str(i)] = t.getName()
-            i += 1
+    for i, t in enumerate((x for x in threading.enumerate() if x not in info_threads)):
+        info[str(i)] = t.getName()
     #platform specific bits:
     try:
         from xpra.platform.info import get_sys_info
         info.update(get_sys_info())
     except:
         log.error("error getting system info", exc_info=True)
+    #extract frame info:
+    try:
+        def nn(x):
+            if x is None:
+                return ""
+            return x
+        import traceback
+        frames = sys._current_frames()
+        for i,frame_pair in enumerate(frames.items()):
+            stack = traceback.extract_stack(frame_pair[1])
+            info["frame[%s].thread" % i] = thread_ident.get(frame_pair[0], "unknown")
+            #sanitize stack to prevent None values (which cause encoding errors with the bencoder)
+            sanestack = []
+            for e in stack:
+                sanestack.append(tuple([nn(x) for x in e]))
+            info["frame[%s].stack" % i] = sanestack
+    except Exception as e:
+        log.error("failed to get frame info: %s", e)
     return info
 
 
@@ -92,6 +124,7 @@ class ServerCore(object):
         log("ServerCore.__init__()")
         self.start_time = time.time()
         self.auth_class = None
+        self.tcp_auth_class = None
         self._when_ready = []
         self.child_reaper = None
 
@@ -105,6 +138,7 @@ class ServerCore(object):
         self.socket_types = {}
         self._max_connections = MAX_CONCURRENT_CONNECTIONS
         self._socket_timeout = 0.1
+        self._socket_dir = None
 
         self.session_name = ""
 
@@ -120,6 +154,7 @@ class ServerCore(object):
 
         self.init_packet_handlers()
         self.init_aliases()
+        sanity_checks()
 
     def idle_add(self, *args, **kwargs):
         raise NotImplementedError()
@@ -135,6 +170,7 @@ class ServerCore(object):
         self.session_name = opts.session_name
         set_application_name(self.session_name or "Xpra")
 
+        self._socket_dir = opts.socket_dir
         self._tcp_proxy = opts.tcp_proxy
         self.encryption_keyfile = opts.encryption_keyfile
         self.password_file = opts.password_file
@@ -144,53 +180,59 @@ class ServerCore(object):
         self.init_auth(opts)
 
     def init_auth(self, opts):
-        auth = opts.auth
+        self.auth_class = self.get_auth_module(opts.auth, opts)
+        self.tcp_auth_class = self.get_auth_module(opts.tcp_auth or opts.auth, opts)
+        log("init_auth(%s) auth class=%s, tcp auth class=%s", opts, self.auth_class, self.tcp_auth_class)
+
+    def get_auth_module(self, auth, opts):
         if not auth and opts.password_file:
             log.warn("no authentication module specified with 'password_file', using 'file' based authentication")
             auth = "file"
+        if not auth and os.environ.get('XPRA_PASSWORD'):
+            log.warn("no authentication module specified with 'XPRA_PASSWORD', using 'file' based authentication")
+            auth = "file"
         if auth=="":
-            return
+            return None
         elif auth=="sys":
+            #resolve virtual "sys" auth:
             if sys.platform.startswith("win"):
                 auth = "win32"
             else:
                 auth = "pam"
             log("will try to use sys auth module '%s' for %s", auth, sys.platform)
-        if auth=="fail":
-            from xpra.server.auth import fail_auth
-            auth_module = fail_auth
-        elif auth=="reject":
-            from xpra.server.auth import reject_auth
-            auth_module = reject_auth
-        elif auth=="allow":
-            from xpra.server.auth import allow_auth
-            auth_module = allow_auth
-        elif auth=="none":
-            from xpra.server.auth import none_auth
-            auth_module = none_auth
-        elif auth=="file":
-            from xpra.server.auth import file_auth
-            auth_module = file_auth
-        elif auth=="pam":
+        from xpra.server.auth import fail_auth, reject_auth, allow_auth, none_auth, file_auth
+        AUTH_MODULES = {
+                        "fail"      : fail_auth,
+                        "reject"    : reject_auth,
+                        "allow"     : allow_auth,
+                        "none"      : none_auth,
+                        "file"      : file_auth
+                        }
+        try:
             from xpra.server.auth import pam_auth
-            auth_module = pam_auth
-        elif auth=="win32":
+            AUTH_MODULES["pam"] = pam_auth
+        except Exception as e:
+            log("cannot load pam auth: %s", e)
+        try:
             from xpra.server.auth import win32_auth
-            auth_module = win32_auth
-        else:
-            raise Exception("invalid auth module: %s" % auth)
+            AUTH_MODULES["win32"] = win32_auth
+        except Exception as e:
+            log("cannot load win32 auth: %s", e)
+        auth_module = AUTH_MODULES.get(auth.lower())
+        if not auth_module:
+            raise Exception("cannot find authentication module '%s' (supported: %s)", auth, AUTH_MODULES.keys())
         try:
             auth_module.init(opts)
-        except Exception, e:
-            raise Exception("failed to initialize %s module: %s" % (auth_module, e))
-        try:
-            self.auth_class = getattr(auth_module, "Authenticator")
-        except Exception, e:
+            auth_class = getattr(auth_module, "Authenticator")
+            return auth_class
+        except Exception as e:
             raise Exception("Authenticator class not found in %s" % auth_module)
 
+
     def init_sockets(self, sockets):
         ### All right, we're ready to accept customers:
         for socktype, sock in sockets:
+            log("init_sockets(%s) will add %s socket %s", sockets, socktype, sock)
             self.idle_add(self.add_listen_socket, socktype, sock)
 
     def init_when_ready(self, callbacks):
@@ -218,18 +260,18 @@ class ServerCore(object):
 
 
     def reaper_quit(self):
-        self.clean_quit(False)
+        self.clean_quit()
 
     def signal_quit(self, signum, frame):
         log.info("")
         log.info("got signal %s, exiting", SIGNAMES.get(signum, signum))
         signal.signal(signal.SIGINT, deadly_signal)
         signal.signal(signal.SIGTERM, deadly_signal)
-        self.idle_add(self.clean_quit, True)
+        self.idle_add(self.clean_quit)
         self.idle_add(sys.exit, 128+signum)
 
-    def clean_quit(self, from_signal=False, upgrading=False):
-        log("clean_quit(%s, %s)", from_signal, upgrading)
+    def clean_quit(self, upgrading=False):
+        log("clean_quit(%s)", upgrading)
         #ensure the reaper doesn't call us again:
         if self.child_reaper:
             def noop():
@@ -241,10 +283,9 @@ class ServerCore(object):
             log.debug("quit_timer()")
             self.quit(upgrading)
         #if from a signal, just force quit:
-        stop_worker(from_signal)
-        if not from_signal:
-            #not from signal: use force stop worker after delay
-            self.timeout_add(250, stop_worker, True)
+        stop_worker()
+        #not from signal: use force stop worker after delay
+        self.timeout_add(250, stop_worker, True)
         self.timeout_add(500, quit_timer)
         def force_quit(*args):
             log.debug("force_quit()")
@@ -281,7 +322,7 @@ class ServerCore(object):
             for x in self._when_ready:
                 try:
                     x()
-                except Exception, e:
+                except Exception as e:
                     log.error("error on %s: %s", x, e)
         self.idle_add(start_ready_callbacks)
         def print_ready():
@@ -295,10 +336,10 @@ class ServerCore(object):
         raise NotImplementedError()
 
     def cleanup(self, *args):
-        log("cleanup() stopping %s tcp proxy clients: %s", len(self._tcp_proxy_clients), self._tcp_proxy_clients)
+        netlog("cleanup() stopping %s tcp proxy clients: %s", len(self._tcp_proxy_clients), self._tcp_proxy_clients)
         for p in list(self._tcp_proxy_clients):
             p.quit()
-        log("cleanup will disconnect: %s", self._potential_protocols)
+        netlog("cleanup will disconnect: %s", self._potential_protocols)
         if self._upgrading:
             reason = SERVER_EXIT
         else:
@@ -311,10 +352,11 @@ class ServerCore(object):
         raise NotImplementedError()
 
     def _new_connection(self, listener, *args):
-        socktype = self.socket_types.get(listener, "")
+        socktype = self.socket_types.get(listener)
+        assert socktype, "cannot find socket type for %s" % listener
         sock, address = listener.accept()
         if len(self._potential_protocols)>=self._max_connections:
-            log.error("too many connections (%s), ignoring new one", len(self._potential_protocols))
+            netlog.error("too many connections (%s), ignoring new one", len(self._potential_protocols))
             sock.close()
             return  True
         try:
@@ -324,13 +366,22 @@ class ServerCore(object):
         sockname = sock.getsockname()
         target = peername or sockname
         sock.settimeout(self._socket_timeout)
-        log("new_connection(%s) sock=%s, sockname=%s, address=%s, peername=%s", args, sock, sockname, address, peername)
+        netlog("new_connection(%s) sock=%s, timeout=%s, sockname=%s, address=%s, peername=%s", args, sock, self._socket_timeout, sockname, address, peername)
         sc = SocketConnection(sock, sockname, address, target, socktype)
-        log.info("New connection received: %s", sc)
+        netlog("socket connection: %s", sc)
+        frominfo = ""
+        if peername:
+            frominfo = " from %s" % str(peername)
+        netlog.info("New %s connection received%s", socktype, frominfo)
         protocol = Protocol(self, sc, self.process_packet)
         self._potential_protocols.append(protocol)
         protocol.large_packets.append("info-response")
         protocol.authenticator = None
+        if socktype=="tcp":
+            protocol.auth_class = self.tcp_auth_class
+        else:
+            protocol.auth_class = self.auth_class
+        protocol.socket_type = socktype
         protocol.invalid_header = self.invalid_header
         protocol.receive_aliases.update(self._aliases)
         protocol.start()
@@ -338,22 +389,27 @@ class ServerCore(object):
         return True
 
     def invalid_header(self, proto, data):
-        log("invalid_header(%s, %s)", proto, repr_ellipsized(data))
-        if proto.input_packetcount==0 and self._tcp_proxy:
-            self.start_tcp_proxy(proto, data)
+        netlog("invalid_header(%s, %s bytes: '%s') input_packetcount=%s, tcp_proxy=%s", proto, len(data or ""), repr_ellipsized(data), proto.input_packetcount, self._tcp_proxy)
+        if proto.input_packetcount==0 and self._tcp_proxy and not proto._closed:
+            #start a new proxy in a thread
+            def run_proxy():
+                self.start_tcp_proxy(proto, data)
+            t = make_daemon_thread(run_proxy, "web-proxy-for-%s" % proto)
+            t.start()
             return
         err = "invalid packet format, not an xpra client?"
         proto.gibberish(err, data)
 
     def start_tcp_proxy(self, proto, data):
-        log("start_tcp_proxy(%s, %s)", proto, data[:10])
-        #any buffers read after we steal the connection will be placed in this temporary queue:
-        temp_read_buffer = Queue()
-        client_connection = proto.steal_connection(temp_read_buffer.put)
+        proxylog("start_tcp_proxy(%s, '%s')", proto, repr_ellipsized(data))
         try:
             self._potential_protocols.remove(proto)
         except:
             pass        #might already have been removed by now
+        proxylog("start_tcp_proxy: protocol state before stealing: %s", proto.get_info(alias_info=False))
+        #any buffers read after we steal the connection will be placed in this temporary queue:
+        temp_read_buffer = Queue()
+        client_connection = proto.steal_connection(temp_read_buffer.put)
         #connect to web server:
         sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
         sock.settimeout(10)
@@ -361,15 +417,15 @@ class ServerCore(object):
         try:
             web_server_connection = _socket_connect(sock, (host, int(port)), "web-proxy-for-%s" % proto, "tcp")
         except:
-            log.warn("failed to connect to proxy: %s:%s", host, port)
+            proxylog.warn("failed to connect to proxy: %s:%s", host, port)
             proto.gibberish("invalid packet header", data)
             return
-        log("proxy connected to tcp server at %s:%s : %s", host, port, web_server_connection)
+        proxylog("proxy connected to tcp server at %s:%s : %s", host, port, web_server_connection)
         sock.settimeout(self._socket_timeout)
 
         ioe = proto.wait_for_io_threads_exit(0.5+self._socket_timeout)
         if not ioe:
-            log.warn("proxy failed to stop all existing network threads!")
+            proxylog.warn("proxy failed to stop all existing network threads!")
             self.disconnect_protocol(proto, "internal threading error")
             return
         #now that we own it, we can start it again:
@@ -377,29 +433,28 @@ class ServerCore(object):
         #and we can use blocking sockets:
         self.set_socket_timeout(client_connection, None)
         sock.settimeout(None)
- 
-        log("pushing initial buffer to its new destination: %s", repr_ellipsized(data))
+
+        proxylog("pushing initial buffer to its new destination: %s", repr_ellipsized(data))
         web_server_connection.write(data)
         while not temp_read_buffer.empty():
             buf = temp_read_buffer.get()
             if buf:
-                log("pushing read buffer to its new destination: %s", repr_ellipsized(buf))
+                proxylog("pushing read buffer to its new destination: %s", repr_ellipsized(buf))
                 web_server_connection.write(buf)
-        p = XpraProxy(client_connection, web_server_connection)
+        p = XpraProxy(client_connection.target, client_connection, web_server_connection)
         self._tcp_proxy_clients.append(p)
-        def run_proxy():
-            p.run()
-            log("run_proxy() %s ended", p)
-            if p in self._tcp_proxy_clients:
-                self._tcp_proxy_clients.remove(p)
-        t = make_daemon_thread(run_proxy, "web-proxy-for-%s" % proto)
-        t.start()
-        log.info("client %s forwarded to proxy server %s:%s", client_connection, host, port)
+        proxylog.info("client connection from %s forwarded to proxy server on %s:%s", client_connection.target, host, port)
+        p.run()
+        proxylog("run_proxy() %s ended", p)
+        if p in self._tcp_proxy_clients:
+            self._tcp_proxy_clients.remove(p)
 
     def is_timedout(self, protocol):
         #subclasses may override this method (ServerBase does)
-        return not protocol._closed and protocol in self._potential_protocols and \
+        v = not protocol._closed and protocol in self._potential_protocols and \
             protocol not in self._tcp_proxy_clients
+        netlog("is_timedout(%s)=%s", protocol, v)
+        return v
 
     def verify_connection_accepted(self, protocol):
         if self.is_timedout(protocol):
@@ -407,42 +462,50 @@ class ServerCore(object):
             self.send_disconnect(protocol, LOGIN_TIMEOUT)
 
     def send_disconnect(self, proto, reason, *extra):
-        log("send_disconnect(%s, %s, %s)", proto, reason, extra)
+        netlog("send_disconnect(%s, %s, %s)", proto, reason, extra)
         if proto._closed:
             return
         proto.send_now(["disconnect", reason]+list(extra))
         self.timeout_add(1000, self.force_disconnect, proto)
 
     def force_disconnect(self, proto):
+        netlog("force_disconnect(%s)", proto)
         proto.close()
 
     def disconnect_client(self, protocol, reason, *extra):
+        netlog("disconnect_client(%s, %s, %s)", protocol, reason, extra)
         if protocol and not protocol._closed:
             self.disconnect_protocol(protocol, reason, *extra)
 
     def disconnect_protocol(self, protocol, reason, *extra):
+        netlog("disconnect_protocol(%s, %s, %s)", protocol, reason, extra)
         i = str(reason)
         if extra:
             i += " (%s)" % extra
-        log.info("Disconnecting client %s: %s", protocol, i)
+        try:
+            proto_info = " '%s'" % protocol._conn.get_info().get("endpoint")
+        except:
+            proto_info = " %s" % protocol
+        netlog.info("Disconnecting client%s: %s", proto_info, i)
         protocol.flush_then_close(["disconnect", reason]+list(extra))
 
 
     def _process_connection_lost(self, proto, packet):
+        netlog("process_connection_lost(%s, %s)", proto, packet)
         if proto in self._potential_protocols:
-            log.info("Connection lost")
+            netlog.info("Connection lost")
             self._potential_protocols.remove(proto)
 
     def _process_gibberish(self, proto, packet):
         (_, message, data) = packet
-        log("Received uninterpretable nonsense from %s: %s", proto, message)
-        log(" data: %s", repr_ellipsized(data))
+        netlog("Received uninterpretable nonsense from %s: %s", proto, message)
+        netlog(" data: %s", repr_ellipsized(data))
         self.disconnect_client(proto, message)
 
     def _process_invalid(self, protocol, packet):
         (_, message, data) = packet
-        log("Received invalid packet: %s", message)
-        log(" data: %s", repr_ellipsized(data))
+        netlog("Received invalid packet: %s", message)
+        netlog(" data: %s", repr_ellipsized(data))
         self.disconnect_client(protocol, message)
 
 
@@ -479,21 +542,16 @@ class ServerCore(object):
                 #call from UI thread:
                 self.idle_add(self.handle_command_request, proto, command_req)
                 return
-            #continue processing hello packet in UI thread:
-            self.idle_add(self.call_hello_oked, proto, packet, c, auth_caps)
-
-    def call_hello_oked(self, proto, packet, c, auth_caps):
-        try:
-            self.hello_oked(proto, packet, c, auth_caps)
-        except ClientException, e:
-            log.error("error setting up connection for %s: %s", proto, e)
-            self.disconnect_client(proto, SERVER_ERROR, str(e))
-        except Exception, e:
-            #log full stack trace at debug level,
-            #log exception as error
-            #but don't disclose internal details to the client
-            log.error("server error processing new connection from %s: %s", proto, e, exc_info=True)
-            self.disconnect_client(proto, SERVER_ERROR, "error accepting new connection")
+            #continue processing hello packet:
+            try:
+                self.hello_oked(proto, packet, c, auth_caps)
+            except ClientException as e:
+                log.error("error setting up connection for %s: %s", proto, e)
+                self.disconnect_client(proto, SERVER_ERROR, str(e))
+            except Exception as e:
+                #log exception but don't disclose internal details to the client
+                log.error("server error processing new connection from %s: %s", proto, e, exc_info=True)
+                self.disconnect_client(proto, SERVER_ERROR, "error accepting new connection")
 
     def set_socket_timeout(self, conn, timeout=None):
         #FIXME: this is ugly, but less intrusive than the alternative?
@@ -515,11 +573,11 @@ class ServerCore(object):
 
         #authenticator:
         username = c.strget("username")
-        if proto.authenticator is None and self.auth_class:
+        if proto.authenticator is None and proto.auth_class:
             try:
-                proto.authenticator = self.auth_class(username)
-            except Exception, e:
-                log.warn("error instantiating %s: %s", self.auth_class, e)
+                proto.authenticator = proto.auth_class(username)
+            except Exception as e:
+                log.warn("error instantiating %s: %s", proto.auth_class, e)
                 auth_failed("authentication failed")
                 return False
         self.digest_modes = c.get("digest", ("hmac", ))
@@ -547,21 +605,28 @@ class ServerCore(object):
             auth_caps = None
 
         #verify authentication if required:
-        if proto.authenticator and proto.authenticator.requires_challenge():
+        if (proto.authenticator and proto.authenticator.requires_challenge()) or c.get("challenge") is not None:
             challenge_response = c.strget("challenge_response")
             client_salt = c.strget("challenge_client_salt")
             log("processing authentication with %s, response=%s, client_salt=%s", proto.authenticator, challenge_response, binascii.hexlify(client_salt or ""))
             #send challenge if this is not a response:
             if not challenge_response:
-                challenge = proto.authenticator.get_challenge()
-                if challenge is None:
-                    auth_failed("invalid authentication state: unexpected challenge response")
-                    return False
-                salt, digest = challenge
-                log.info("Authentication required, %s sending challenge for '%s' using digest %s", proto.authenticator, username, digest)
-                if digest not in self.digest_modes:
-                    auth_failed("cannot proceed without %s digest support" % digest)
-                    return False
+                if proto.authenticator:
+                    challenge = proto.authenticator.get_challenge()
+                    if challenge is None:
+                        auth_failed("invalid authentication state: unexpected challenge response")
+                        return False
+                    salt, digest = challenge
+                    log.info("Authentication required, %s sending challenge for '%s' using digest %s", proto.authenticator, username, digest)
+                    if digest not in self.digest_modes:
+                        auth_failed("cannot proceed without %s digest support" % digest)
+                        return False
+                else:
+                    authlog.warn("Warning: client expects a challenge but this connection is unauthenticated")
+                    #fake challenge so the client will send the real hello:
+                    from xpra.os_util import get_hex_uuid
+                    salt = get_hex_uuid()+get_hex_uuid()
+                    digest = "hmac"
                 proto.send_now(("challenge", salt, auth_caps or "", digest))
                 return False
 
@@ -585,15 +650,20 @@ class ServerCore(object):
     def get_encryption_key(self, authenticator=None):
         #if we have a keyfile specified, use that:
         if self.encryption_keyfile:
-            log("trying to load encryption key from keyfile: %s", self.encryption_keyfile)
+            netlog("trying to load encryption key from keyfile: %s", self.encryption_keyfile)
             return self.filedata_nocrlf(self.encryption_keyfile)
+        env_key = os.environ.get('XPRA_ENCRYPTION_KEY')
+        if env_key:
+            return env_key
         v = None
         if authenticator:
-            log("trying to get encryption key from: %s", authenticator)
+            netlog("trying to get encryption key from: %s", authenticator)
             v = authenticator.get_password()
         if v is None and self.password_file:
-            log("trying to load encryption key from password file: %s", self.password_file)
+            netlog("trying to load encryption key from password file: %s", self.password_file)
             v = self.filedata_nocrlf(self.password_file)
+        if v is None and os.environ.get('XPRA_PASSWORD'):
+            v = os.environ.get('XPRA_PASSWORD')
         return v
 
     def hello_oked(self, proto, packet, c, auth_caps):
@@ -607,12 +677,12 @@ class ServerCore(object):
             command = args[0]
             error = 0
             if command not in self.control_commands:
-                commandlog.warn("invalid command: %s (must be one of: %s)", command, self.control_commands)
+                commandlog.warn("invalid command: '%s' (must be one of: %s)", command, ", ".join(self.control_commands))
                 error = 6
                 response = "invalid command"
             else:
                 error, response = self.do_handle_command_request(command, args[1:])
-        except Exception, e:
+        except Exception as e:
             commandlog.error("error processing command %s", command, exc_info=True)
             error = 127
             response = "error processing command: %s" % e
@@ -634,6 +704,7 @@ class ServerCore(object):
 
     def accept_client(self, proto, c):
         #max packet size from client (the biggest we can get are clipboard packets)
+        netlog("accept_client(%s, %s)", proto, c)
         proto.max_packet_size = 1024*1024  #1MB
         proto.send_aliases = c.dictget("aliases")
         if proto in self._potential_protocols:
@@ -679,7 +750,7 @@ class ServerCore(object):
             try:
                 info = self.get_info(proto, *args)
                 ui_info.update(info)
-            except Exception, e:
+            except Exception as e:
                 log.error("error during info collection: %s", e, exc_info=True)
             callback(proto, ui_info)
         thread.start_new_thread(in_thread, ())
@@ -688,21 +759,33 @@ class ServerCore(object):
         #this function is for info which MUST be collected from the UI thread
         return {}
 
+    def get_thread_info(self, proto):
+        return get_thread_info(proto)
+
     def get_info(self, proto, *args):
         #this function is for non UI thread info
         info = {}
         def up(prefix, d):
             updict(info, prefix, d)
+        filtered_env = os.environ.copy()
+        if filtered_env.get('XPRA_PASSWORD'):
+            filtered_env['XPRA_PASSWORD'] = "*****"
+        if filtered_env.get('XPRA_ENCRYPTION_KEY'):
+            filtered_env['XPRA_ENCRYPTION_KEY'] = "*****"
 
         up("network",   get_network_caps())
         up("server",    get_server_info())
-        up("threads",   get_thread_info(proto))
-        up("env",       os.environ)
+        up("threads",   self.get_thread_info(proto))
+        up("env",       filtered_env)
         up("server", {
                 "mode"              : self.get_server_mode(),
                 "type"              : "Python",
                 "start_time"        : int(self.start_time),
                 "authenticator"     : str((self.auth_class or str)("")),
+                "argv"              : sys.argv,
+                "path"              : sys.path,
+                "exec_prefix"       : sys.exec_prefix,
+                "executable"        : sys.executable,
                 })
         if self.session_name:
             info["session.name"] = self.session_name
@@ -717,12 +800,12 @@ class ServerCore(object):
             assert isinstance(packet_type, types.StringTypes), "packet_type %s is not a string: %s..." % (type(packet_type), str(packet_type)[:100])
             handler = self._default_packet_handlers.get(packet_type)
             if handler:
-                log("process packet %s", packet_type)
+                netlog("process packet %s", packet_type)
                 handler(proto, packet)
                 return
-            log.error("unknown or invalid packet type: %s from %s", packet_type, proto)
+            netlog.error("unknown or invalid packet type: %s from %s", packet_type, proto)
             proto.close()
         except KeyboardInterrupt:
             raise
         except:
-            log.error("Unhandled error while processing a '%s' packet from peer using %s", packet_type, handler, exc_info=True)
+            netlog.error("Unhandled error while processing a '%s' packet from peer using %s", packet_type, handler, exc_info=True)
