@@ -4,6 +4,7 @@
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
+import sys
 import socket
 
 from xpra.log import Logger
@@ -11,7 +12,7 @@ log = Logger("shadow")
 
 from xpra.net.compression import Compressed
 from xpra.server.batch_config import DamageBatchConfig
-from xpra.util import AdHocStruct, prettify_plug_name, DONE
+from xpra.util import prettify_plug_name, DONE
 
 DEFAULT_DELAY = 50              #50ms refresh
 
@@ -39,6 +40,9 @@ class RootWindowModel(object):
     def is_shadow(self):
         return True
 
+    def get_default_window_icon(self):
+        return None
+
     def acknowledge_changes(self):
         pass
 
@@ -46,7 +50,20 @@ class RootWindowModel(object):
         raise NotImplementedError()
 
     def get_property_names(self):
-        return ("title", "client-machine", "window-type", "size-hints")
+        return ("title", "class-instance", "client-machine", "window-type", "size-hints", "icon", "shadow")
+
+    def get_dynamic_property_names(self):
+        #nothing changes
+        return ()
+
+    def get_generic_os_name(self):
+        for k,v in {"linux"     : "linux",
+                    "darwin"    : "osx",
+                    "win"       : "win32",
+                    "freebsd"   : "freebsd"}.items():
+            if sys.platform.startswith(k):
+                return v
+        return sys.platform
 
     def get_property(self, prop):
         if prop=="title":
@@ -57,22 +74,50 @@ class RootWindowModel(object):
             return ["NORMAL"]
         elif prop=="fullscreen":
             return False
+        elif prop=="shadow":
+            return True
         elif prop=="scaling":
             return None
         elif prop=="opacity":
             return None
         elif prop=="size-hints":
             size = self.window.get_size()
-            return RootSizeHints(size)
+            return {"maximum-size"  : size,
+                    "minimum-size"  : size,
+                    "base-size" : size}
+        elif prop=="class-instance":
+            osn = self.get_generic_os_name()
+            return ("xpra-%s" % osn, "Xpra-%s" % osn.upper())
+        elif prop=="icon":
+            #convert it to a cairo surface..
+            #because that's what the property is expected to be
+            try:
+                import gtk.gdk
+                from xpra.platform.paths import get_icon
+                icon_name = self.get_generic_os_name()+".png"
+                icon = get_icon(icon_name)
+                log("icon(%s)=%s", icon_name, icon)
+                if not icon:
+                    return None
+                import cairo
+                surf = cairo.ImageSurface(cairo.FORMAT_ARGB32, icon.get_width(), icon.get_height())
+                gc = gtk.gdk.CairoContext(cairo.Context(surf))
+                gc.set_source_pixbuf(icon, 0, 0)
+                gc.paint()
+                log("icon=%s", surf)
+                return surf
+            except:
+                log("failed to return window icon")
+                return None
         else:
             raise Exception("invalid property: %s" % prop)
         return None
 
     def connect(self, *args):
-        log("ignoring signal connect request: %s", args)
+        log.warn("ignoring signal connect request: %s", args)
 
     def disconnect(self, *args):
-        log("ignoring signal disconnect request: %s", args)
+        log.warn("ignoring signal disconnect request: %s", args)
 
     def get_dimensions(self):
         return self.window.get_size()
@@ -80,18 +125,6 @@ class RootWindowModel(object):
     def get_position(self):
         return 0, 0
 
-class RootSizeHints(AdHocStruct):
-
-    def __init__(self, size):
-        self.max_size = size
-        self.min_size = size
-        self.base_size = size
-        for x in ("resize_inc", "min_aspect", "max_aspect", "min_aspect_ratio", "max_aspect_ratio"):
-            setattr(self, x, None)
-
-    def to_dict(self):
-        return self.__dict__
-
 
 class ShadowServerBase(object):
 
@@ -102,7 +135,6 @@ class ShadowServerBase(object):
         self.sharing = False
         DamageBatchConfig.ALWAYS = True             #always batch
         DamageBatchConfig.MIN_DELAY = 50            #never lower than 50ms
-        DamageBatchConfig.RECALCULATE_DELAY = 0.1   #re-compute delay 10 times per second at most
 
     def get_server_mode(self):
         return "shadow"
@@ -117,9 +149,6 @@ class ShadowServerBase(object):
     def watch_keymap_changes(self):
         pass
 
-    def calculate_workarea(self, w, h):
-        pass
-
     def start_refresh(self, delay=DEFAULT_DELAY):
         self.timeout_add(delay, self.refresh)
 
@@ -148,14 +177,24 @@ class ShadowServerBase(object):
                 log.warn("This client is running within the Xpra server %s", server_uuid)
         return True
 
+    def do_parse_screen_info(self, ss):
+        try:
+            log.info("client root window size is %sx%s", *ss.desktop_size)
+        except:
+            log.info("unknown client desktop size")
+        return self.get_root_window_size()
+
+    def _process_desktop_size(self, proto, packet):
+        #just record the screen size info in the source
+        ss = self._server_sources.get(proto)
+        if ss and len(packet)>=4:
+            ss.set_screen_sizes(packet[3])
+
+
     def set_keyboard_repeat(self, key_repeat):
         """ don't override the existing desktop """
         pass
 
-    def set_best_screen_size(self):
-        """ we don't change resolutions when shadowing """
-        return self.root_window_model.get_dimensions()
-
     def set_keymap(self, server_source, force=False):
         log.info("shadow server: setting default keymap translation")
         self.keyboard_config = server_source.set_default_keymap()
@@ -172,7 +211,7 @@ class ShadowServerBase(object):
         log("send_windows_and_cursors(%s, %s) will send: %s", ss, sharing, self._id_to_window)
         for wid in sorted(self._id_to_window.keys()):
             window = self._id_to_window[wid]
-            assert window == self.root_window_model
+            assert window == self.root_window_model, "expected window to be %s, but got %s" % (self.root_window_model, window)
             w, h = self.root.get_size()
             ss.new_window("new-window", wid, window, 0, 0, w, h, self.client_properties.get(ss.uuid))
 
@@ -216,7 +255,6 @@ class ShadowServerBase(object):
         if len(packet)>=7:
             self._set_client_properties(proto, wid, self.root_window_model, packet[6])
 
-
     def _process_close_window(self, proto, packet):
         wid = packet[1]
         self._process_window_common(wid)
