@@ -46,6 +46,15 @@ class VideoSubregion(object):
     def cleanup(self):
         self.reset()
 
+
+    def __repr__(self):
+        return "VideoSubregion(%s)" % self.get_info()
+
+
+    def set_auto_refresh_delay(self, d):
+        refreshlog("subregion auto-refresh delay: %s", d)
+        self.auto_refresh_delay = d
+
     def cancel_refresh_timer(self):
         rt = self.refresh_timer
         refreshlog("cancel_refresh_timer() timer=%s", rt)
@@ -69,10 +78,8 @@ class VideoSubregion(object):
                 "non_max_wait" :  self.non_max_wait}
         rr = list(self.refresh_regions)
         if rr:
-            i = 0
-            for r in rr:
+            for i, r in enumerate(rr):
                 info["refresh_region[%s]" % i] = (r.x, r.y, r.width, r.height)
-                i += 1
         return info
 
 
@@ -129,7 +136,9 @@ class VideoSubregion(object):
         self.set_at = 0
         self.counter = 0
 
-    def identify_video_subregion(self, ww, wh, damage_events_count, last_damage_events):
+    def identify_video_subregion(self, ww, wh, damage_events_count, last_damage_events, starting_at=0):
+        sslog("%s.identify_video_subregion(..)", self)
+        sslog("identify_video_subregion(%s, %s, %s, %s)", ww, wh, damage_events_count, last_damage_events)
 
         def setnewregion(rect, msg="", *args):
             if rect.x<=0 and rect.y<=0 and rect.width>=ww and rect.height>=wh:
@@ -163,7 +172,7 @@ class VideoSubregion(object):
             #how many damage events occurred since we chose this region:
             event_count = max(0, damage_events_count - self.set_at)
             #make the timeout longer when the region has worked longer:
-            slow_region_timeout = 10 + math.log(2+event_count, 1.5)
+            slow_region_timeout = 2 + math.log(2+event_count, 1.5)
             if rect and elapsed>=slow_region_timeout:
                 update_markers()
                 return self.novideoregion("too much time has passed (%is for %s %s events)", elapsed, event_types, event_count)
@@ -176,7 +185,7 @@ class VideoSubregion(object):
             return
 
         #create a list (copy) to work on:
-        lde = list(last_damage_events)
+        lde = [x for x in list(last_damage_events) if x[0]>=starting_at]
         dc = len(lde)
         if dc<=MIN_EVENTS:
             return self.novideoregion("not enough damage events yet (%s)", dc)
@@ -193,16 +202,16 @@ class VideoSubregion(object):
             if h>=MIN_H:
                 hc.setdefault(h, dict()).setdefault(y, set()).add(r)
 
-        def score_region(info, region):
+        def score_region(info, region, ignore_size=0):
             #check if the region given is a good candidate, and if so we use it
             #clamp it:
-            region.width = min(ww, region.width)
-            region.height = min(wh, region.height)
-            if region.width<MIN_W or region.height<MIN_H:
+            width = min(ww, region.width)
+            height = min(wh, region.height)
+            if width<MIN_W or height<MIN_H:
                 #too small, ignore it:
                 return 0
             #and make sure this does not end up much bigger than needed:
-            insize = region.width*region.height
+            insize = width*height
             if ww*wh<=insize:
                 return 0
             #count how many pixels are in or out if this region
@@ -213,6 +222,9 @@ class VideoSubregion(object):
                     incount += inregion.width*inregion.height*int(count)
                 outregions = r.substract_rect(region)
                 for x in outregions:
+                    if ignore_size>0 and x.width*x.height<ignore_size:
+                        #skip small region outside rectangle
+                        continue
                     outcount += x.width*x.height*int(count)
             total = incount+outcount
             assert total>0
@@ -226,7 +238,8 @@ class VideoSubregion(object):
             # we would otherwise end up excluding the ones on the edge
             # if they ever happen to have a slightly lower hit count)
             score = inpct * ww*wh*2 / (ww*wh + insize)
-            sslog("testing %12s video region %34s: %3i%% in, %3i%% out, score=%2i", info, region, inpct, outpct, score)
+            sslog("testing %12s video region %34s: %3i%% in, %3i%% out, %3i%% of window, score=%2i",
+                  info, region, inpct, outpct, 100*width*height/ww/wh, score)
             return score
 
         update_markers()
@@ -239,6 +252,8 @@ class VideoSubregion(object):
                 sslog("keeping existing video region %s with score %s", rect, cur_score)
                 return
 
+        scores = {None : 0}
+
         #split the regions we really care about (enough pixels, big enough):
         damage_count = {}
         min_count = max(2, len(lde)/40)
@@ -264,10 +279,11 @@ class VideoSubregion(object):
                         return self.novideoregion("most damaged region is (almost?) the whole window: %s", r)
                     setnewregion(r, "%s%% of large damage requests, score=%s", most_pct, score)
                     return
+                elif score>=100:
+                    scores[r] = score
 
         #try harder: try combining regions with the same width or height:
         #(some video players update the video region in bands)
-        scores = {None : 0}
         for w, d in wc.items():
             for x,regions in d.items():
                 if len(regions)>=2:
@@ -277,7 +293,7 @@ class VideoSubregion(object):
                     sslog("vertical regions of width %i at %i with at least %i hits: %s", w, x, min_count, keep)
                     if keep:
                         merged = merge_all(keep)
-                        scores[merged] = score_region("vertical", merged)
+                        scores[merged] = score_region("vertical", merged, 48*48)
         for h, d in hc.items():
             for y,regions in d.items():
                 if len(regions)>=2:
@@ -287,7 +303,7 @@ class VideoSubregion(object):
                     sslog("horizontal regions of height %i at %i with at least %i hits: %s", h, y, min_count, keep)
                     if keep:
                         merged = merge_all(keep)
-                        scores[merged] = score_region("horizontal", merged)
+                        scores[merged] = score_region("horizontal", merged, 48*48)
 
         sslog("merged regions scores: %s", scores)
         highscore = max(scores.values())
@@ -301,7 +317,7 @@ class VideoSubregion(object):
             sslog("keeping existing video region %s with score %s", rect, cur_score)
             return
 
-        if highscore>=105:
+        if highscore>=100:
             region = [r for r,s in scores.items() if s==highscore][0]
             return setnewregion(region, "high score: %s", highscore)
 
@@ -313,6 +329,6 @@ class VideoSubregion(object):
             merged = merge_all(damage_count.keys())
             score = score_region("merged", merged)
             if score>=110:
-                return setnewregion(merged, "merged all regions, score=%s", score)
+                return setnewregion(merged, "merged all regions, score=%s", score, 48*48)
 
         self.novideoregion("failed to identify a video region")
