@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # This file is part of Xpra.
-# Copyright (C) 2010-2014 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2010-2015 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -10,10 +10,57 @@ import os
 from xpra.log import Logger
 log = Logger("sound")
 
-SOUND_TEST_MODE = os.environ.get("XPRA_SOUND_TEST", "0")!="0"
+GST_QUEUE_NO_LEAK             = 0
+GST_QUEUE_LEAK_UPSTREAM       = 1
+GST_QUEUE_LEAK_DOWNSTREAM     = 2
+GST_QUEUE_LEAK_DEFAULT = GST_QUEUE_LEAK_DOWNSTREAM
+MS_TO_NS = 1000000
+
+QUEUE_LEAK = int(os.environ.get("XPRA_SOUND_QUEUE_LEAK", GST_QUEUE_LEAK_DEFAULT))
+if QUEUE_LEAK not in (GST_QUEUE_NO_LEAK, GST_QUEUE_LEAK_UPSTREAM, GST_QUEUE_LEAK_DOWNSTREAM):
+    log.error("invalid leak option %s", QUEUE_LEAK)
+    QUEUE_LEAK = GST_QUEUE_LEAK_DEFAULT
+
+def get_queue_time(default_value=450, prefix=""):
+    queue_time = int(os.environ.get("XPRA_SOUND_QUEUE_%sTIME" % prefix, default_value))*MS_TO_NS
+    queue_time = max(0, queue_time)
+    return queue_time
+
+
 ALLOW_SOUND_LOOP = os.environ.get("XPRA_ALLOW_SOUND_LOOP", "0")=="1"
 
 
+NAME_TO_SRC_PLUGIN = {
+    "auto"          : "autoaudiosrc",
+    "alsa"          : "alsasrc",
+    "oss"           : "osssrc",
+    "oss4"          : "oss4src",
+    "jack"          : "jackaudiosrc",
+    "osx"           : "osxaudiosrc",
+    "test"          : "audiotestsrc",
+    "pulse"         : "pulsesrc",
+    "direct"        : "directsoundsrc",
+    }
+SRC_TO_NAME_PLUGIN = {}
+for k,v in NAME_TO_SRC_PLUGIN.items():
+    SRC_TO_NAME_PLUGIN[v] = k
+PLUGIN_TO_DESCRIPTION = {
+    "pulsesrc"      : "Pulseaudio",
+    "jacksrc"       : "JACK Audio Connection Kit",
+    }
+NAME_TO_INFO_PLUGIN = {
+    "auto"          : "Wrapper audio source for automatically detected audio source",
+    "alsa"          : "Read from a sound card via ALSA",
+    "oss"           : "Capture from a sound card via OSS",
+    "oss4"          : "Capture from a sound card via OSS version 4",
+    "jack"          : "Captures audio from a JACK server",
+    "osx"           : "Input from a sound card in OS X",
+    "test"          : "Creates audio test signals of given frequency and volume",
+    "pulse"         : "Captures audio from a PulseAudio server",
+    "direct"        : "directsoundsrc",
+    }
+
+
 VORBIS = "vorbis"
 AAC = "aac"
 FLAC = "flac"
@@ -26,7 +73,6 @@ WAVPACK = "wavpack"
 #format: encoder, container-formatter, decoder, container-parser
 #we keep multiple options here for the same encoding
 #and will populate the ones that are actually available into the "CODECS" dict
-MS_TO_NS = 1000000
 OGG_DELAY = 20*MS_TO_NS
 C_FORMAT = "oggmux"
 C_PARSER = "oggdemux"
@@ -45,7 +91,7 @@ CODEC_OPTIONS = [
 CODECS = {}
 
 CODEC_ORDER = [MP3, WAVPACK, WAV, FLAC, SPEEX]
-
+#CODEC_ORDER = [MP3, FLAC, SPEEX]
 
 #we may want to review this if/when we implement UDP transport:
 GDPPAY_CRC = False
@@ -58,6 +104,7 @@ MUXER_DEFAULT_OPTIONS = {
                                },
            }
 
+
 #code to temporarily redirect stderr and restore it afterwards, adapted from:
 #http://stackoverflow.com/questions/5081657/how-do-i-prevent-a-c-shared-library-to-print-on-stdout-in-python
 #so we can get rid of the stupid gst warning below:
@@ -169,6 +216,12 @@ except:
     log("failed to import GStreamer", exc_info=True)
 
 
+def normv(v):
+    if v==2**64-1:
+        return -1
+    return v
+
+
 def get_all_plugin_names():
     global all_plugin_names, has_gst
     if len(all_plugin_names)==0 and has_gst:
@@ -217,8 +270,7 @@ def get_sound_codecs(is_speaker, is_server):
             return can_encode()
         else:
             return can_decode()
-    except:
-        e = sys.exc_info()[1]
+    except Exception as e:
         log.warn("failed to get list of codecs: %s" % e)
         return []
 
@@ -298,77 +350,155 @@ def plugin_str(plugin, options):
     return s
 
 
+def get_source_plugins():
+    sources = []
+    from xpra.sound.pulseaudio_util import has_pa
+    #we have to put pulsesrc first if pulseaudio is installed
+    #because using autoaudiosource does not work properly for us:
+    #it may still choose pulse, but without choosing the right device.
+    if has_pa():
+        sources.append("pulsesrc")
+    sources.append("autoaudiosrc")
+    if sys.platform.startswith("darwin"):
+        sources.append("osxaudiosrc")
+    elif sys.platform.startswith("win"):
+        sources.append("directsoundsrc")
+    if os.name=="posix":
+        sources += ["alsasrc", "jackaudiosrc",
+                    "osssrc", "oss4src",
+                    "osxaudiosrc", "jackaudiosrc"]
+    sources.append("audiotestsrc")
+    return sources
+
+def get_available_source_plugins():
+    return [x for x in get_source_plugins() if has_plugins(x)]
+
+def get_test_defaults(remote):
+    return  {"wave" : 2, "freq" : 110, "volume" : 0.4}
+
 WARNED_MULTIPLE_DEVICES = False
-def start_sending_sound(codec, volume, remote_decoders, local_decoders, remote_pulseaudio_server, remote_pulseaudio_id):
-    assert has_gst
-    try:
-        matching_codecs = [x for x in remote_decoders if x in local_decoders]
-        ordered_codecs = [x for x in CODEC_ORDER if x in matching_codecs]
-        if len(ordered_codecs)==0:
-            log.error("no matching codecs between remote (%s) and local (%s) - sound disabled", remote_decoders, local_decoders)
-            return    None
-        if codec is not None and codec not in matching_codecs:
-            log.warn("invalid codec specified: %s", codec)
-            codec = None
-        if codec is None:
-            codec = ordered_codecs[0]
-        log("using sound codec %s", codec)
-        from xpra.sound.src import SoundSource
-        if SOUND_TEST_MODE:
-            sound_source = SoundSource("audiotestsrc", {"wave":2, "freq":110, "volume":0.4}, codec, volume, {})
-            log.info("using test sound source")
-        else:
-            from xpra.sound.pulseaudio_util import has_pa, get_pa_device_options, get_default_sink
-            from xpra.sound.pulseaudio_util import get_pulse_server, get_pulse_id, set_source_mute
-            if not has_pa():
-                log.error("pulseaudio not supported - sound disabled")
-                return    None
-            pa_server = get_pulse_server()
-            log("start sound, remote pulseaudio server=%s, local pulseaudio server=%s", remote_pulseaudio_server, pa_server)
-            #only worth comparing if we have a real server string
-            #one that starts with {UUID}unix:/..
-            if pa_server and pa_server.startswith("{") and \
-                remote_pulseaudio_server and remote_pulseaudio_server==pa_server:
-                log.error("identical pulseaudio server, refusing to create a sound loop - sound disabled")
-                return    None
-            pa_id = get_pulse_id()
-            log("start sound, client id=%s, server id=%s", remote_pulseaudio_id, pa_id)
-            if remote_pulseaudio_id and remote_pulseaudio_id==pa_id:
-                log.error("identical pulseaudio ID, refusing to create a sound loop - sound disabled")
-                return    None
-            monitor_devices = get_pa_device_options(True, False)
-            log("found pulseaudio monitor devices: %s", monitor_devices)
-            if len(monitor_devices)==0:
-                log.error("could not detect any pulseaudio monitor devices - sound forwarding is disabled")
-                return    None
-            #default to first one:
-            monitor_device, monitor_device_name = monitor_devices.items()[0]
-            if len(monitor_devices)>1:
-                default_sink = get_default_sink()
-                default_monitor = default_sink+".monitor"
-                global WARNED_MULTIPLE_DEVICES
-                if not WARNED_MULTIPLE_DEVICES:
-                    WARNED_MULTIPLE_DEVICES = True
-                    log.warn("found more than one audio monitor device:")
-                    for k,v in monitor_devices.items():
-                        log.warn(" * %s (\"%s\")", v, k)
-                if default_monitor in monitor_devices:
-                    monitor_device = default_monitor
-                    monitor_device_name = monitor_devices.get(default_monitor)
-                    if not WARNED_MULTIPLE_DEVICES:
-                        log.warn("using monitor of default sink: %s", monitor_device_name)
-                else:
-                    if not WARNED_MULTIPLE_DEVICES:
-                        log.warn("using the first device")
-            #make sure it is not muted:
-            set_source_mute(monitor_device, mute=False)
-            sound_source = SoundSource("pulsesrc", {"device" : monitor_device}, codec, volume, {})
-            log.info("starting sound capture using pulseaudio device: %s", monitor_device_name)
-        return sound_source
-    except:
-        e = sys.exc_info()[1]
-        log.error("error setting up sound: %s", e, exc_info=True)
+def get_pulse_defaults(remote):
+    """
+        choose the device to use
+    """
+    from xpra.sound.pulseaudio_util import has_pa, get_pa_device_options, get_default_sink
+    from xpra.sound.pulseaudio_util import get_pulse_server, get_pulse_id, set_source_mute
+    if not has_pa():
+        log.warn("pulseaudio is not available!")
         return    None
+    pa_server = get_pulse_server()
+    log("start sound, remote pulseaudio server=%s, local pulseaudio server=%s", remote.pulseaudio_server, pa_server)
+    #only worth comparing if we have a real server string
+    #one that starts with {UUID}unix:/..
+    if pa_server and pa_server.startswith("{") and \
+        remote.pulseaudio_server and remote.pulseaudio_server==pa_server:
+        log.error("identical Pulseaudio server, refusing to create a sound loop - sound disabled")
+        return    None
+    pa_id = get_pulse_id()
+    log("start sound, client id=%s, server id=%s", remote.pulseaudio_id, pa_id)
+    if remote.pulseaudio_id and remote.pulseaudio_id==pa_id:
+        log.error("identical Pulseaudio ID, refusing to create a sound loop - sound disabled")
+        return    None
+    monitor_devices = get_pa_device_options(True, False)
+    log("found pulseaudio monitor devices: %s", monitor_devices)
+    if len(monitor_devices)==0:
+        log.error("could not detect any Pulseaudio monitor devices - sound forwarding is disabled")
+        return    None
+    #default to first one:
+    monitor_device, monitor_device_name = monitor_devices.items()[0]
+    if len(monitor_devices)>1:
+        default_sink = get_default_sink()
+        default_monitor = default_sink+".monitor"
+        global WARNED_MULTIPLE_DEVICES
+        if not WARNED_MULTIPLE_DEVICES:
+            WARNED_MULTIPLE_DEVICES = True
+            log.warn("found more than one audio monitor device:")
+            for k,v in monitor_devices.items():
+                log.warn(" * %s (\"%s\")", v, k)
+        if default_monitor in monitor_devices:
+            monitor_device = default_monitor
+            monitor_device_name = monitor_devices.get(default_monitor)
+            if not WARNED_MULTIPLE_DEVICES:
+                log.warn("using monitor of default sink: %s", monitor_device_name)
+        else:
+            if not WARNED_MULTIPLE_DEVICES:
+                log.warn("using the first device")
+    log.info("using Pulseaudio device '%s'", monitor_device_name)
+    #make sure it is not muted:
+    set_source_mute(monitor_device, mute=False)
+    return {"device" : monitor_device}
+
+#a list of functions to call to get the plugin options
+#at runtime (so we can perform runtime checks on remote data,
+# to avoid sound loops for example)
+DEFAULT_SRC_PLUGIN_OPTIONS = {
+    "test"                  : get_test_defaults,
+    "pulse"                 : get_pulse_defaults,
+    }
+
+
+def parse_element_options(options_str):
+    #parse the options string and add the pairs:
+    options = {}
+    for s in options_str.split(","):
+        if not s:
+            continue
+        try:
+            k,v = s.split("=", 1)
+            options[k] = v
+        except Exception as e:
+            log.warn("failed to parse plugin option '%s': %s", s, e)
+    return options
+
+def get_sound_source_options(plugin, options_str, remote):
+    """
+        Given a plugin (short name), options string and remote info,
+        return the options for the plugin given,
+        using the dynamic defaults (which may use remote info)
+        and applying the options string on top.
+    """
+    #ie: get_sound_source_options("audiotestsrc", "wave=4,freq=220", {remote_pulseaudio_server=XYZ}):
+    #use the defaults as starting point:
+    defaults_fn = DEFAULT_SRC_PLUGIN_OPTIONS.get(plugin)
+    if defaults_fn:
+        options = defaults_fn(remote)
+        if options is None:
+            #means failure
+            return None
+    else:
+        options = {}
+    options.update(parse_element_options(options_str))
+    return options
+
+def parse_sound_source(sound_source_plugin, remote):
+    #format: PLUGINNAME:options
+    #ie: test:wave=2,freq=110,volume=0.4
+    #ie: pulse:device=device.alsa_input.pci-0000_00_14.2.analog-stereo
+    plugin = sound_source_plugin.split(":")[0]
+    options_str = (sound_source_plugin+":").split(":",1)[1]
+    simple_str = (plugin).lower().strip()
+    if not simple_str:
+        #choose the first one from
+        options = get_available_source_plugins()
+        if not options:
+            log.error("no source plugins available")
+            return None
+        log("parse_sound_source: no plugin specified, using default: %s", options[0])
+        simple_str = options[0]
+    for s in ("src", "sound", "audio"):
+        if simple_str.endswith(s):
+            simple_str = simple_str[:-len(s)]
+    gst_sound_source_plugin = NAME_TO_SRC_PLUGIN.get(simple_str)
+    if not gst_sound_source_plugin:
+        log.error("unknown source plugin: '%s' / '%s'", simple_str, sound_source_plugin)
+        return  None, {}
+    log("parse_sound_source(%s, %s) plugin=%s", sound_source_plugin, remote, gst_sound_source_plugin)
+    options = get_sound_source_options(simple_str, options_str, remote)
+    log("get_sound_source_options%s=%s", (simple_str, options_str, remote), options)
+    if options is None:
+        #means error
+        return None, {}
+    return gst_sound_source_plugin, options
 
 
 def get_info(receive=True, send=True, receive_codecs=[], send_codecs=[]):
@@ -397,8 +527,8 @@ def main():
         print("")
         encs = [x for x in CODEC_ORDER if has_encoder(x)]
         decs = [x for x in CODEC_ORDER if has_decoder(x)]
-        print("encoders supported: %s" % str(encs))
-        print("decoders supported: %s" % str(decs))
+        print("encoders supported: %s" % ", ".join(encs))
+        print("decoders supported: %s" % ", ".join(decs))
     finally:
         clean()
 
