@@ -21,7 +21,7 @@ def which(name):
         returncode, out, _ = safe_exec(cmd, log_errors=False)
         if returncode!=0 or not out:
             return ""
-        c = out.replace("\n", "").replace("\r", "")
+        c = out.decode("utf8").replace("\n", "").replace("\r", "")
         if os.path.exists(c) and os.path.isfile(c):
             if os.name=="posix" and not os.access(c, os.X_OK):
                 #odd, it's there but we can't run it!?
@@ -58,7 +58,7 @@ def pactl_output(log_errors=True, *pactl_args):
         code, out, _ = safe_exec(cmd, log_errors=log_errors, **kwargs)
         log("pactl_output%s returned %s", pactl_args, code)
         return  code, out
-    except Exception, e:
+    except Exception as e:
         if log_errors:
             log.error("failed to execute %s: %s", cmd, e)
         else:
@@ -66,13 +66,13 @@ def pactl_output(log_errors=True, *pactl_args):
         return  -1, None
 
 def get_x11_property(atom_name):
-    if sys.platform.startswith("win"):
+    if sys.platform.startswith("darwin") or sys.platform.startswith("win"):
         return ""
     try:
         from gtk import gdk
         root = gdk.get_default_root_window()
-        pulse_server_atom = gdk.atom_intern(atom_name)
-        p = root.property_get(pulse_server_atom)
+        atom = gdk.atom_intern(atom_name)
+        p = root.property_get(atom)
         if p is None:
             return ""
         v = p[2]
@@ -82,8 +82,6 @@ def get_x11_property(atom_name):
         return ""
 
 def has_pa_x11_property():
-    if os.name!="posix":
-        return False
     #try root window property (faster)
     ps = get_x11_property("PULSE_SERVER")
     return len(ps)>0
