@@ -11,6 +11,8 @@ from xpra.log import Logger
 log = Logger("x11", "focus")
 
 CurrentTime = constants["CurrentTime"]
+SubstructureNotifyMask = constants["SubstructureNotifyMask"]
+SubstructureRedirectMask = constants["SubstructureRedirectMask"]
 
 
 def send_wm_take_focus(target, timestamp):
@@ -21,11 +23,17 @@ def send_wm_take_focus(target, timestamp):
         raise OverflowError("invalid time: %#x" % timestamp)
     X11Window.sendClientMessage(target.xid, target.xid, False, 0,
                       "WM_PROTOCOLS",
-                      "WM_TAKE_FOCUS", timestamp, 0, 0, 0)
+                      "WM_TAKE_FOCUS", timestamp)
 
 def send_wm_delete_window(target):
     log("sending WM_DELETE_WINDOW to %#x", target.xid)
     X11Window.sendClientMessage(target.xid, target.xid, False, 0,
                       "WM_PROTOCOLS",
                       "WM_DELETE_WINDOW",
-                      CurrentTime, 0, 0, 0)
+                      CurrentTime)
+
+def send_wm_workspace(root, win, workspace=0):
+    event_mask = SubstructureNotifyMask | SubstructureRedirectMask
+    X11Window.sendClientMessage(root.xid, win.xid, False, event_mask,
+                      "_NET_WM_DESKTOP",
+                      workspace, CurrentTime)
