@@ -4,19 +4,11 @@
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
-import os
 import gtk
 import gobject
+import os
 
-# Maintain compatibility with old versions of Python, while avoiding a
-# deprecation warning on new versions:
-import sys
-if sys.version_info < (2, 6):
-    from sets import ImmutableSet
-else:
-    ImmutableSet = frozenset
-
-from xpra.gtk_common.error import trap
+from xpra.gtk_common.error import xsync, xswallow
 import xpra.x11.gtk_x11.selection
 from xpra.x11.gtk_x11.world_window import WorldWindow
 from xpra.x11.gtk_x11.prop import prop_set, prop_get
@@ -35,51 +27,19 @@ X11Keyboard = X11KeyboardBindings()
 from xpra.log import Logger
 log = Logger("x11", "window")
 focuslog = Logger("x11", "window", "focus")
+screenlog = Logger("x11", "window", "screen")
 
 
-XPRA_NET_WM_NAME = os.environ.get("XPRA_NET_WM_NAME", "Xpra")
-
 NotifyPointerRoot   = constants["NotifyPointerRoot"]
 NotifyDetailNone    = constants["NotifyDetailNone"]
 
-
-def wm_check(display, wm_name, upgrading=False):
-    #there should only be one screen... but let's check all of them
-    for i in range(display.get_n_screens()):
-        screen = display.get_screen(i)
-        root = screen.get_root_window()
-        wm_prop = "WM_S%s" % i
-        cwm_prop = "_NEW_WM_CM_S%s" % i
-        wm_so = X11Window.XGetSelectionOwner(wm_prop)
-        cwm_so = X11Window.XGetSelectionOwner(cwm_prop)
-        log("ewmh selection owner for %s: %s", wm_prop, wm_so)
-        log("compositing window manager %s: %s", cwm_prop, cwm_so)
-
-        try:
-            ewmh_wm = prop_get(root, "_NET_SUPPORTING_WM_CHECK", "window", ignore_errors=True, raise_xerrors=False)
-        except:
-            #errors here generally indicate that the window is gone
-            #which is fine: it means the previous window manager is no longer active
-            continue
-        log("_NET_SUPPORTING_WM_CHECK for screen %s: %s", i, ewmh_wm)
-        if ewmh_wm:
-            try:
-                name = prop_get(ewmh_wm, "_NET_WM_NAME", "utf8", ignore_errors=False, raise_xerrors=False)
-            except:
-                name = None
-            if upgrading and name and name==wm_name:
-                log.info("found previous Xpra instance")
-            else:
-                log.warn("Warning: found an existing window manager on screen %s using window %#x: %s", i, ewmh_wm.xid, name or "unknown")
-            if (wm_so is None or wm_so==0) and (cwm_so is None or cwm_so==0):
-                log.error("it does not own the selection '%s' or '%s' so we cannot take over and make it exit", wm_prop, cwm_prop)
-                log.error("please stop %s so you can run xpra on this display", name or "the existing window manager")
-                return False
-    return True
+FORCE_REPLACE_WM = os.environ.get("XPRA_FORCE_REPLACE_WM", "0")=="1"
+LOG_MANAGE_FAILURES = os.environ.get("XPRA_LOG_MANAGE_FAILURES", "0")=="1"
 
 
-class Wm(gobject.GObject):
-    _NET_SUPPORTED = [
+NO_NET_SUPPORTED = os.environ.get("XPRA_NO_NET_SUPPORTED", "").split(",")
+
+DEFAULT_NET_SUPPORTED = [
         "_NET_SUPPORTED", # a bit redundant, perhaps...
         "_NET_SUPPORTING_WM_CHECK",
         "_NET_WM_FULL_PLACEMENT",
@@ -102,6 +62,8 @@ class Wm(gobject.GObject):
         "WM_CLIENT_MACHINE",
         "WM_STATE",
 
+        "_NET_WM_FULLSCREEN_MONITORS",
+
         "_NET_WM_ALLOWED_ACTIONS",
         "_NET_WM_ACTION_CLOSE",
         "_NET_WM_ACTION_FULLSCREEN",
@@ -122,6 +84,8 @@ class Wm(gobject.GObject):
         "_NET_WM_STRUT_PARTIAL"
         "_NET_WM_ICON",
 
+        "_NET_CLOSE_WINDOW",
+
         # These aren't supported in any particularly meaningful way, but hey.
         "_NET_FRAME_EXTENTS",
 
@@ -145,27 +109,75 @@ class Wm(gobject.GObject):
         "_NET_WM_STATE_DEMANDS_ATTENTION",
         "_NET_WM_STATE_MODAL",
         # More states to support:
-        # _NET_WM_STATE_STICKY,
+        "_NET_WM_STATE_STICKY",
         "_NET_WM_STATE_MAXIMIZED_VERT",
         "_NET_WM_STATE_MAXIMIZED_HORZ",
-        # _NET_WM_STATE_SHADED,
+        "_NET_WM_STATE_SHADED",
         "_NET_WM_STATE_SKIP_TASKBAR",
         "_NET_WM_STATE_SKIP_PAGER",
         "_NET_WM_STATE_HIDDEN",
         "_NET_WM_STATE_FULLSCREEN",
-        # _NET_WM_STATE_ABOVE,
-        # _NET_WM_STATE_BELOW,
+        "_NET_WM_STATE_ABOVE",
+        "_NET_WM_STATE_BELOW",
+
+        "_NET_WM_MOVERESIZE",
+        "_NET_MOVERESIZE_WINDOW",
 
         "_MOTIF_WM_HINTS",
         "_MOTIF_WM_INFO",
-        "_NET_WM_MOVERESIZE",
-
         # Not at all yet:
         #"_NET_REQUEST_FRAME_EXTENTS",
-        #"_NET_CLOSE_WINDOW",
         #"_NET_RESTACK_WINDOW",
         #"_NET_WM_DESKTOP",
         ]
+NET_SUPPORTED = [x for x in DEFAULT_NET_SUPPORTED if x not in NO_NET_SUPPORTED]
+
+
+def wm_check(display, wm_name, upgrading=False):
+    #there should only be one screen... but let's check all of them
+    for i in range(display.get_n_screens()):
+        screen = display.get_screen(i)
+        root = screen.get_root_window()
+        wm_prop = "WM_S%s" % i
+        cwm_prop = "_NEW_WM_CM_S%s" % i
+        wm_so = X11Window.XGetSelectionOwner(wm_prop)
+        cwm_so = X11Window.XGetSelectionOwner(cwm_prop)
+        log("ewmh selection owner for %s: %s", wm_prop, wm_so)
+        log("compositing window manager %s: %s", cwm_prop, cwm_so)
+
+        try:
+            ewmh_wm = prop_get(root, "_NET_SUPPORTING_WM_CHECK", "window", ignore_errors=True, raise_xerrors=False)
+        except:
+            #errors here generally indicate that the window is gone
+            #which is fine: it means the previous window manager is no longer active
+            continue
+        def xid(w):
+            if w:
+                return "%#x" % w.xid
+            return None
+        log("_NET_SUPPORTING_WM_CHECK for screen %s: %s (root=%s)", i, xid(ewmh_wm), xid(root))
+        if ewmh_wm:
+            try:
+                name = prop_get(ewmh_wm, "_NET_WM_NAME", "utf8", ignore_errors=False, raise_xerrors=False)
+            except:
+                name = None
+            if upgrading and name and name==wm_name:
+                log.info("found previous Xpra instance")
+            else:
+                log.warn("Warning: found an existing window manager on screen %s using window %#x: %s", i, ewmh_wm.xid, name or "unknown")
+            if (wm_so is None or wm_so==0) and (cwm_so is None or cwm_so==0):
+                if FORCE_REPLACE_WM:
+                    log.warn("XPRA_FORCE_REPLACE_WM is set, replacing it forcibly")
+                else:
+                    log.error("it does not own the selection '%s' or '%s' so we cannot take over and make it exit", wm_prop, cwm_prop)
+                    log.error("please stop %s so you can run xpra on this display", name or "the existing window manager")
+                    log.warn("if you are certain that the window manager is already gone,")
+                    log.warn(" you may set XPRA_FORCE_REPLACE_WM=1 to force xpra to continue, at your own risk")
+                    return False
+    return True
+
+
+class Wm(gobject.GObject):
 
     __gproperties__ = {
         "windows": (gobject.TYPE_PYOBJECT,
@@ -181,12 +193,11 @@ class Wm(gobject.GObject):
         "new-window": one_arg_signal,
         # X11 bell event:
         "bell": one_arg_signal,
+        "show-desktop": one_arg_signal,
         # You can emit this to cause the WM to quit, or the WM may
         # spontaneously raise it if another WM takes over the display.  By
         # default, unmanages all windows:
         "quit": no_arg_signal,
-        # Emit this when the list of desktop names has changed:
-        "desktop-list-changed": one_arg_signal,
 
         # Mostly intended for internal use:
         "child-map-request-event": one_arg_signal,
@@ -203,7 +214,6 @@ class Wm(gobject.GObject):
         if display is None:
             display = gtk.gdk.display_manager_get().get_default_display()
         self._display = display
-        self._alt_display = gtk.gdk.Display(self._display.get_name())
         self._root = self._display.get_default_screen().get_root_window()
         self._wm_name = wm_name
         self._ewmh_window = None
@@ -229,18 +239,17 @@ class Wm(gobject.GObject):
         # Set up the necessary EWMH properties on the root window.
         self._setup_ewmh_window()
         # Start with just one desktop:
-        self.do_desktop_list_changed([u"Main"])
+        self.set_desktop_list([u"Main"])
         self.set_current_desktop(0)
         # Start with the full display as workarea:
         root_w, root_h = gtk.gdk.get_default_root_window().get_size()
-        self.root_set("_NET_SUPPORTED", ["atom"], self._NET_SUPPORTED)
+        self.root_set("_NET_SUPPORTED", ["atom"], NET_SUPPORTED)
         self.set_workarea(0, 0, root_w, root_h)
         self.set_desktop_geometry(root_w, root_h)
         self.root_set("_NET_DESKTOP_VIEWPORT", ["u32"], [0, 0])
 
         # Load up our full-screen widget
-        self._world_window = WorldWindow()
-        self._world_window.set_screen(self._display.get_default_screen())
+        self._world_window = WorldWindow(self._display.get_default_screen())
         self.notify("toplevel")
         self._world_window.show_all()
 
@@ -276,12 +285,17 @@ class Wm(gobject.GObject):
         #(and will not be honoured by anything else..)
         self.root_set("dummy-constant-xdpi", "u32", xdpi)
         self.root_set("dummy-constant-ydpi", "u32", ydpi)
+        screenlog("set_dpi(%s, %s)", xdpi, ydpi)
 
     def set_workarea(self, x, y, width, height):
-        self.root_set("_NET_WORKAREA", ["u32"], [x, y, width, height])
+        v = [x, y, width, height]
+        screenlog("_NET_WORKAREA=%s", v)
+        self.root_set("_NET_WORKAREA", ["u32"], v)
 
     def set_desktop_geometry(self, width, height):
-        self.root_set("_NET_DESKTOP_GEOMETRY", ["u32"], [width, height])
+        v = [width, height]
+        screenlog("_NET_DESKTOP_GEOMETRY=%s", v)
+        self.root_set("_NET_DESKTOP_GEOMETRY", ["u32"], v)
 
 
     def enableCursors(self, on):
@@ -301,7 +315,7 @@ class Wm(gobject.GObject):
 
     def do_get_property(self, pspec):
         if pspec.name == "windows":
-            return ImmutableSet(self._windows.itervalues())
+            return frozenset(self._windows.itervalues())
         elif pspec.name == "toplevel":
             return self._world_window
         else:
@@ -310,11 +324,18 @@ class Wm(gobject.GObject):
     # This is in some sense the key entry point to the entire WM program.  We
     # have detected a new client window, and start managing it:
     def _manage_client(self, gdkwindow):
+        if gdkwindow in self._windows:
+            #already managed
+            return
         try:
-            if gdkwindow not in self._windows:
-                trap.call_synced(self.do_manage_client, gdkwindow)
-        except Exception, e:
-            log("failed to manage client %s: %s", gdkwindow, e)
+            with xsync:
+                self.do_manage_client(gdkwindow)
+        except Exception as e:
+            if LOG_MANAGE_FAILURES or type(e) not in (Unmanageable, ):
+                l = log.warn
+            else:
+                l = log
+            l("failed to manage client %s: %s", gdkwindow, e, exc_info=True)
 
     def do_manage_client(self, gdkwindow):
         try:
@@ -346,16 +367,14 @@ class Wm(gobject.GObject):
         # in a moment we'll get a signal telling us about the window that
         # doesn't exist anymore, will remove it from the list, and then call
         # _update_window_list again.
-        trap.swallow_synced(self.root_set, "_NET_CLIENT_LIST",
-                     ["window"], self._windows_in_order)
-        # This is a lie, but we don't maintain a stacking order, so...
-        trap.swallow_synced(self.root_set, "_NET_CLIENT_LIST_STACKING",
-                     ["window"], self._windows_in_order)
+        with xswallow:
+            self.root_set("_NET_CLIENT_LIST", ["window"], self._windows_in_order)
+            # This is a lie, but we don't maintain a stacking order, so...
+            self.root_set("_NET_CLIENT_LIST_STACKING", ["window"], self._windows_in_order)
 
     def do_xpra_client_message_event(self, event):
         # FIXME
         # Need to listen for:
-        #   _NET_CLOSE_WINDOW
         #   _NET_ACTIVE_WINDOW
         #   _NET_CURRENT_DESKTOP
         #   _NET_REQUEST_FRAME_EXTENTS
@@ -365,6 +384,9 @@ class Wm(gobject.GObject):
         #   _NET_WM_DESKTOP
         #   _NET_WM_STATE
         log("do_xpra_client_message_event(%s)", event)
+        if event.message_type=="_NET_SHOWING_DESKTOP":
+            show = bool(event.data[0])
+            self.emit("show-desktop", show)
 
     def _lost_wm_selection(self, selection):
         log.info("Lost WM selection %s, exiting", selection)
@@ -393,10 +415,9 @@ class Wm(gobject.GObject):
         if event.window in self._windows:
             return
         log("Reconfigure on withdrawn window")
-        trap.swallow_synced(X11Window.configureAndNotify,
-                     event.window.xid, event.x, event.y,
-                     event.width, event.height,
-                     event.value_mask)
+        with xswallow:
+            X11Window.configureAndNotify(event.window.xid, event.x, event.y,
+                     event.width, event.height, event.value_mask)
 
     def do_xpra_focus_in_event(self, event):
         # The purpose of this function is to detect when the focus mode has
@@ -410,9 +431,10 @@ class Wm(gobject.GObject):
     def do_xpra_focus_out_event(self, event):
         focuslog("wm.do_xpra_focus_out_event(%s) XGetInputFocus=%s", event, X11Window.XGetInputFocus())
 
-    def do_desktop_list_changed(self, desktops):
+    def set_desktop_list(self, desktops):
+        log("set_desktop_list(%s)", desktops)
         self.root_set("_NET_NUMBER_OF_DESKTOPS", "u32", len(desktops))
-        self.root_set("_NET_DESKTOP_NAMES", ["utf8"], desktops)
+        self.root_set("_NET_DESKTOP_NAMES", ["utf8"], [d.decode("utf8") for d in desktops])
 
     def set_current_desktop(self, index):
         self.root_set("_NET_CURRENT_DESKTOP", "u32", index)
@@ -442,16 +464,10 @@ class Wm(gobject.GObject):
         self.root_set("_NET_SUPPORTING_WM_CHECK", "window", self._ewmh_window)
         self.root_set("_NET_WM_NAME", "utf8", self._wm_name.decode("utf8"))
 
-    # Other global actions:
-
-    def _make_window_pseudoclient(self, win):
-        "Used by PseudoclientWindow, only."
-        win.set_screen(self._alt_display.get_default_screen())
-
     def get_net_wm_name(self):
         try:
             return prop_get(self._ewmh_window, "_NET_WM_NAME", "utf8", ignore_errors=False, raise_xerrors=False)
-        except Exception, e:
+        except Exception as e:
             log.error("error querying _NET_WM_NAME: %s", e)
 
 gobject.type_register(Wm)
