@@ -14,6 +14,7 @@
 import os
 import gtk.gdk
 import gobject
+import time
 
 from xpra.util import AdHocStruct
 from xpra.gtk_common.gobject_util import one_arg_signal
@@ -33,7 +34,7 @@ X11Window = X11WindowBindings()
 from xpra.x11.bindings.keyboard_bindings import X11KeyboardBindings #@UnresolvedImport
 X11Keyboard = X11KeyboardBindings()
 from xpra.x11.gtk_x11.window import OverrideRedirectWindowModel, SystemTrayWindowModel, Unmanageable
-from xpra.gtk_common.error import trap
+from xpra.gtk_common.error import trap, xsync
 
 from xpra.log import Logger
 log = Logger("server")
@@ -43,10 +44,12 @@ windowlog = Logger("server", "window")
 cursorlog = Logger("server", "cursor")
 traylog = Logger("server", "tray")
 settingslog = Logger("x11", "xsettings")
+workspacelog = Logger("x11", "workspace")
+metadatalog = Logger("x11", "metadata")
 
 import xpra
-from xpra.util import nonl
-from xpra.os_util import StringIOClass
+from xpra.util import nonl, typedict
+from xpra.os_util import StringIOClass, _memoryview
 from xpra.x11.x11_server_base import X11ServerBase, mouselog
 from xpra.net.compression import Compressed
 
@@ -100,16 +103,25 @@ class DesktopManager(gtk.Widget):
     def configure_window(self, win, x, y, w, h, resize_counter=0):
         log("DesktopManager.configure_window(%s, %s, %s, %s, %s, %s)", win, x, y, w, h, resize_counter)
         model = self._models[win]
+        new_geom = [x, y, w, h]
+        update_geometry = False
+        if model.geom!=new_geom:
+            if resize_counter>0 and resize_counter<model.resize_counter:
+                log("resize ignored: counter %s vs %s", resize_counter, model.resize_counter)
+            else:
+                update_geometry = True
+                model.geom = new_geom
         if not self.visible(win):
             model.shown = True
+            win.map()
+            #Note: this will fire a metadata change event, which will fire a message to the client(s),
+            #which is wasteful when we only have one client and it is the one that configured the window,
+            #but when we have multiple clients, this keeps things in sync
             win.set_property("iconic", False)
-            win.ownership_election()
-        if resize_counter>0 and resize_counter<model.resize_counter:
-            log("resize ignored: counter %s vs %s", resize_counter, model.resize_counter)
-            return
-        new_geom = [x, y, w, h]
-        if model.geom!=new_geom:
-            model.geom = [x, y, w, h]
+            if win.ownership_election():
+                #window has been configured already
+                update_geometry = False
+        if update_geometry:
             win.maybe_recalculate_geometry_for(self)
 
     def hide_window(self, model):
@@ -158,14 +170,14 @@ class XpraServer(gobject.GObject, X11ServerBase):
         "xpra-cursor-event": one_arg_signal,
         }
 
-    def __init__(self):
+    def __init__(self, clobber):
         gobject.GObject.__init__(self)
-        X11ServerBase.__init__(self)
+        X11ServerBase.__init__(self, clobber)
 
-    def init(self, clobber, opts):
+    def init(self, opts):
         self.xsettings_enabled = opts.xsettings
         self.wm_name = opts.wm_name
-        X11ServerBase.init(self, clobber, opts)
+        X11ServerBase.init(self, opts)
 
     def x11_init(self):
         X11ServerBase.x11_init(self)
@@ -189,6 +201,7 @@ class XpraServer(gobject.GObject, X11ServerBase):
         self._wm.connect("new-window", self._new_window_signaled)
         self._wm.connect("bell", self._bell_signaled)
         self._wm.connect("quit", lambda _: self.quit(True))
+        self._wm.connect("show-desktop", self._show_desktop)
 
         #save default xsettings:
         self.default_xsettings = XSettingsHelper().get_settings()
@@ -196,6 +209,10 @@ class XpraServer(gobject.GObject, X11ServerBase):
         self._settings = {}
         self._xsettings_manager = None
 
+        #for handling resize synchronization between client and server (this is not xsync!):
+        self.last_client_configure_event = 0
+        self.snc_timer = 0
+
         #cursor:
         self.default_cursor_data = None
         self.last_cursor_serial = None
@@ -213,12 +230,15 @@ class XpraServer(gobject.GObject, X11ServerBase):
         if source.wants_features:
             capabilities["window.raise"] = True
             capabilities["window.resize-counter"] = True
+            capabilities["window.configure.skip-geometry"] = True
             capabilities["pointer.grabs"] = True
         return capabilities
 
 
     def do_get_info(self, proto, server_sources, window_ids):
         info = X11ServerBase.do_get_info(self, proto, server_sources, window_ids)
+        info["focused"] = self._has_focus
+        info["grabbed"] = self._has_grab
         log("do_get_info: adding cursor=%s", self.last_cursor_data)
         #copy to prevent race:
         cd = self.last_cursor_data
@@ -227,20 +247,22 @@ class XpraServer(gobject.GObject, X11ServerBase):
         else:
             info["cursor.is_default"] = bool(self.default_cursor_data and len(self.default_cursor_data)>=8 and len(cd)>=8 and cd[7]==cd[7])
             #all but pixels:
-            i = 0
-            for x in ("x", "y", "width", "height", "xhot", "yhot", "serial", None, "name"):
+            for i, x in enumerate(("x", "y", "width", "height", "xhot", "yhot", "serial", None, "name")):
                 if x:
                     v = cd[i] or ""
                     info["cursor." + x] = v
-                i += 1
         return info
 
     def get_ui_info(self, proto, wids, *args):
         info = X11ServerBase.get_ui_info(self, proto, wids, *args)
         #_NET_WM_NAME:
-        info["window-manager-name"] = self._wm.get_net_wm_name()
+        wm = self._wm
+        if wm:
+            info["window-manager-name"] = wm.get_net_wm_name()
         #now cursor size info:
         display = gtk.gdk.display_get_default()
+        pos = display.get_default_screen().get_root_window().get_pointer()[:2]
+        info["cursor.position"] = pos
         for prop, size in {"default" : display.get_default_cursor_size(),
                            "max"     : display.get_maximal_cursor_size()}.items():
             if size is None:
@@ -256,14 +278,21 @@ class XpraServer(gobject.GObject, X11ServerBase):
         return info
 
 
+    def set_desktops(self, names):
+        if self._wm:
+            self._wm.set_desktop_list(names)
+
     def set_workarea(self, workarea):
-        self._wm.set_workarea(workarea.x, workarea.y, workarea.width, workarea.height)
+        if self._wm:
+            self._wm.set_workarea(workarea.x, workarea.y, workarea.width, workarea.height)
 
     def set_desktop_geometry(self, width, height):
-        self._wm.set_desktop_geometry(width, height)
+        if self._wm:
+            self._wm.set_desktop_geometry(width, height)
 
     def set_dpi(self, xdpi, ydpi):
-        self._wm.set_dpi(xdpi, ydpi)
+        if self._wm:
+            self._wm.set_dpi(xdpi, ydpi)
 
 
     def get_transient_for(self, window):
@@ -321,7 +350,7 @@ class XpraServer(gobject.GObject, X11ServerBase):
         if system_tray:
             try:
                 self._tray = SystemTray()
-            except Exception, e:
+            except Exception as e:
                 log.error("cannot setup tray forwarding: %s", e, exc_info=True)
 
         ### Create our window managing data structures:
@@ -379,7 +408,7 @@ class XpraServer(gobject.GObject, X11ServerBase):
         display = gtk.gdk.display_get_default()
         self.cursor_sizes = display.get_default_cursor_size(), display.get_maximal_cursor_size()
         cursorlog("cursor_sizes=%s", self.cursor_sizes)
-        ss.send_cursor(self.get_cursor_data)
+        ss.send_cursor()
 
 
     def _new_window_signaled(self, wm, window):
@@ -392,7 +421,7 @@ class XpraServer(gobject.GObject, X11ServerBase):
 
     def _add_new_window_common(self, window):
         windowlog("adding window %s", window)
-        for prop in self._window_export_properties:
+        for prop in window.get_dynamic_property_names():
             window.connect("notify::%s" % prop, self._update_metadata)
         wid = X11ServerBase._add_new_window_common(self, window)
         window.managed_connect("client-contents-changed", self._contents_changed)
@@ -403,7 +432,6 @@ class XpraServer(gobject.GObject, X11ServerBase):
         window.managed_connect("ungrab", self._window_ungrab)
         return wid
 
-    _window_export_properties = ("title", "size-hints", "fullscreen", "maximized", "opacity", "iconic")
     def _add_new_window(self, window):
         self._add_new_window_common(window)
         _, _, w, h, _ = window.get_property("client-window").get_geometry()
@@ -411,8 +439,14 @@ class XpraServer(gobject.GObject, X11ServerBase):
         windowlog("Discovered new ordinary window: %s (geometry=%s)", window, (x, y, w, h))
         self._desktop_manager.add_window(window, x, y, w, h)
         window.connect("notify::geometry", self._window_resized_signaled)
+        window.connect("notify::iconic", self._iconic_changed)
         self._send_new_window_packet(window)
 
+    def _iconic_changed(self, window, pspec):
+        #only defined for debugging purposes
+        log("_iconic_changed(%s, %s) iconic=%s, shown=%s", window, pspec, window.get_property("iconic"), self._desktop_manager.is_shown(window))
+
+
     def _window_resized_signaled(self, window, *args):
         nw, nh = window.get_property("actual-size")
         x, y = window.get_position()
@@ -422,9 +456,32 @@ class XpraServer(gobject.GObject, X11ServerBase):
             #unchanged
             return
         geom[:4] = [x, y, nw, nh]
+        lcce = self.last_client_configure_event
+        if self.snc_timer>0:
+            gobject.source_remove(self.snc_timer)
+        #TODO: find a better way to choose the timer delay:
+        #for now, we wait at least 100ms, up to 250ms if the client has just sent us a resize:
+        #(lcce should always be in the past, so min(..) should be redundant here)
+        delay = max(100, min(250, 250 + 1000 * (lcce-time.time())))
+        self.snc_timer = gobject.timeout_add(int(delay), self.size_notify_clients, window, lcce)
+
+    def size_notify_clients(self, window, lcce):
+        log("size_notify_clients(%s, %s) last_client_configure_event=%s", window, lcce, self.last_client_configure_event)
+        self.snc_timer = 0
+        wid = self._window_to_id.get(window)
+        if lcce!=self.last_client_configure_event or not wid:
+            #we have received a new client resize since,
+            #or the window is simply gone
+            return
+        geom = self._desktop_manager.window_geometry(window)
+        x, y, nw, nh = geom[:4]
         resize_counter = self._desktop_manager.get_resize_counter(window, 1)
         for ss in self._server_sources.values():
             ss.move_resize_window(self._window_to_id[window], window, x, y, nw, nh, resize_counter)
+            #refresh to ensure the client gets the new window contents:
+            #TODO: to save bandwidth, we should compare the dimensions and skip the refresh
+            #if the window is smaller than before, or at least only send the new edges rather than the whole window
+            ss.damage(wid, window, 0, 0, nw, nh, {})
 
     def _add_new_or_window(self, raw_window):
         xid = raw_window.xid
@@ -464,7 +521,7 @@ class XpraServer(gobject.GObject, X11ServerBase):
                 window.call_setup()
                 window.connect("notify::geometry", self._or_window_geometry_changed)
                 self._send_new_or_window_packet(window)
-        except Unmanageable, e:
+        except Unmanageable as e:
             if window:
                 windowlog("window %s is not manageable: %s", window, e)
                 #if window is set, we failed after instantiating it,
@@ -478,9 +535,9 @@ class XpraServer(gobject.GObject, X11ServerBase):
             #so we *should* get a signal when the window goes away
 
     def _or_window_geometry_changed(self, window, pspec=None):
-        (x, y, w, h) = window.get_property("geometry")
+        (x, y, w, h) = window.get_property("geometry")[:4]
         if w>=32768 or h>=32768:
-            self.error("not sending new invalid window dimensions: %ix%i !", w, h)
+            windowlog.error("not sending new invalid window dimensions: %ix%i !", w, h)
             return
         windowlog("or_window_geometry_changed: %s (window=%s)", window.get_property("geometry"), window)
         wid = self._window_to_id[window]
@@ -497,7 +554,7 @@ class XpraServer(gobject.GObject, X11ServerBase):
         cursorlog("cursor_event: %s", event)
         self.last_cursor_serial = event.cursor_serial
         for ss in self._server_sources.values():
-            ss.send_cursor(self.get_cursor_data)
+            ss.send_cursor()
         return False
 
 
@@ -516,6 +573,11 @@ class XpraServer(gobject.GObject, X11ServerBase):
             ss.bell(wid, event.device, event.percent, event.pitch, event.duration, event.bell_class, event.bell_id, event.bell_name or "")
 
 
+    def _show_desktop(self, wm, show):
+        log("show_desktop(%s, %s)", wm, show)
+        for ss in self._server_sources.values():
+            ss.show_desktop(show)
+
 
     def _focus(self, server_source, wid, modifiers):
         focuslog("focus wid=%s has_focus=%s", wid, self._has_focus)
@@ -638,73 +700,125 @@ class XpraServer(gobject.GObject, X11ServerBase):
 
 
     def _set_window_state(self, proto, wid, window, new_window_state):
-        #only used for setting maximized state:
-        if "maximized" in new_window_state:
-            new_state = bool(new_window_state.get("maximized", False))
-            cur_state = bool(window.get_property("maximized"))
-            if cur_state!=new_state:
-                window.set_property("maximized", new_state)
-        #TODO: also sync "above", "below", "sticky" and "fullscreen"
+        metadatalog("set_window_state%s", (wid, window, new_window_state))
+        changes = []
+        for k in ("maximized", "above", "below", "fullscreen", "sticky", "shaded", "skip-pager", "skip-taskbar", "iconified"):
+            if k in new_window_state:
+                #stupid naming conflict (should have used the same at both ends):
+                wpropname = {"iconified" : "iconic"}.get(k, k)
+                new_state = bool(new_window_state.get(k, False))
+                cur_state = bool(window.get_property(wpropname))
+                metadatalog("set window state for '%s': current state=%s, new state=%s", k, cur_state, new_state)
+                if cur_state!=new_state:
+                    window.set_property(wpropname, new_state)
+                    changes.append(k)
+        metadatalog("set_window_state: changes=%s", changes)
+        return changes
 
     def _process_map_window(self, proto, packet):
-        wid, x, y, width, height = packet[1:6]
+        wid, x, y, w, h = packet[1:6]
         window = self._id_to_window.get(wid)
         if not window:
             windowlog("cannot map window %s: already removed!", wid)
             return
         assert not window.is_OR()
-        windowlog("client mapped window %s - %s, at: %s", wid, window, (x, y, width, height))
+        windowlog("client mapped window %s - %s, at: %s", wid, window, (x, y, w, h))
         if len(packet)>=8:
             self._set_window_state(proto, wid, window, packet[7])
-        self._desktop_manager.configure_window(window, x, y, width, height)
+        ax, ay, aw, ah = self._clamp_window(proto, wid, window, x, y, w, h)
+        self._desktop_manager.configure_window(window, ax, ay, aw, ah)
         self._desktop_manager.show_window(window)
         if len(packet)>=7:
             self._set_client_properties(proto, wid, window, packet[6])
-        self._damage(window, 0, 0, width, height)
+        self._damage(window, 0, 0, w, h)
 
 
     def _process_unmap_window(self, proto, packet):
         wid = packet[1]
-        iconified = len(packet)>=3 and bool(packet[2])
         window = self._id_to_window.get(wid)
         if not window:
             log("cannot map window %s: already removed!", wid)
             return
+        if len(packet)>=4:
+            #optional window_state added in 0.15 to update flags
+            #during iconification events:
+            self._set_window_state(proto, wid, window, packet[3])
         assert not window.is_OR()
         windowlog("client unmapped window %s - %s", wid, window)
         for ss in self._server_sources.values():
             ss.unmap_window(wid, window)
+        window.unmap()
+        iconified = len(packet)>=3 and bool(packet[2])
         if iconified and not window.get_property("iconic"):
             window.set_property("iconic", True)
         self._desktop_manager.hide_window(window)
 
+    def _clamp_window(self, proto, wid, window, x, y, w, h):
+        rw, rh = self.get_root_window_size()
+        #clamp to root window size
+        if x>=rw or y>=rh:
+            log("clamping window position %ix%i to root window size %ix%i", x, y, rw, rh)
+            x = max(0, min(x, rw-w))
+            y = max(0, min(y, rh-h))
+            #tell this client to honour the new location
+            ss = self._server_sources.get(proto)
+            if ss:
+                resize_counter = self._desktop_manager.get_resize_counter(window, 1)
+                ss.move_resize_window(wid, window, x, y, w, h, resize_counter)
+        return x, y, w, h
+
     def _process_configure_window(self, proto, packet):
         wid, x, y, w, h = packet[1:6]
         resize_counter = 0
         if len(packet)>=8:
             resize_counter = packet[7]
+        #some "configure-window" packets are only meant for metadata updates:
+        skip_geometry = len(packet)>=10 and packet[9]
         window = self._id_to_window.get(wid)
         windowlog("client configured window %s - %s, at: %s", wid, window, (x, y, w, h))
         if not window:
             windowlog("cannot map window %s: already removed!", wid)
             return
+        damage = False
         if window.is_tray():
             assert self._tray
-            traylog("tray %s configured to: %s", window, (x, y, w, h))
-            self._tray.move_resize(window, x, y, w, h)
+            if not skip_geometry:
+                traylog("tray %s configured to: %s", window, (x, y, w, h))
+                self._tray.move_resize(window, x, y, w, h)
+                damage = True
         else:
-            assert not window.is_OR()
+            assert not window.is_OR() or skip_geometry, "received a configure packet with geometry for OR window %s from %s: %s" % (window, proto, packet)
+            self.last_client_configure_event = time.time()
             if len(packet)>=9:
-                self._set_window_state(proto, wid, window, packet[8])
-            owx, owy, oww, owh = self._desktop_manager.window_geometry(window)
-            windowlog("_process_configure_window(%s) old window geometry: %s", packet[1:], (owx, owy, oww, owh))
-            self._desktop_manager.configure_window(window, x, y, w, h, resize_counter)
+                changes = self._set_window_state(proto, wid, window, packet[8])
+                damage = len(changes)>0
+            if not skip_geometry:
+                owx, owy, oww, owh = self._desktop_manager.window_geometry(window)
+                windowlog("_process_configure_window(%s) old window geometry: %s", packet[1:], (owx, owy, oww, owh))
+                ax, ay, aw, ah = self._clamp_window(proto, wid, window, x, y, w, h)
+                self._desktop_manager.configure_window(window, ax, ay, aw, ah, resize_counter)
+                damage |= owx!=ax or owy!=ay or oww!=aw or owh!=ah
         if len(packet)>=7:
             self._set_client_properties(proto, wid, window, packet[6])
-        if window.is_tray() or (self._desktop_manager.visible(window) and (oww!=w or owh!=h)):
+        if damage:
             self._damage(window, 0, 0, w, h)
 
 
+    def _set_client_properties(self, proto, wid, window, new_client_properties):
+        """
+        Override so we can update the workspace on the window directly,
+        instead of storing it as a client property
+        """
+        workspace = new_client_properties.get("workspace")
+        workspacelog("workspace from client properties %s: %s", new_client_properties, workspace)
+        if workspace is not None:
+            window.move_to_workspace(workspace)
+            #we have handled it on the window directly, so remove it from client properties
+            del new_client_properties["workspace"]
+        #handle the rest as normal:
+        X11ServerBase._set_client_properties(self, proto, wid, window, new_client_properties)
+
+
     """ override so we can raise the window under the cursor
         (gtk raise does not change window stacking, just focus) """
     def _move_pointer(self, wid, pos):
@@ -735,37 +849,37 @@ class XpraServer(gobject.GObject, X11ServerBase):
             return None
 
     def do_make_screenshot_packet(self):
-        debug = log.debug
-        debug("grabbing screenshot")
+        log("grabbing screenshot")
         regions = []
         OR_regions = []
         for wid in reversed(sorted(self._id_to_window.keys())):
             window = self._id_to_window.get(wid)
-            debug("screenshot: window(%s)=%s", wid, window)
+            log("screenshot: window(%s)=%s", wid, window)
             if window is None:
                 continue
             if window.is_tray():
-                debug("screenshot: skipping tray window %s", wid)
+                log("screenshot: skipping tray window %s", wid)
                 continue
             if not window.is_managed():
-                debug("screenshot: window %s is not/no longer managed", wid)
+                log("screenshot: window %s is not/no longer managed", wid)
                 continue
             if window.is_OR():
                 x, y = window.get_property("geometry")[:2]
             else:
                 x, y = self._desktop_manager.window_geometry(window)[:2]
-            debug("screenshot: position(%s)=%s,%s", window, x, y)
+            log("screenshot: position(%s)=%s,%s", window, x, y)
             w, h = window.get_dimensions()
-            debug("screenshot: size(%s)=%sx%s", window, w, h)
+            log("screenshot: size(%s)=%sx%s", window, w, h)
             try:
-                img = trap.call_synced(window.get_image, 0, 0, w, h)
+                with xsync:
+                    img = window.get_image(0, 0, w, h)
             except:
                 log.warn("screenshot: window %s could not be captured", wid)
                 continue
             if img is None:
                 log.warn("screenshot: no pixels for window %s", wid)
                 continue
-            debug("screenshot: image=%s, size=%s", (img, img.get_size()))
+            log("screenshot: image=%s, size=%s", (img, img.get_size()))
             if img.get_pixel_format() not in ("RGB", "RGBA", "XRGB", "BGRX", "ARGB", "BGRA"):
                 log.warn("window pixels for window %s using an unexpected rgb format: %s", wid, img.get_pixel_format())
                 continue
@@ -779,9 +893,9 @@ class XpraServer(gobject.GObject, X11ServerBase):
                 regions.append(item)
         all_regions = OR_regions+regions
         if len(all_regions)==0:
-            debug("screenshot: no regions found, returning empty 0x0 image!")
+            log("screenshot: no regions found, returning empty 0x0 image!")
             return ["screenshot", 0, 0, "png", -1, ""]
-        debug("screenshot: found regions=%s, OR_regions=%s", len(regions), len(OR_regions))
+        log("screenshot: found regions=%s, OR_regions=%s", len(regions), len(OR_regions))
         #in theory, we could run the rest in a non-UI thread since we're done with GTK..
         minx = min([x for (_,x,_,_) in all_regions])
         miny = min([y for (_,_,y,_) in all_regions])
@@ -789,7 +903,7 @@ class XpraServer(gobject.GObject, X11ServerBase):
         maxy = max([(y+img.get_height()) for (_,_,y,img) in all_regions])
         width = maxx-minx
         height = maxy-miny
-        debug("screenshot: %sx%s, min x=%s y=%s", width, height, minx, miny)
+        log("screenshot: %sx%s, min x=%s y=%s", width, height, minx, miny)
         from PIL import Image                           #@UnresolvedImport
         screenshot = Image.new("RGBA", (width, height))
         for wid, x, y, img in reversed(all_regions):
@@ -798,10 +912,14 @@ class XpraServer(gobject.GObject, X11ServerBase):
                      "XRGB"   : "RGB",
                      "BGRX"   : "RGB",
                      "BGRA"   : "RGBA"}.get(pixel_format, pixel_format)
+            pixels = img.get_pixels()
+            #PIL cannot use the memoryview directly:
+            if _memoryview and isinstance(pixels, _memoryview):
+                pixels = pixels.tobytes()
             try:
-                window_image = Image.frombuffer(target_format, (w, h), img.get_pixels(), "raw", pixel_format, img.get_rowstride())
+                window_image = Image.frombuffer(target_format, (w, h), pixels, "raw", pixel_format, img.get_rowstride())
             except:
-                log.warn("failed to parse window pixels in %s format", pixel_format)
+                log.error("Error parsing window pixels in %s format", pixel_format, exc_info=True)
                 continue
             tx = x-minx
             ty = y-miny
@@ -811,7 +929,7 @@ class XpraServer(gobject.GObject, X11ServerBase):
         data = buf.getvalue()
         buf.close()
         packet = ["screenshot", width, height, "png", width*4, Compressed("png", data)]
-        debug("screenshot: %sx%s %s", packet[1], packet[2], packet[-1])
+        log("screenshot: %sx%s %s", packet[1], packet[2], packet[-1])
         return packet
 
 
@@ -826,20 +944,41 @@ class XpraServer(gobject.GObject, X11ServerBase):
             self._xsettings_manager = XSettingsManager()
         self._xsettings_manager.set_settings(v)
 
+    def _get_antialias_hintstyle(self):
+        ad = typedict(self.antialias)
+        hintstyle = ad.strget("hintstyle", "").lower()
+        if hintstyle in ("hintnone", "hintslight", "hintmedium", "hintfull"):
+            #X11 clients can give us what we need directly:
+            return hintstyle
+        #win32 style contrast value:
+        contrast = ad.intget("contrast", -1)
+        if contrast>1600:
+            return "hintfull"
+        elif contrast>1000:
+            return "hintmedium"
+        elif contrast>0:
+            return "hintslight"
+        return "hintnone"
+
     def update_server_settings(self, settings, reset=False):
         if not self.xsettings_enabled:
             settingslog("ignoring xsettings update: %s", settings)
             return
         if reset:
+            #FIXME: preserve serial? (what happens when we change values which had the same serial?)
             self.reset_settings()
-            self._settings = self.default_xsettings or {}
+            self._settings = {}
+            if self.default_xsettings:
+                self._settings = self._default_xsettings[1]
         old_settings = dict(self._settings)
         settingslog("server_settings: old=%s, updating with=%s", nonl(old_settings), nonl(settings))
+        settingslog("overrides: dpi=%s, double click time=%s, double click distance=%s", self.dpi, self.double_click_time, self.double_click_distance)
+        settingslog("overrides: antialias=%s", self.antialias)
         self._settings.update(settings)
         root = gtk.gdk.get_default_root_window()
         for k, v in settings.items():
             #cook the "resource-manager" value to add the DPI:
-            if k == "resource-manager" and self.dpi>0:
+            if k=="resource-manager" and self.dpi>0:
                 value = v.decode("utf-8")
                 #parse the resources into a dict:
                 values={}
@@ -852,7 +991,15 @@ class XpraServer(gobject.GObject, X11ServerBase):
                         continue
                     values[parts[0]] = parts[1]
                 values["Xft.dpi"] = self.dpi
+                values["Xft/DPI"] = self.dpi*1024
                 values["gnome.Xft/DPI"] = self.dpi*1024
+                if self.antialias:
+                    ad = typedict(self.antialias)
+                    values.update({
+                                   "Xft.antialias"  : ad.intget("enabled", -1),
+                                   "Xft.hinting"    : ad.intget("hinting", -1),
+                                   "Xft.rgba"       : ad.strget("orientation", "none").lower(),
+                                   "Xft.hintstyle"  : self._get_antialias_hintstyle()})
                 settingslog("server_settings: resource-manager values=%s", nonl(values))
                 #convert the dict back into a resource string:
                 value = ''
@@ -862,6 +1009,42 @@ class XpraServer(gobject.GObject, X11ServerBase):
                 self._settings["resource-manager"] = value
                 v = value.encode("utf-8")
 
+            #cook xsettings to add double-click settings:
+            #(as those may not be present in xsettings on some platforms.. like win32 and osx)
+            if k=="xsettings-blob" and (self.double_click_time>0 or self.double_click_distance!=(-1, -1)):
+                from xpra.x11.xsettings_prop import XSettingsTypeInteger, XSettingsTypeString
+                def set_xsettings_value(name, value_type, value):
+                    #remove existing one, if any:
+                    serial, values = v
+                    new_values = [(_t,_n,_v,_s) for (_t,_n,_v,_s) in values if _n!=name]
+                    new_values.append((value_type, name, value, 0))
+                    return serial, new_values
+                def set_xsettings_int(name, value):
+                    if value<0: #not set, return v unchanged
+                        return v
+                    return set_xsettings_value(name, XSettingsTypeInteger, value)
+                if self.dpi>0:
+                    v = set_xsettings_int("Xft/DPI", self.dpi*1024)
+                if self.double_click_time>0:
+                    v = set_xsettings_int("Net/DoubleClickTime", self.double_click_time)
+                if self.antialias:
+                    ad = typedict(self.antialias)
+                    v = set_xsettings_int("Xft/Antialias",  ad.intget("enabled", -1))
+                    v = set_xsettings_int("Xft/Hinting",    ad.intget("hinting", -1))
+                    v = set_xsettings_value("Xft/RGBA",     XSettingsTypeString, ad.strget("orientation", "none").lower())
+                    v = set_xsettings_value("Xft/HintStyle", XSettingsTypeString, self._get_antialias_hintstyle())
+                if self.double_click_distance!=(-1, -1):
+                    #some platforms give us a value for each axis,
+                    #but X11 only has one, so take the average
+                    try:
+                        x,y = self.double_click_distance
+                        if x>0 and y>0:
+                            d = (x+y)//2
+                            d = max(1, min(128, d))     #sanitize it a bit
+                            v = set_xsettings_int("Net/DoubleClickDistance", d)
+                    except Exception as e:
+                        log.warn("error setting double click distance from %s: %s", self.double_click_distance, e)
+
             if k not in old_settings or v != old_settings[k]:
                 def root_set(p):
                     settingslog("server_settings: setting %s to %s", nonl(p), nonl(v))
