@@ -117,15 +117,24 @@ class KeyboardConfig(KeyboardConfigBase):
             to set the keyboard attributes """
         KeyboardConfigBase.parse_options(self, props)
         modded = []
-        for x in ("print", "query", "mod_meanings",
-                  "mod_managed", "mod_pointermissing",
-                  "keycodes", "x11_keycodes"):
-            prop = "xkbmap_%s" % x
+        def parse_option(name, parse_fn):
+            prop = "xkbmap_%s" % name
             cv = getattr(self, prop)
-            nv = props.get(prop)
+            nv = parse_fn(prop)
             if cv!=nv:
                 setattr(self, prop, nv)
                 modded.append(prop)
+        #plain strings:
+        for x in ("print", "query"):
+            parse_option(x, props.strget)
+        #lists:
+        parse_option("keycodes", props.listget)
+        #dicts:
+        for x in ("mod_meanings", "x11_keycodes"):
+            parse_option(x, props.dictget)
+        #lists of strings:
+        for x in ("mod_managed", "mod_pointermissing"):
+            parse_option(x, props.strlistget)
         log("assign_keymap_options(..) modified %s", modded)
         return len(modded)>0
 
@@ -134,13 +143,8 @@ class KeyboardConfig(KeyboardConfigBase):
         """
             This hash will be different whenever the keyboard configuration changes.
         """
-        try:
-            import hashlib
-            m = hashlib.sha1()
-        except:
-            #try python2.4 variant:
-            import sha
-            m = sha.new()
+        import hashlib
+        m = hashlib.sha1()
         m.update(KeyboardConfigBase.get_hash(self))
         for x in (self.xkbmap_print, self.xkbmap_query, \
                   self.xkbmap_mod_meanings, self.xkbmap_mod_pointermissing, \
@@ -186,7 +190,7 @@ class KeyboardConfig(KeyboardConfigBase):
                 self.modifier_client_keycodes[modifier] = client_keycodes
             log("compute_client_modifier_keycodes() mappings=%s", self.modifier_client_keycodes)
             log("compute_client_modifier_keycodes() mod nuisance=%s", self.xkbmap_mod_nuisance)
-        except Exception, e:
+        except Exception as e:
             log.error("do_set_keymap: %s" % e, exc_info=True)
 
     def compute_modifier_map(self):
@@ -346,7 +350,7 @@ class KeyboardConfig(KeyboardConfigBase):
         if ignored_modifier_keynames is None:
             #this is not a keyboard event, ignore modifiers in "mod_pointermissing"
             def is_ignored(modifier, modifier_keynames):
-                m = modifier in self.xkbmap_mod_pointermissing
+                m = modifier in (self.xkbmap_mod_pointermissing or [])
                 log("is_ignored(%s, %s)=%s", modifier, modifier_keynames, m)
                 return m
         else:
