@@ -11,17 +11,21 @@ from xpra.server.shadow_server_base import ShadowServerBase
 from xpra.server.gtk_root_window_model import GTKRootWindowModel
 
 
+class GTKX11RootWindowModel(GTKRootWindowModel):
+
+    def __repr__(self):
+        return "GTKX11RootWindowModel(%#x)" % self.window.xid
+
+
 class ShadowX11Server(ShadowServerBase, X11ServerBase):
 
     def __init__(self):
         ShadowServerBase.__init__(self, gtk.gdk.get_default_root_window())
-        X11ServerBase.__init__(self)
+        X11ServerBase.__init__(self, False)
 
-    def init(self, opts):
-        X11ServerBase.init(self, False, opts)
 
     def makeRootWindowModel(self):
-        return GTKRootWindowModel(self.root)
+        return GTKX11RootWindowModel(self.root)
 
     def _process_mouse_common(self, proto, wid, pointer, modifiers):
         #adjust pointer position for offset in client:
