@@ -9,6 +9,7 @@
 import gtk.gdk
 import gobject
 import os.path
+import sys
 
 #ensure that we use gtk as display source:
 from xpra.x11.gtk_x11 import gdk_display_source
@@ -20,7 +21,7 @@ from xpra.x11.bindings.keyboard_bindings import X11KeyboardBindings #@Unresolved
 X11Keyboard = X11KeyboardBindings()
 from xpra.x11.bindings.core_bindings import X11CoreBindings #@UnresolvedImport
 X11Core = X11CoreBindings()
-from xpra.gtk_common.error import XError, trap
+from xpra.gtk_common.error import XError, xswallow, xsync
 from xpra.server.server_uuid import save_uuid, get_uuid
 
 from xpra.log import Logger
@@ -31,6 +32,7 @@ grablog = Logger("server", "grab")
 cursorlog = Logger("server", "cursor")
 
 from xpra.util import prettify_plug_name
+from xpra.os_util import find_lib, find_lib_ldconfig
 from xpra.server.gtk_server_base import GTKServerBase
 from xpra.x11.xkbhelper import clean_keyboard_state
 from xpra.x11.server_keyboard_config import KeyboardConfig
@@ -65,14 +67,16 @@ class X11ServerBase(GTKServerBase):
         (see XpraServer or XpraX11ShadowServer for actual implementations)
     """
 
-    def init(self, clobber, opts):
+    def __init__(self, clobber):
         self.clobber = clobber
+        GTKServerBase.__init__(self)
+
+    def init(self, opts):
         self.fake_xinerama = opts.fake_xinerama
         self.current_xinerama_config = None
         self.x11_init()
         GTKServerBase.init(self, opts)
 
-
     def x11_init(self):
         self.init_x11_atoms()
         self.randr = RandR.has_randr()
@@ -124,6 +128,26 @@ class X11ServerBase(GTKServerBase):
         self._authenticated_ui_packet_handlers["force-ungrab"] = self._process_force_ungrab
 
 
+    def get_child_env(self):
+        #adds fakexinerama:
+        env = GTKServerBase.get_child_env(self)
+        if self.fake_xinerama:
+            libfakeXinerama_so = self.find_fakeXinerama()
+            if libfakeXinerama_so:
+                env["LD_PRELOAD"] = libfakeXinerama_so
+        return env
+
+    def find_fakeXinerama(self):
+        if sys.platform.startswith("linux"):
+            try:
+                libpath = find_lib_ldconfig("fakeXinerama")
+                if libpath:
+                    return libpath
+            except:
+                log.warn("Failed to launch ldconfig -p")
+        return find_lib("libfakeXinerama.so.1")
+
+
     def get_uuid(self):
         return get_uuid()
 
@@ -168,8 +192,7 @@ class X11ServerBase(GTKServerBase):
         except:
             pass
         try:
-            from xpra.scripts.server import find_fakeXinerama
-            fx = find_fakeXinerama()
+            fx = self.find_fakeXinerama()
         except:
             fx = None
         info["server.fakeXinerama"] = self.fake_xinerama and bool(fx)
@@ -228,7 +251,12 @@ class X11ServerBase(GTKServerBase):
 
     def get_cursor_data(self):
         #must be called from the UI thread!
-        cursor_data = trap.call(X11Keyboard.get_cursor_image)
+        try:
+            with xsync:
+                cursor_data = X11Keyboard.get_cursor_image()
+        except Exception as e:
+            cursorlog.error("Error getting cursor data: %s", e)
+            return None, []
         if cursor_data is None:
             cursorlog("get_cursor_data() failed to get cursor image")
             return None, []
@@ -282,26 +310,34 @@ class X11ServerBase(GTKServerBase):
         root_w, root_h = gtk.gdk.get_default_root_window().get_size()
         if desired_w==root_w and desired_h==root_h and not self.fake_xinerama:
             return    root_w,root_h    #unlikely: perfect match already!
-        #find the "physical" screen dimensions, so we can calculate the required dpi
-        #(and do this before changing the resolution)
-        wmm, hmm = 0, 0
-        client_w, client_h = 0, 0
-        sss = self._server_sources.values()
-        for ss in sss:
-            for s in ss.screen_sizes:
-                if len(s)>=10:
-                    #display_name, width, height, width_mm, height_mm, monitors, work_x, work_y, work_width, work_height
-                    client_w = max(client_w, s[1])
-                    client_h = max(client_h, s[2])
-                    wmm = max(wmm, s[3])
-                    hmm = max(hmm, s[4])
-        xdpi = self.default_dpi or self.dpi or 96
-        ydpi = self.default_dpi or self.dpi or 96
-        if wmm>0 and hmm>0 and client_w>0 and client_h>0:
-            #calculate "real" dpi using integer calculations:
-            xdpi = client_w * 254 / wmm / 10
-            ydpi = client_h * 254 / hmm / 10
-        log("calculated DPI: %s x %s (from w: %s / %s, h: %s / %s)", xdpi, ydpi, client_w, wmm, client_h, hmm)
+        #clients may supply "xdpi" and "ydpi" (v0.15 onwards), or just "dpi", or nothing...
+        xdpi = self.xdpi or self.dpi
+        ydpi = self.ydpi or self.dpi
+        log("set_screen_size(%s, %s) xdpi=%s, ydpi=%s", desired_w, desired_h, xdpi, ydpi)
+        if xdpi<=0 or ydpi<=0:
+            #use some sane defaults: either the command line option, or fallback to 96
+            #(96 is better than nothing, because we do want to set the dpi
+            # to avoid Xdummy setting a crazy dpi from the virtual screen dimensions)
+            xdpi = self.default_dpi or 96
+            ydpi = self.default_dpi or 96
+            #find the "physical" screen dimensions, so we can calculate the required dpi
+            #(and do this before changing the resolution)
+            wmm, hmm = 0, 0
+            client_w, client_h = 0, 0
+            sss = self._server_sources.values()
+            for ss in sss:
+                for s in ss.screen_sizes:
+                    if len(s)>=10:
+                        #display_name, width, height, width_mm, height_mm, monitors, work_x, work_y, work_width, work_height
+                        client_w = max(client_w, s[1])
+                        client_h = max(client_h, s[2])
+                        wmm = max(wmm, s[3])
+                        hmm = max(hmm, s[4])
+            if wmm>0 and hmm>0 and client_w>0 and client_h>0:
+                #calculate "real" dpi:
+                xdpi = int(client_w * 25.4 / wmm + 0.5)
+                ydpi = int(client_h * 25.4 / hmm + 0.5)
+                log("calculated DPI: %s x %s (from w: %s / %s, h: %s / %s)", xdpi, ydpi, client_w, wmm, client_h, hmm)
         self.set_dpi(xdpi, ydpi)
 
         #try to find the best screen size to resize to:
@@ -345,11 +381,13 @@ class X11ServerBase(GTKServerBase):
                     tw, th = temp[k]
                     log.info("temporarily switching to %sx%s as a Xinerama workaround", tw, th)
                     RandR.set_screen_size(tw, th)
-            log.debug("calling RandR.set_screen_size(%s, %s)", w, h)
-            trap.call_synced(RandR.set_screen_size, w, h)
-            log.debug("calling RandR.get_screen_size()")
+            log("calling RandR.set_screen_size(%s, %s)", w, h)
+            with xsync:
+                RandR.set_screen_size(w, h)
+            log("calling RandR.get_screen_size()")
             root_w, root_h = RandR.get_screen_size()
-            log.debug("RandR.get_screen_size()=%s,%s", root_w, root_h)
+            log("RandR.get_screen_size()=%s,%s", root_w, root_h)
+            log("RandR.get_vrefresh()=%s", RandR.get_vrefresh())
             if root_w!=w or root_h!=h:
                 log.error("odd, failed to set the new resolution, "
                           "tried to set it to %sx%s and ended up with %sx%s", w, h, root_w, root_h)
@@ -359,10 +397,7 @@ class X11ServerBase(GTKServerBase):
                     msg += " (best match for %sx%s)" % (desired_w, desired_h)
                 log.info(msg)
             def show_dpi():
-                sizes_mm = RandR.get_screen_sizes_mm()      #ie: [(1280, 1024)]
-                assert len(sizes_mm)>0
-                wmm = sum([x[0] for x in sizes_mm]) / len(sizes_mm)
-                hmm = sum([x[1] for x in sizes_mm]) / len(sizes_mm)
+                wmm, hmm = RandR.get_screen_size_mm()      #ie: (1280, 1024)
                 actual_xdpi = int(root_w * 25.4 / wmm + 0.5)
                 actual_ydpi = int(root_h * 25.4 / hmm + 0.5)
                 if actual_xdpi==xdpi and actual_ydpi==ydpi:
@@ -379,27 +414,17 @@ class X11ServerBase(GTKServerBase):
                         l(" to fix this issue, try the dpi switch, or use a patched Xdummy driver")
             #show dpi via idle_add so server has time to change the screen size (mm)
             self.idle_add(show_dpi)
-        except Exception, e:
+        except Exception as e:
             log.error("ouch, failed to set new resolution: %s", e, exc_info=True)
         return  root_w, root_h
 
     def save_fakexinerama_config(self):
         """ returns True if the fakexinerama config was modified """
-        xinerama_files = [
-                          #the new fakexinerama file:
-                          os.path.expanduser("~/.%s-fakexinerama" % os.environ.get("DISPLAY")),
-                          #compat file for "old" version found on github:
-                          os.path.expanduser("~/.fakexinerama"),
-                          ]
+        xinerama_files = self.get_xinerama_filenames()
         def delfile(msg):
             if msg:
                 log.warn(msg)
-            for f in xinerama_files:
-                if os.path.exists(f) and os.path.isfile(f):
-                    try:
-                        os.unlink(f)
-                    except Exception, e:
-                        log.warn("failed to delete fake xinerama file %s: %s", f, e)
+            self.cleanup_fakexinerama()
             oldconf = self.current_xinerama_config
             self.current_xinerama_config = None
             return oldconf is not None
@@ -426,41 +451,60 @@ class X11ServerBase(GTKServerBase):
         if len(monitors)>=10:
             return delfile("cannot save fake xinerama settings: too many monitors! (%s)" % len(monitors))
         #generate the file data:
-        data = ["# %s monitors:" % len(monitors),
+        from xpra import __version__
+        data = ["# file generated by xpra %s for display %s" % (__version__, os.environ.get("DISPLAY")),
+                "# %s monitors:" % len(monitors),
                 "%s" % len(monitors)]
         #the new config (numeric values only)
         config = [len(monitors)]
-        i = 0
-        for m in monitors:
+        for i, m in enumerate(monitors):
             if len(m)<7:
                 return delfile("cannot save fake xinerama settings: incomplete monitor data for monitor: %s" % m)
             plug_name, x, y, width, height, wmm, hmm = m[:7]
             data.append("# %s (%smm x %smm)" % (prettify_plug_name(plug_name, "monitor %s" % i), wmm, hmm))
             data.append("%s %s %s %s" % (x, y, width, height))
             config.append((x, y, width, height))
-            i += 1
+        if self.current_xinerama_config==config:
+            #we assume that no other process is going to overwrite the deprecated .fakexinerama
+            log("fake xinerama config unchanged")
+            return False
+        self.current_xinerama_config = config
         data.append("")
         contents = "\n".join(data)
         for filename in xinerama_files:
             try:
-                f = None
-                try:
-                    f = open(filename, 'wb')
+                with open(filename, 'wb') as f:
                     f.write(contents)
-                except Exception, e:
-                    log.warn("error writing fake xinerama file %s: %s", filename, e)
-                    pass
-            finally:
-                if f:
-                    f.close()
+            except Exception as e:
+                log.warn("error writing fake xinerama file %s: %s", filename, e)
         log("saved %s monitors to fake xinerama files: %s", len(monitors), xinerama_files)
-        oldconf = self.current_xinerama_config
-        self.current_xinerama_config = config
-        return oldconf!=config
+        return True
+
+    def get_xinerama_filenames(self):
+        return [
+                #the new fakexinerama file:
+                os.path.expanduser("~/.%s-fakexinerama" % os.environ.get("DISPLAY")),
+                #compat file for "old" version found on github:
+                os.path.expanduser("~/.fakexinerama"),
+               ]
+
+    def cleanup_fakexinerama(self):
+        for f in self.get_xinerama_filenames():
+            try:
+                if os.path.exists(f):
+                    log=("cleanup_fakexinerama() deleting fake xinerama file '%s'", f)
+                    os.unlink(f)
+            except Exception as e:
+                log.warn("cleanup_fakexinerama() error deleting fake xinerama file '%s': %s", f, e)
+
+    def cleanup(self, *args):
+        GTKServerBase.cleanup(self)
+        self.cleanup_fakexinerama()
 
 
     def _process_server_settings(self, proto, packet):
         settings = packet[1]
+        log("process_server_settings: %s", settings)
         self.update_server_settings(settings)
 
     def update_server_settings(self, settings, reset=False):
@@ -476,32 +520,35 @@ class X11ServerBase(GTKServerBase):
 
     def X11_ungrab(self):
         grablog("X11_ungrab")
-        def do_X11_ungrab():
+        with xsync:
             X11Core.UngrabKeyboard()
             X11Core.UngrabPointer()
-        trap.call_synced(do_X11_ungrab)
 
 
     def fake_key(self, keycode, press):
         keylog("fake_key(%s, %s)", keycode, press)
-        trap.call_synced(X11Keyboard.xtest_fake_key, keycode, press)
+        with xsync:
+            X11Keyboard.xtest_fake_key(keycode, press)
 
 
     def _move_pointer(self, wid, pos):
+        #(this is called within an xswallow context)
         mouselog("move_pointer(%s, %s)", wid, pos)
         x, y = pos
-        display = gtk.gdk.display_get_default()
-        display.warp_pointer(display.get_default_screen(), x, y)
+        n = gtk.gdk.display_get_default().get_default_screen().get_number()
+        X11Keyboard.xtest_fake_motion(n, x, y)
 
     def _process_mouse_common(self, proto, wid, pointer, modifiers):
         ss = self._server_sources.get(proto)
         if ss is None:
             return
         pos = gtk.gdk.get_default_root_window().get_pointer()[:2]
-        if pos==pointer:
-            return
-        trap.swallow_synced(self._move_pointer, wid, pointer)
+        if pos!=pointer:
+            with xswallow:
+                self._move_pointer(wid, pointer)
         ss.make_keymask_match(modifiers)
+        if wid==self.get_focus():
+            ss.user_event()
 
     def _process_button_action(self, proto, packet):
         ss = self._server_sources.get(proto)
@@ -510,8 +557,10 @@ class X11ServerBase(GTKServerBase):
         wid, button, pressed, pointer, modifiers = packet[1:6]
         self._process_mouse_common(proto, wid, pointer, modifiers)
         ss.user_event()
+        mouselog("xtest_fake_button(%s, %s) at %s", button, pressed, pointer)
         try:
-            trap.call_synced(X11Keyboard.xtest_fake_button, button, pressed)
+            with xsync:
+                X11Keyboard.xtest_fake_button(button, pressed)
         except XError:
             err = "Failed to pass on (un)press of mouse button %s" % button
             if button>=4:
