@@ -10,15 +10,16 @@ import re
 from xpra.x11.gtk_x11 import gdk_display_source
 assert gdk_display_source
 
-from xpra.util import std
+from xpra.util import std, nonl
 from xpra.keyboard.layouts import parse_xkbmap_query
-from xpra.gtk_common.error import trap
+from xpra.gtk_common.error import xsync
 from xpra.x11.bindings.keyboard_bindings import X11KeyboardBindings #@UnresolvedImport
 X11Keyboard = X11KeyboardBindings()
 
 from xpra.log import Logger
 log = Logger("x11", "keyboard")
-vlog = Logger("x11", "keyboard", "verbose")
+
+DEBUG_KEYSYMS = [x for x in os.environ.get("XPRA_DEBUG_KEYSYMS", "").split(",") if len(x)>0]
 
 #keys we choose not to map if the free space in the keymap is too limited
 #this list was generated using:
@@ -41,9 +42,9 @@ def exec_keymap_command(args, stdin=None):
         from xpra.scripts.exec_util import safe_exec
         returncode, _, _ = safe_exec(args, stdin)
         def logstdin():
-            if not stdin or len(stdin)<32:
-                return  stdin
-            return stdin[:30].replace("\n", "\\n")+".."
+            if not stdin or len(stdin)<500:
+                return  nonl(stdin)
+            return nonl(stdin[:500])+".."
         if returncode==0:
             if not stdin:
                 log("%s", args)
@@ -52,7 +53,7 @@ def exec_keymap_command(args, stdin=None):
         else:
             log.error("%s with stdin=%s, failed with exit code %s", args, logstdin(), returncode)
         return returncode
-    except Exception, e:
+    except Exception as e:
         log.error("error calling '%s': %s" % (str(args), e))
         return -1
 
@@ -123,7 +124,7 @@ def do_set_keymap(xkbmap_layout, xkbmap_variant,
                     log.info("guessing keyboard layout='%s'" % layout)
                     exec_keymap_command(["setxkbmap", layout])
                     break
-        except Exception, e:
+        except Exception as e:
             log.info("error setting keymap: %s" % e)
     else:
         layout = xkbmap_layout or "us"
@@ -151,7 +152,8 @@ def do_set_keymap(xkbmap_layout, xkbmap_variant,
 
 def apply_xmodmap(instructions):
     try:
-        unset = trap.call_synced(X11Keyboard.set_xmodmap, instructions)
+        with xsync:
+            unset = X11Keyboard.set_xmodmap(instructions)
     except:
         log.error("apply_xmodmap", exc_info=True)
         unset = instructions
@@ -186,35 +188,47 @@ def set_all_keycodes(xkbmap_x11_keycodes, xkbmap_keycodes, preserve_server_keyco
                 log.error("ERROR: keysym %s is mapped to both %s and %s !", keysym, modifier, existing_mod)
             else:
                 keysym_to_modifier[keysym] = modifier
+                if keysym in DEBUG_KEYSYMS:
+                    log.info("set_all_keycodes() keysym_to_modifier[%s]=%s", keysym, modifier)
     log("keysym_to_modifier=%s", keysym_to_modifier)
 
     def modifiers_for(entries):
         """ entries can only point to a single modifier - verify """
         modifiers = set()
+        l = log
         for keysym, _ in entries:
             modifier = keysym_to_modifier.get(keysym)
             if modifier:
                 modifiers.add(modifier)
+            if keysym in DEBUG_KEYSYMS:
+                l = log.info
+        l("modifiers_for(%s)=%s", entries, modifiers)
         return modifiers
 
     def filter_mappings(mappings, drop_extra_keys=False):
         filtered = {}
+        invalid_keysyms = set()
         for keycode, entries in mappings.items():
             mods = modifiers_for(entries)
             if len(mods)>1:
                 log.warn("keymapping removed invalid keycode entry %s pointing to more than one modifier (%s): %s", keycode, mods, entries)
                 continue
             #now remove entries for keysyms we don't have:
-            f_entries = set([(keysym, index) for keysym, index in entries if X11Keyboard.parse_keysym(keysym) is not None])
+            f_entries = set([(keysym, index) for keysym, index in entries if keysym and X11Keyboard.parse_keysym(keysym) is not None])
+            for keysym, _ in entries:
+                if keysym and X11Keyboard.parse_keysym(keysym) is None:
+                    invalid_keysyms.add(keysym)
             if len(f_entries)==0:
-                log.warn("keymapping removed invalid keycode entry %s pointing to only unknown keysyms: %s", keycode, entries)
+                log("keymapping removed invalid keycode entry %s pointing to only unknown keysyms: %s", keycode, entries)
                 continue
             if drop_extra_keys:
                 noopt = [keysym for keysym, index in entries if (X11Keyboard.parse_keysym(keysym) is not None and keysym not in OPTIONAL_KEYS)]
                 if len(noopt)==0:
                     log("keymapping removed keycode entry %s pointing to optional keys: %s", keycode, entries)
-                continue
+                    continue
             filtered[keycode] = f_entries
+        if invalid_keysyms:
+            log.warn("the following keysyms are invalid and have not been mapped: %s", ", ".join((str(x) for x in invalid_keysyms)))
         return filtered
 
     #get the list of keycodes (either from x11 keycodes or gtk keycodes):
@@ -254,17 +268,27 @@ def dump_dict(d):
 
 def group_by_keycode(entries):
     keycodes = {}
+    log_keycodes = []
     for keysym, keycode, index in entries:
         keycodes.setdefault(keycode, set()).add((keysym, index))
+        if keysym in DEBUG_KEYSYMS:
+            log_keycodes.append(keycode)
+    if log_keycodes:
+        log.info("group_by_keycode: %s", dict((keycode, keycodes.get(keycode)) for keycode in log_keycodes))
     return keycodes
 
 def indexed_mappings(raw_mappings):
     indexed = {}
     for keycode, keysyms in raw_mappings.items():
         pairs = set()
+        l = log
         for i in range(0, len(keysyms)):
-            pairs.add((keysyms[i], i))
+            keysym = keysyms[i]
+            if keysym in DEBUG_KEYSYMS:
+                l = log.info
+            pairs.add((keysym, i))
         indexed[keycode] = pairs
+        l("indexed_mappings: %s=%s", keycode, pairs)
     return indexed
 
 
@@ -298,6 +322,8 @@ def x11_keycodes_to_list(x11_mappings):
             for keysym in keysyms:
                 if keysym:
                     entries.append([keysym, int(keycode), index])
+                    if keysym in DEBUG_KEYSYMS:
+                        log.info("x11_keycodes_to_list: (%s, %s) : %s", keycode, keysyms, (keysym, int(keycode), index))
                 index += 1
     return entries
 
@@ -317,7 +343,7 @@ def translate_keycodes(kcmin, kcmax, keycodes, preserve_keycode_entries={}, keys
     """
     log("translate_keycodes(%s, %s, %s, %s, %s, %s)", kcmin, kcmax, keycodes, preserve_keycode_entries, keysym_to_modifier, try_harder)
     #list of free keycodes we can use:
-    free_keycodes = [i for i in range(kcmin, kcmax) if i not in preserve_keycode_entries]
+    free_keycodes = [i for i in range(kcmin, kcmax+1) if i not in preserve_keycode_entries]
     keycode_trans = {}              #translation map from client keycode to our server keycode
     server_keycodes = {}            #the new keycode definitions
     missing_keycodes = []           #the groups of entries we failed to map due to lack of free keycodes
@@ -327,31 +353,37 @@ def translate_keycodes(kcmin, kcmax, keycodes, preserve_keycode_entries={}, keys
     for keycode, entries in preserve_keycode_entries.items():
         for keysym, _ in entries:
             preserve_keysyms_map.setdefault(keysym, set()).add(keycode)
+    for k in DEBUG_KEYSYMS:
+        log.info("preserve_keysyms_map[%s]=%s", k, preserve_keysyms_map.get(k))
 
-    def do_assign(keycode, server_keycode, entries):
+    def do_assign(keycode, server_keycode, entries, override_server_keycode=False):
         """ may change the keycode if needed
             in which case we update the entries and populate 'keycode_trans'
         """
-        if server_keycode in server_keycodes:
-            log("assign: keycode %s already in use: %s", server_keycode, server_keycodes.get(server_keycode))
+        l = log
+        for name, _ in entries:
+            if name in DEBUG_KEYSYMS:
+                l = log.info
+        if (server_keycode in server_keycodes) and not override_server_keycode:
+            l("assign: server keycode %s already in use: %s", server_keycode, server_keycodes.get(server_keycode))
             server_keycode = -1
         elif server_keycode>0 and (server_keycode<kcmin or server_keycode>kcmax):
-            log("assign: keycode %s out of range (%s to %s)", server_keycode, kcmin, kcmax)
+            l("assign: keycode %s out of range (%s to %s)", server_keycode, kcmin, kcmax)
             server_keycode = -1
         if server_keycode<=0:
             if len(free_keycodes)>0:
                 server_keycode = free_keycodes[0]
-                log("set_keycodes key %s using free keycode=%s", entries, server_keycode)
+                l("set_keycodes key %s using free keycode=%s", entries, server_keycode)
             else:
-                msg = "set_keycodes: no free keycodes!, cannot translate %s: %s", server_keycode, entries
+                msg = "set_keycodes: no free keycodes!, cannot translate %s: %s", server_keycode, tuple(entries)
                 if try_harder:
                     log.error(*msg)
                 else:
-                    log(*msg)
+                    l(*msg)
                 missing_keycodes.append(entries)
                 server_keycode = -1
         if server_keycode>0:
-            vlog("set_keycodes key %s (%s) mapped to keycode=%s", keycode, entries, server_keycode)
+            l("set_keycodes key %s (%s) mapped to keycode=%s", keycode, tuple(entries), server_keycode)
             #can't use it any more!
             if server_keycode in free_keycodes:
                 free_keycodes.remove(server_keycode)
@@ -359,19 +391,29 @@ def translate_keycodes(kcmin, kcmax, keycodes, preserve_keycode_entries={}, keys
             for name, _ in entries:
                 if keycode>=0 and server_keycode!=keycode:
                     keycode_trans[(keycode, name)] = server_keycode
+                    l("keycode_trans[(%s, %s)]=%s", keycode, name, server_keycode)
                 keycode_trans[name] = server_keycode
+                l("keycode_trans[%s]=%s", name, server_keycode)
             server_keycodes[server_keycode] = entries
         return server_keycode
 
     def assign(client_keycode, entries):
         if len(entries)==0:
             return 0
-        if len(preserve_keycode_entries)==0:
-            return do_assign(client_keycode, client_keycode, entries)
         #all the keysyms for this keycode:
         keysyms = set([keysym for keysym, _ in entries])
         if len(keysyms)==0:
             return 0
+        if len(keysyms)==1 and list(keysyms)[0]=='0xffffff':
+            log("skipped invalid keysym: %s / %s", client_keycode, entries)
+            return 0
+        l = log
+        if [k for k in keysyms if k in DEBUG_KEYSYMS]:
+            l = log.info
+        l("assign(%s, %s)", client_keycode, entries)
+
+        if len(preserve_keycode_entries)==0:
+            return do_assign(client_keycode, client_keycode, entries)
         if len(keysyms)==1:
             #only one keysym, replace with single entry
             entries = set([(list(keysyms)[0], 0)])
@@ -381,32 +423,34 @@ def translate_keycodes(kcmin, kcmax, keycodes, preserve_keycode_entries={}, keys
         for keysym in list(keysyms):
             keycodes = preserve_keysyms_map.get(keysym, [])
             for keycode in keycodes:
-                preserve_keycode_matches[keycode] = preserve_keycode_entries.get(keycode)
+                v = preserve_keycode_entries.get(keycode)
+                preserve_keycode_matches[keycode] = v
+                l("preserve_keycode_matches[%s]=%s", keycode, v)
 
         if len(preserve_keycode_matches)==0:
-            log("no preserve matches for %s", entries)
+            l("no preserve matches for %s", tuple(entries))
             return do_assign(client_keycode, -1, entries)         #nothing to preserve
 
-        log("preserve matches for %s : %s", entries, preserve_keycode_matches)
+        l("preserve matches for %s : %s", entries, preserve_keycode_matches)
         #direct superset:
         for p_keycode, p_entries in preserve_keycode_matches.items():
             if entries.issubset(p_entries):
-                vlog("found direct preserve superset for %s : %s -> %s : %s", client_keycode, entries, p_keycode, p_entries)
-                return do_assign(client_keycode, p_keycode, p_entries)
+                l("found direct preserve superset for client keycode %s %s -> server keycode %s %s", client_keycode, tuple(entries), p_keycode, tuple(p_entries))
+                return do_assign(client_keycode, p_keycode, p_entries, override_server_keycode=True)
             if p_entries.issubset(entries):
-                vlog("found direct superset of preserve for %s : %s -> %s : %s", client_keycode, entries, p_keycode, p_entries)
-                return do_assign(client_keycode, p_keycode, entries)
+                l("found direct superset of preserve for client keycode %s %s -> server keycode %s %s", client_keycode, tuple(entries), p_keycode, tuple(p_entries))
+                return do_assign(client_keycode, p_keycode, entries, override_server_keycode=True)
 
         #ignoring indexes, but requiring at least as many keysyms:
         for p_keycode, p_entries in preserve_keycode_matches.items():
             p_keysyms = set([keysym for keysym,_ in p_entries])
             if keysyms.issubset(p_keysyms):
                 if len(p_entries)>len(entries):
-                    vlog("found keysym preserve superset with more keys for %s : %s", entries, p_entries)
-                    return do_assign(client_keycode, p_keycode, p_entries)
+                    l("found keysym preserve superset with more keys for %s : %s", tuple(entries), tuple(p_entries))
+                    return do_assign(client_keycode, p_keycode, p_entries, override_server_keycode=True)
             if p_keysyms.issubset(keysyms):
-                vlog("found keysym superset of preserve with more keys for %s : %s", entries, p_entries)
-                return do_assign(client_keycode, p_keycode, entries)
+                l("found keysym superset of preserve with more keys for %s : %s", tuple(entries), tuple(p_entries))
+                return do_assign(client_keycode, p_keycode, entries, override_server_keycode=True)
 
         if try_harder:
             #try to match the main key only:
@@ -415,10 +459,10 @@ def translate_keycodes(kcmin, kcmax, keycodes, preserve_keycode_entries={}, keys
                 for p_keycode, p_entries in preserve_keycode_matches.items():
                     p_keysyms = set([keysym for keysym,_ in p_entries])
                     if main_key.issubset(p_entries):
-                        vlog("found main key superset for %s : %s", main_key, p_entries)
-                        return do_assign(client_keycode, p_keycode, p_entries)
+                        l("found main key superset for %s : %s", main_key, tuple(p_entries))
+                        return do_assign(client_keycode, p_keycode, p_entries, override_server_keycode=True)
 
-        log("no matches for %s", entries)
+        l("no matches for %s", tuple(entries))
         return do_assign(client_keycode, -1, entries)
 
     #now try to assign each keycode:
@@ -442,10 +486,13 @@ def translate_keycodes(kcmin, kcmax, keycodes, preserve_keycode_entries={}, keys
     #defined keysyms for modifiers if some are missing:
     for keysym, modifier in keysym_to_modifier.items():
         if keysym not in all_keysyms:
-            log("found missing keysym %s for modifier %s, will add it", keysym, modifier)
+            l = log
+            if keysym in DEBUG_KEYSYMS:
+                l = log.info
+            l("found missing keysym %s for modifier %s, will add it", keysym, modifier)
             new_keycode = set([(keysym, 0)])
             server_keycode = assign(-1, new_keycode)
-            log("assigned keycode %s for key '%s' of modifier '%s'", server_keycode, keysym, modifier)
+            l("assigned keycode %s for key '%s' of modifier '%s'", server_keycode, keysym, modifier)
 
     log("translated keycodes=%s", keycode_trans)
     log("%s free keycodes=%s", len(free_keycodes), free_keycodes)
@@ -484,10 +531,15 @@ def keymap_to_xmodmap(trans_keycodes):
                     log.warn("we already have a keysym for %s at index %s: %s, entries=%s", server_keycode, index, keysyms[index], entries)
                 else:
                     keysyms[index] = keysym
+                    if keysym in DEBUG_KEYSYMS:
+                        log.info("keymap_to_xmodmap: keysyms[%s]=%s", index, keysym)
         #remove empty keysyms:
         while len(keysyms)>0 and keysyms[0] is None:
             keysyms = keysyms[1:]
-        log("%s: %s -> %s", server_keycode, names, keysyms)
+        l = log
+        if [k for k in keysyms if k in DEBUG_KEYSYMS]:
+            l = log.info
+        l("%s: %s -> %s", server_keycode, names, keysyms)
         instructions.append(("keycode", server_keycode, keysyms))
 
     if len(missing_keysyms)>0:
