@@ -9,7 +9,7 @@ import gobject
 from xpra.gtk_common.gobject_util import n_arg_signal
 from xpra.x11.gtk_x11.gdk_bindings import add_event_receiver, remove_event_receiver, cleanup_all_event_receivers    #@UnresolvedImport
 from xpra.x11.gtk_x11.gdk_bindings import init_x11_filter, cleanup_x11_filter             #@UnresolvedImport
-from xpra.x11.gtk_x11.error import trap
+from xpra.gtk_common.error import xsync
 
 from xpra.log import Logger
 log = Logger("x11", "util")
@@ -37,12 +37,14 @@ class XRootPropWatcher(gobject.GObject):
         if self._own_x11_filter:
             #only remove the x11 filter if we initialized it (ie: when running in client)
             try:
-                trap.call_synced(cleanup_x11_filter)
-            except Exception, e:
+                with xsync:
+                    cleanup_x11_filter()
+            except Exception as e:
                 log.error("failed to remove x11 event filter: %s", e)
             try:
-                trap.call_synced(cleanup_all_event_receivers)
-            except Exception, e:
+                with xsync:
+                    cleanup_all_event_receivers()
+            except Exception as e:
                 log.error("failed to remove event receivers: %s", e)
 
     def do_xpra_property_notify_event(self, event):
