@@ -103,35 +103,81 @@ def set_settings(disp, d):
     for setting in settings:
         setting_type, prop_name, value, last_change_serial = setting
         prop_name = str(prop_name)
-        log("set_settings(..) processing property %s of type %s", prop_name, XSettingsNames.get(setting_type, "INVALID!"))
-        x = struct.pack("=BBH", setting_type, 0, len(prop_name))
-        x += struct.pack("="+"s"*len(prop_name), *list(prop_name))
-        pad_len = ((len(prop_name) + 0x3) & ~0x3) - len(prop_name)
-        x += '\0'*pad_len
-        x += struct.pack("=I", last_change_serial)
-        if setting_type==XSettingsTypeInteger:
-            assert type(value) in (int, long), "invalid value type (int or long wanted): %s" % type(value)
-            x += struct.pack("=I", int(value))
-        elif setting_type==XSettingsTypeString:
-            if type(value)==unicode:
-                value = str(value)
-            else:
-                assert type(value)==str, "invalid value type (str wanted): %s" % type(value)
-            x += struct.pack("=I", len(value))
-            x += struct.pack("="+"s"*len(value), *list(value))
-            pad_len = ((len(value) + 0x3) & ~0x3) - len(value)
+        try:
+            log("set_settings(..) processing property %s of type %s", prop_name, XSettingsNames.get(setting_type, "INVALID!"))
+            x = struct.pack("=BBH", setting_type, 0, len(prop_name))
+            x += struct.pack("="+"s"*len(prop_name), *list(prop_name))
+            pad_len = ((len(prop_name) + 0x3) & ~0x3) - len(prop_name)
             x += '\0'*pad_len
-        elif setting_type==XSettingsTypeColor:
-            red, blue, green, alpha = value
-            x = struct.pack("=HHHH", red, blue, green, alpha)
-        else:
-            log.error("invalid xsetting type: %s, skipped %s", setting_type, prop_name)
-            continue
-        log("set_settings(..) %s -> %s", setting, list(x))
-        all_bin_settings.append(x)
+            x += struct.pack("=I", last_change_serial)
+            if setting_type==XSettingsTypeInteger:
+                assert type(value) in (int, long), "invalid value type (int or long wanted): %s" % type(value)
+                x += struct.pack("=I", int(value))
+            elif setting_type==XSettingsTypeString:
+                if type(value)==unicode:
+                    value = str(value)
+                else:
+                    assert type(value)==str, "invalid value type (str wanted): %s" % type(value)
+                x += struct.pack("=I", len(value))
+                x += struct.pack("="+"s"*len(value), *list(value))
+                pad_len = ((len(value) + 0x3) & ~0x3) - len(value)
+                x += '\0'*pad_len
+            elif setting_type==XSettingsTypeColor:
+                red, blue, green, alpha = value
+                x = struct.pack("=HHHH", red, blue, green, alpha)
+            else:
+                log.error("invalid xsetting type: %s, skipped %s", setting_type, prop_name)
+                continue
+            log("set_settings(..) %s -> %s", setting, list(x))
+            all_bin_settings.append(x)
+        except Exception as e:
+            log.error("Error processing XSettings property %s:", prop_name)
+            log.error(" type=%s, value=%s", XSettingsNames.get(setting_type, "INVALID!"), value)
+            log.error(" %s", e)
     #header
     v = struct.pack("=BBBBII", get_local_byteorder(), 0, 0, 0, serial, len(all_bin_settings))
     v += "".join(all_bin_settings)  #values
     v += '\0'                       #null terminated
     log("set_settings(%s)=%s", d, list(v))
     return  v
+
+
+def main():
+    from xpra.platform.gui import init as gui_init
+    from xpra.platform import init as platform_init,clean
+    try:
+        platform_init("XSettings")
+        gui_init()
+        verbose = "-v" in sys.argv or "--verbose" in sys.argv
+        if verbose:
+            from xpra.log import get_all_loggers
+            for x in get_all_loggers():
+                x.enable_debug()
+
+        #naughty, but how else can I hook this up?
+        import os
+        if os.name!="posix":
+            print("xsettings require a posix OS")
+            return 1
+
+        from xpra.x11.bindings import posix_display_source  #@UnresolvedImport - takes care of hooking up the display
+        assert posix_display_source
+        from xpra.x11.bindings.window_bindings import X11WindowBindings #@UnresolvedImport
+        window_bindings = X11WindowBindings()
+        selection = "_XSETTINGS_S0"
+        owner = window_bindings.XGetSelectionOwner(selection)
+        print("owner(%s)=%#x" % (selection, owner))
+        XSETTINGS = "_XSETTINGS_SETTINGS"
+        data = window_bindings.XGetWindowProperty(owner, XSETTINGS, XSETTINGS)
+        serial, settings = get_settings(None, data)
+        print("serial=%s" % serial)
+        print("%s settings:" % len(settings))
+        for s in settings:
+            print(s)
+        return 0
+    finally:
+        clean()
+
+
+if __name__ == "__main__":
+    sys.exit(main())
